@@ -1,5 +1,6 @@
 /* This file is part of Strawberry.
    Copyright 2011, David Sansome <me@davidsansome.com>
+   Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
 
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
@@ -63,7 +64,7 @@ void _MessageHandlerBase::SetDevice(QIODevice *device) {
 
 void _MessageHandlerBase::DeviceReadyRead() {
 
-  while (device_->bytesAvailable()) {
+  while (device_->bytesAvailable() > 0) {
     if (!reading_protobuf_) {
       // Read the length of the next message
       QDataStream s(device_);
@@ -98,7 +99,7 @@ void _MessageHandlerBase::WriteMessage(const QByteArray &data) {
 
   QDataStream s(device_);
   s << quint32(data.length());
-  s.writeRawData(data.data(), data.length());
+  s.writeRawData(data.data(), static_cast<int>(data.length()));
 
   // Sorry.
   if (flush_abstract_socket_) {
