@@ -1,5 +1,6 @@
 /* This file is part of Strawberry.
    Copyright 2011, David Sansome <me@davidsansome.com>
+   Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
 
    Strawberry is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +19,9 @@
 #ifndef WORKERPOOL_H
 #define WORKERPOOL_H
 
+#include "config.h"
+
+#include <cstdio>
 #include <cstddef>
 
 #include <QtGlobal>
@@ -55,6 +59,8 @@ class _WorkerPoolBase : public QObject {
  protected slots:
   virtual void DoStart() {}
   virtual void NewConnection() {}
+  virtual void ProcessReadyReadStandardOutput() {}
+  virtual void ProcessReadyReadStandardError() {}
   virtual void ProcessError(QProcess::ProcessError) {}
   virtual void SendQueuedMessages() {}
 };
@@ -97,6 +103,8 @@ class WorkerPool : public _WorkerPoolBase {
   // These are all reimplemented slots, they are called on the WorkerPool's thread.
   void DoStart() override;
   void NewConnection() override;
+  void ProcessReadyReadStandardOutput() override;
+  void ProcessReadyReadStandardError() override;
   void ProcessError(QProcess::ProcessError error) override;
   void SendQueuedMessages() override;
 
@@ -163,8 +171,9 @@ WorkerPool<HandlerType>::WorkerPool(QObject *parent)
   worker_count_ = qBound(1, QThread::idealThreadCount() / 2, 4);
   local_server_name_ = qApp->applicationName().toLower();
 
-  if (local_server_name_.isEmpty())
+  if (local_server_name_.isEmpty()) {
     local_server_name_ = "workerpool";
+  }
 
 }
 
@@ -174,6 +183,8 @@ WorkerPool<HandlerType>::~WorkerPool() {
   for (const Worker &worker : workers_) {
     if (worker.local_socket_ && worker.process_) {
       QObject::disconnect(worker.process_, &QProcess::errorOccurred, this, &WorkerPool::ProcessError);
+      QObject::disconnect(worker.process_, &QProcess::readyReadStandardOutput, this, &WorkerPool::ProcessReadyReadStandardOutput);
+      QObject::disconnect(worker.process_, &QProcess::readyReadStandardError, this, &WorkerPool::ProcessReadyReadStandardError);
 
       // The worker is connected.  Close his socket and wait for him to exit.
       qLog(Debug) << "Closing worker socket";
@@ -217,7 +228,7 @@ void WorkerPool<HandlerType>::SetExecutableName(const QString &executable_name)
 
 template <typename HandlerType>
 void WorkerPool<HandlerType>::Start() {
-  metaObject()->invokeMethod(this, "DoStart");
+  QMetaObject::invokeMethod(this, "DoStart");
 }
 
 template <typename HandlerType>
@@ -231,9 +242,9 @@ void WorkerPool<HandlerType>::DoStart() {
   executable_path_ = executable_name_;
 
   QStringList search_path;
-  search_path << qApp->applicationDirPath();
+  search_path << QCoreApplication::applicationDirPath();
 #if defined(Q_OS_MACOS) && defined(USE_BUNDLE)
-  search_path << qApp->applicationDirPath() + "/" + USE_BUNDLE_DIR;
+  search_path << QCoreApplication::applicationDirPath() + "/" + USE_BUNDLE_DIR;
 #endif
 
   for (const QString &path_prefix : search_path) {
@@ -274,13 +285,15 @@ void WorkerPool<HandlerType>::StartOneWorker(Worker *worker) {
 
   QObject::connect(worker->local_server_, &QLocalServer::newConnection, this, &WorkerPool::NewConnection);
   QObject::connect(worker->process_, &QProcess::errorOccurred, this, &WorkerPool::ProcessError);
+  QObject::connect(worker->process_, &QProcess::readyReadStandardOutput, this, &WorkerPool::ProcessReadyReadStandardOutput);
+  QObject::connect(worker->process_, &QProcess::readyReadStandardError, this, &WorkerPool::ProcessReadyReadStandardError);
 
   // Create a server, find an unused name and start listening
   forever {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
-    const int unique_number = QRandomGenerator::global()->bounded((int)(quint64(this) & 0xFFFFFFFF));
+    const quint32 unique_number = QRandomGenerator::global()->bounded(static_cast<quint32>(quint64(this) & 0xFFFFFFFF));
 #else
-    const int unique_number = qrand() ^ ((int)(quint64(this) & 0xFFFFFFFF));
+    const quint32 unique_number = qrand() ^ (static_cast<quint32>(quint64(this) & 0xFFFFFFFF));
 #endif
     const QString name = QString("%1_%2").arg(local_server_name_).arg(unique_number);
 
@@ -291,12 +304,12 @@ void WorkerPool<HandlerType>::StartOneWorker(Worker *worker) {
 
   qLog(Debug) << "Starting worker" << worker << executable_path_ << worker->local_server_->fullServerName();
 
-  // Start the process
-#if defined(Q_OS_WIN32) && defined(QT_NO_DEBUG_OUTPUT) && !defined(ENABLE_WIN32_CONSOLE)
+#ifdef Q_OS_WIN32
   worker->process_->setProcessChannelMode(QProcess::SeparateChannels);
 #else
   worker->process_->setProcessChannelMode(QProcess::ForwardedChannels);
 #endif
+
   worker->process_->start(executable_path_, QStringList() << worker->local_server_->fullServerName());
 
 }
@@ -326,6 +339,7 @@ void WorkerPool<HandlerType>::NewConnection() {
   worker->handler_ = new HandlerType(worker->local_socket_, this);
 
   SendQueuedMessages();
+
 }
 
 template <typename HandlerType>
@@ -356,6 +370,32 @@ void WorkerPool<HandlerType>::ProcessError(QProcess::ProcessError error) {
 
 }
 
+template <typename HandlerType>
+void WorkerPool<HandlerType>::ProcessReadyReadStandardOutput() {
+
+  Q_ASSERT(QThread::currentThread() == thread());
+
+  QProcess *process = qobject_cast<QProcess*>(sender());
+  QByteArray data = process->readAllStandardOutput();
+
+  fprintf(stdout, "%s", data.data());
+  fflush(stdout);
+
+}
+
+template <typename HandlerType>
+void WorkerPool<HandlerType>::ProcessReadyReadStandardError() {
+
+  Q_ASSERT(QThread::currentThread() == thread());
+
+  QProcess *process = qobject_cast<QProcess*>(sender());
+  QByteArray data = process->readAllStandardError();
+
+  fprintf(stderr, "%s", data.data());
+  fflush(stderr);
+
+}
+
 template <typename HandlerType>
 typename WorkerPool<HandlerType>::ReplyType*
 WorkerPool<HandlerType>::NewReply(MessageType *message) {
@@ -380,7 +420,7 @@ WorkerPool<HandlerType>::SendMessageWithReply(MessageType *message) {
   }
 
   // Wake up the main thread
-  metaObject()->invokeMethod(this, "SendQueuedMessages", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(this, "SendQueuedMessages", Qt::QueuedConnection);
 
   return reply;
 
