@@ -27,17 +27,19 @@
 
 #include <cmath>
 
-#include <QObject>
+#include <QtGlobal>
 #include <QWidget>
 #include <QPixmap>
 #include <QPainter>
 #include <QColor>
 #include <QBrush>
 #include <QPen>
+#include <QPoint>
+#include <QSize>
 #include <QTimerEvent>
 
-#include "core/arraysize.h"
-#include "core/logging.h"
+#include "fht.h"
+#include "analyzerbase.h"
 
 using Analyzer::Scope;
 
@@ -48,13 +50,13 @@ const int Rainbow::RainbowAnalyzer::kRainbowHeight[] = {21, 16};
 const int Rainbow::RainbowAnalyzer::kRainbowOverlap[] = {13, 15};
 const int Rainbow::RainbowAnalyzer::kSleepingHeight[] = {24, 33};
 
-const char* Rainbow::NyanCatAnalyzer::kName = "Nyanalyzer Cat";
-const char* Rainbow::RainbowDashAnalyzer::kName = "Rainbow Dash";
-const float Rainbow::RainbowAnalyzer::kPixelScale = 0.02f;
+const char *Rainbow::NyanCatAnalyzer::kName = "Nyanalyzer Cat";
+const char *Rainbow::RainbowDashAnalyzer::kName = "Rainbow Dash";
+const float Rainbow::RainbowAnalyzer::kPixelScale = 0.02F;
 
 Rainbow::RainbowAnalyzer::RainbowType Rainbow::RainbowAnalyzer::rainbowtype;
 
-Rainbow::RainbowAnalyzer::RainbowAnalyzer(const RainbowType& rbtype, QWidget* parent)
+Rainbow::RainbowAnalyzer::RainbowAnalyzer(const RainbowType rbtype, QWidget *parent)
     : Analyzer::Base(parent, 9),
       timer_id_(startTimer(kFrameIntervalMs)),
       frame_(0),
@@ -62,8 +64,7 @@ Rainbow::RainbowAnalyzer::RainbowAnalyzer(const RainbowType& rbtype, QWidget* pa
       available_rainbow_width_(0),
       px_per_frame_(0),
       x_offset_(0),
-      background_brush_(QColor(0x0f, 0x43, 0x73))
-      {
+      background_brush_(QColor(0x0f, 0x43, 0x73)) {
 
   rainbowtype = rbtype;
   cat_dash_[0] = QPixmap(":/pictures/nyancat.png");
@@ -71,18 +72,18 @@ Rainbow::RainbowAnalyzer::RainbowAnalyzer(const RainbowType& rbtype, QWidget* pa
   memset(history_, 0, sizeof(history_));
 
   for (int i = 0; i < kRainbowBands; ++i) {
-    colors_[i] = QPen(QColor::fromHsv(i * 255 / kRainbowBands, 255, 255), kRainbowHeight[rainbowtype] / kRainbowBands, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin);
+    colors_[i] = QPen(QColor::fromHsv(i * 255 / kRainbowBands, 255, 255), kRainbowHeight[rainbowtype] / kRainbowBands, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin);  // NOLINT(bugprone-integer-division)
 
     // pow constants computed so that
     // | band_scale(0) | ~= .5 and | band_scale(5) | ~= 32
-    band_scale_[i] = -std::cos(M_PI * i / (kRainbowBands - 1)) * 0.5 * std::pow(2.3, i);
+    band_scale_[i] = -static_cast<float>(std::cos(M_PI * i / (kRainbowBands - 1))) * 0.5F * static_cast<float>(std::pow(2.3, i));
   }
 
 }
 
-void Rainbow::RainbowAnalyzer::transform(Scope& s) { fht_->spectrum(s.data()); }
+void Rainbow::RainbowAnalyzer::transform(Scope &s) { fht_->spectrum(s.data()); }
 
-void Rainbow::RainbowAnalyzer::timerEvent(QTimerEvent* e) {
+void Rainbow::RainbowAnalyzer::timerEvent(QTimerEvent *e) {
 
   if (e->timerId() == timer_id_) {
     frame_ = (frame_ + 1) % kFrameCount[rainbowtype];
@@ -93,7 +94,7 @@ void Rainbow::RainbowAnalyzer::timerEvent(QTimerEvent* e) {
 
 }
 
-void Rainbow::RainbowAnalyzer::resizeEvent(QResizeEvent* e) {
+void Rainbow::RainbowAnalyzer::resizeEvent(QResizeEvent *e) {
 
   Q_UNUSED(e);
 
@@ -102,20 +103,20 @@ void Rainbow::RainbowAnalyzer::resizeEvent(QResizeEvent* e) {
   buffer_[1] = QPixmap();
 
   available_rainbow_width_ = width() - kWidth[rainbowtype] + kRainbowOverlap[rainbowtype];
-  px_per_frame_ = static_cast<float>(available_rainbow_width_) / (kHistorySize - 1) + 1;
+  px_per_frame_ = available_rainbow_width_ / (kHistorySize - 1) + 1;
   x_offset_ = px_per_frame_ * (kHistorySize - 1) - available_rainbow_width_;
 
 }
 
-void Rainbow::RainbowAnalyzer::analyze(QPainter& p, const Analyzer::Scope& s, bool new_frame) {
+void Rainbow::RainbowAnalyzer::analyze(QPainter &p, const Analyzer::Scope &s, bool new_frame) {
 
   // Discard the second half of the transform
-  const int scope_size = s.size() / 2;
+  const int scope_size = static_cast<int>(s.size() / 2);
 
   if ((new_frame && is_playing_) || (buffer_[0].isNull() && buffer_[1].isNull())) {
     // Transform the music into rainbows!
     for (int band = 0; band < kRainbowBands; ++band) {
-      float* band_start = history_ + band * kHistorySize;
+      float *band_start = history_ + band * kHistorySize;
 
       // Move the history of each band across by 1 frame.
       memmove(band_start, band_start + 1, (kHistorySize - 1) * sizeof(float));
@@ -137,13 +138,13 @@ void Rainbow::RainbowAnalyzer::analyze(QPainter& p, const Analyzer::Scope& s, bo
 
     // Create polylines for the rainbows.
     QPointF polyline[kRainbowBands * kHistorySize];
-    QPointF* dest = polyline;
-    float* source = history_;
+    QPointF *dest = polyline;
+    float *source = history_;
 
     const float top_of = static_cast<float>(height()) / 2 - static_cast<float>(kRainbowHeight[rainbowtype]) / 2;
     for (int band = 0; band < kRainbowBands; ++band) {
       // Calculate the Y position of this band.
-      const float y = static_cast<float>(kRainbowHeight[rainbowtype]) / (kRainbowBands + 1) * (band + 0.5) + top_of;
+      const float y = static_cast<float>(kRainbowHeight[rainbowtype]) / static_cast<float>(kRainbowBands + 1) * (static_cast<float>(band) + 0.5F) + top_of;
 
       // Add each point in the line.
       for (int x = 0; x < kHistorySize; ++x) {
@@ -202,8 +203,8 @@ void Rainbow::RainbowAnalyzer::analyze(QPainter& p, const Analyzer::Scope& s, bo
 
 }
 
-Rainbow::NyanCatAnalyzer::NyanCatAnalyzer(QWidget* parent)
+Rainbow::NyanCatAnalyzer::NyanCatAnalyzer(QWidget *parent)
     : RainbowAnalyzer(Rainbow::RainbowAnalyzer::Nyancat, parent) {}
 
-Rainbow::RainbowDashAnalyzer::RainbowDashAnalyzer(QWidget* parent)
+Rainbow::RainbowDashAnalyzer::RainbowDashAnalyzer(QWidget *parent)
     : RainbowAnalyzer(Rainbow::RainbowAnalyzer::Dash, parent) {}
