@@ -47,17 +47,17 @@ class RainbowAnalyzer : public Analyzer::Base {
  public:
   enum RainbowType {
     Nyancat = 0,
-    Dash = 1 
+    Dash = 1
   };
 
-  RainbowAnalyzer(const RainbowType& rbtype, QWidget* parent);
+  RainbowAnalyzer(const RainbowType rbtype, QWidget *parent);
 
  protected:
   void transform(Analyzer::Scope&) override;
-  void analyze(QPainter& p, const Analyzer::Scope&, bool new_frame) override;
+  void analyze(QPainter &p, const Analyzer::Scope&, bool new_frame) override;
 
-  void timerEvent(QTimerEvent* e) override;
-  void resizeEvent(QResizeEvent* e) override;
+  void timerEvent(QTimerEvent *e) override;
+  void resizeEvent(QResizeEvent *e) override;
 
  private:
   static const int kHeight[];
@@ -93,7 +93,7 @@ class RainbowAnalyzer : public Analyzer::Base {
 
  private:
   // "constants" that get initialized in the constructor
-  float band_scale_[kRainbowBands];
+  float band_scale_[kRainbowBands]{};
   QPen colors_[kRainbowBands];
 
   // Rainbow Nyancat & Dash
@@ -104,9 +104,9 @@ class RainbowAnalyzer : public Analyzer::Base {
   int frame_;
 
   // The y positions of each point on the rainbow.
-  float history_[kHistorySize * kRainbowBands];
+  float history_[kHistorySize * kRainbowBands]{};
 
-  // A cache of the last frame's rainbow, 
+  // A cache of the last frame's rainbow,
   // so it can be used in the next frame.
   QPixmap buffer_[2];
   int current_buffer_;
@@ -118,7 +118,7 @@ class RainbowAnalyzer : public Analyzer::Base {
   // X spacing between each point in the polyline.
   int px_per_frame_;
 
-  // Amount the buffer_ is shifted to the left (off the edge of the widget) 
+  // Amount the buffer_ is shifted to the left (off the edge of the widget)
   // to make the rainbow extend from 0 to available_rainbow_width_.
   int x_offset_;
 
@@ -129,18 +129,18 @@ class NyanCatAnalyzer : public RainbowAnalyzer {
   Q_OBJECT
 
  public:
-  Q_INVOKABLE explicit NyanCatAnalyzer(QWidget* parent);
-  
-  static const char* kName;
+  Q_INVOKABLE explicit NyanCatAnalyzer(QWidget *parent);
+
+  static const char *kName;
 };
 
 class RainbowDashAnalyzer : public RainbowAnalyzer {
   Q_OBJECT
 
  public:
-  Q_INVOKABLE explicit RainbowDashAnalyzer(QWidget* parent);
+  Q_INVOKABLE explicit RainbowDashAnalyzer(QWidget *parent);
 
-  static const char* kName;
+  static const char *kName;
 };
 }
 
