@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,9 +66,9 @@ class CollectionView : public AutoExpandingTreeView {
   void keyboardSearch(const QString &search) override;
   void scrollTo(const QModelIndex &idx, ScrollHint hint = EnsureVisible) override;
 
-  int TotalSongs();
-  int TotalArtists();
-  int TotalAlbums();
+  int TotalSongs() const;
+  int TotalArtists() const;
+  int TotalAlbums() const;
 
  public slots:
   void TotalSongCountUpdated(const int count);
@@ -106,7 +107,7 @@ class CollectionView : public AutoExpandingTreeView {
   void CopyToDevice();
   void EditTracks();
   void RescanSongs();
-  void ShowInBrowser();
+  void ShowInBrowser() const;
   void ShowInVarious();
   void NoShowInVarious();
   void Delete();
