@@ -1,7 +1,8 @@
 /*
  * Strawberry Music Player
  * This file was part of Clementine.
- * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2015, Nick Lanham <nick@afternight.org>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,7 +49,8 @@
 SavedGroupingManager::SavedGroupingManager(QWidget *parent)
     : QDialog(parent),
       ui_(new Ui_SavedGroupingManager),
-      model_(new QStandardItemModel(0, 4, this)) {
+      model_(new QStandardItemModel(0, 4, this)),
+      filter_(nullptr) {
 
   ui_->setupUi(this);
 
@@ -72,7 +74,7 @@ SavedGroupingManager::~SavedGroupingManager() {
   delete model_;
 }
 
-QString SavedGroupingManager::GroupByToString(const CollectionModel::GroupBy &g) {
+QString SavedGroupingManager::GroupByToString(const CollectionModel::GroupBy g) {
 
   switch (g) {
     case CollectionModel::GroupBy_None:
