@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2012, David Sansome <me@davidsansome.com>
- * Copyright 2018-2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 #include "config.h"
 
 #include <cstddef>
-#include <cassert>
 #include <sqlite3.h>
 #include <boost/scope_exit.hpp>
 
@@ -51,10 +50,11 @@
 #include "taskmanager.h"
 #include "database.h"
 #include "application.h"
+#include "sqlquery.h"
 #include "scopedtransaction.h"
 
 const char *Database::kDatabaseFilename = "strawberry.db";
-const int Database::kSchemaVersion = 13;
+const int Database::kSchemaVersion = 15;
 const char *Database::kMagicAllSongsTables = "%allsongstables";
 
 int Database::sNextConnectionId = 1;
@@ -96,12 +96,12 @@ Database::~Database() {
 }
 
 void Database::ExitAsync() {
-  metaObject()->invokeMethod(this, "Exit", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(this, "Exit", Qt::QueuedConnection);
 }
 
 void Database::Exit() {
 
-  assert(QThread::currentThread() == thread());
+  Q_ASSERT(QThread::currentThread() == thread());
   Close();
   moveToThread(original_thread_);
   emit ExitFinished();
@@ -135,10 +135,12 @@ QSqlDatabase Database::Connect() {
   db.setConnectOptions("QSQLITE_BUSY_TIMEOUT=30000");
   //qLog(Debug) << "Opened database with connection id" << connection_id;
 
-  if (!injected_database_name_.isNull())
-    db.setDatabaseName(injected_database_name_);
-  else
+  if (injected_database_name_.isNull()) {
     db.setDatabaseName(directory_ + "/" + kDatabaseFilename);
+  }
+  else {
+    db.setDatabaseName(injected_database_name_);
+  }
 
   if (!db.open()) {
     app_->AddError("Database: " + db.lastError().text());
@@ -151,7 +153,7 @@ QSqlDatabase Database::Connect() {
     UpdateDatabaseSchema(0, db);
   }
 
-  {
+  if (SchemaVersion(&db) < 10) {
     //  Register unicode from unicode61 tokenizer to drop old FTS3 tables.
     //  We need it also to drop old devices later when loading devices.
     //  And that's done in a different thread after schemas are upgraded, so register it anyway.
@@ -160,20 +162,19 @@ QSqlDatabase Database::Connect() {
     if (v.isValid() && qstrcmp(v.typeName(), "sqlite3*") == 0) {
       sqlite3 *handle = *static_cast<sqlite3**>(v.data());
       if (handle) {
-        int result = sqlite3_db_config(handle, SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER, 1, NULL);
-        if (result != SQLITE_OK) qLog(Fatal) << "Unable to enable FTS3 tokenizer";
+        (void)sqlite3_db_config(handle, SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER, 1, nullptr);
       }
       else qLog(Fatal) << "Unable to enable FTS3 tokenizer";
     }
 #endif
-    QSqlQuery get_fts_tokenizer(db);
+    SqlQuery get_fts_tokenizer(db);
     get_fts_tokenizer.prepare("SELECT fts3_tokenizer(:name)");
-    get_fts_tokenizer.bindValue(":name", "unicode61");
+    get_fts_tokenizer.BindValue(":name", "unicode61");
     if (get_fts_tokenizer.exec() && get_fts_tokenizer.next()) {
-      QSqlQuery set_fts_tokenizer(db);
+      SqlQuery set_fts_tokenizer(db);
       set_fts_tokenizer.prepare("SELECT fts3_tokenizer(:name, :pointer)");
-      set_fts_tokenizer.bindValue(":name", "unicode");
-      set_fts_tokenizer.bindValue(":pointer", get_fts_tokenizer.value(0));
+      set_fts_tokenizer.BindValue(":name", "unicode");
+      set_fts_tokenizer.BindValue(":pointer", get_fts_tokenizer.value(0));
       if (!set_fts_tokenizer.exec()) {
         qLog(Warning) << "Couldn't register FTS3 tokenizer : " << set_fts_tokenizer.lastError();
       }
@@ -184,17 +185,18 @@ QSqlDatabase Database::Connect() {
   }
 
   // Attach external databases
-  for (const QString &key : attached_databases_.keys()) {
+  QStringList keys = attached_databases_.keys();
+  for (const QString &key : keys) {
     QString filename = attached_databases_[key].filename_;
 
     if (!injected_database_name_.isNull()) filename = injected_database_name_;
 
     // Attach the db
-    QSqlQuery q(db);
+    SqlQuery q(db);
     q.prepare("ATTACH DATABASE :filename AS :alias");
-    q.bindValue(":filename", filename);
-    q.bindValue(":alias", key);
-    if (!q.exec()) {
+    q.BindValue(":filename", filename);
+    q.BindValue(":alias", key);
+    if (!q.Exec()) {
       qFatal("Couldn't attach external database '%s'", key.toLatin1().constData());
     }
   }
@@ -204,13 +206,15 @@ QSqlDatabase Database::Connect() {
   }
 
   // We might have to initialize the schema in some attached databases now, if they were deleted and don't match up with the main schema version.
-  for (const QString &key : attached_databases_.keys()) {
-    if (attached_databases_[key].is_temporary_ && attached_databases_[key].schema_.isEmpty())
+  keys = attached_databases_.keys();
+  for (const QString &key : keys) {
+    if (attached_databases_[key].is_temporary_ && attached_databases_[key].schema_.isEmpty()) {
       continue;
+    }
     // Find out if there are any tables in this database
-    QSqlQuery q(db);
+    SqlQuery q(db);
     q.prepare(QString("SELECT ROWID FROM %1.sqlite_master WHERE type='table'").arg(key));
-    if (!q.exec() || !q.next()) {
+    if (!q.Exec() || !q.next()) {
       q.finish();
       ExecSchemaCommandsFromFile(db, attached_databases_[key].schema_, 0);
     }
@@ -245,9 +249,12 @@ int Database::SchemaVersion(QSqlDatabase *db) {
   // Get the database's schema version
   int schema_version = 0;
   {
-    QSqlQuery q("SELECT version FROM schema_version", *db);
-    if (q.next()) schema_version = q.value(0).toInt();
-    // Implicit invocation of ~QSqlQuery() when leaving the scope to release any remaining database locks!
+    SqlQuery q(*db);
+    q.prepare("SELECT version FROM schema_version");
+    if (q.Exec() && q.next()) {
+      schema_version = q.value(0).toInt();
+    }
+    // Implicit invocation of ~SqlQuery() when leaving the scope to release any remaining database locks!
   }
   return schema_version;
 
@@ -283,10 +290,10 @@ void Database::RecreateAttachedDb(const QString &database_name) {
   {
     QSqlDatabase db(Connect());
 
-    QSqlQuery q(db);
+    SqlQuery q(db);
     q.prepare("DETACH DATABASE :alias");
-    q.bindValue(":alias", database_name);
-    if (!q.exec()) {
+    q.BindValue(":alias", database_name);
+    if (!q.Exec()) {
       qLog(Warning) << "Failed to detach database" << database_name;
       return;
     }
@@ -313,11 +320,11 @@ void Database::AttachDatabaseOnDbConnection(const QString &database_name, const
   AttachDatabase(database_name, database);
 
   // Attach the db
-  QSqlQuery q(db);
+  SqlQuery q(db);
   q.prepare("ATTACH DATABASE :filename AS :alias");
-  q.bindValue(":filename", database.filename_);
-  q.bindValue(":alias", database_name);
-  if (!q.exec()) {
+  q.BindValue(":filename", database.filename_);
+  q.BindValue(":alias", database_name);
+  if (!q.Exec()) {
     qFatal("Couldn't attach external database '%s'", database_name.toLatin1().constData());
   }
 
@@ -329,10 +336,10 @@ void Database::DetachDatabase(const QString &database_name) {
   {
     QSqlDatabase db(Connect());
 
-    QSqlQuery q(db);
+    SqlQuery q(db);
     q.prepare("DETACH DATABASE :alias");
-    q.bindValue(":alias", database_name);
-    if (!q.exec()) {
+    q.BindValue(":alias", database_name);
+    if (!q.Exec()) {
       qLog(Warning) << "Failed to detach database" << database_name;
       return;
     }
@@ -345,7 +352,9 @@ void Database::DetachDatabase(const QString &database_name) {
 void Database::UpdateDatabaseSchema(int version, QSqlDatabase &db) {
 
   QString filename;
-  if (version == 0) filename = ":/schema/schema.sql";
+  if (version == 0) {
+    filename = ":/schema/schema.sql";
+  }
   else {
     filename = QString(":/schema/schema-%1.sql").arg(version);
     qLog(Debug) << "Applying database schema update" << version << "from" << filename;
@@ -357,12 +366,13 @@ void Database::UpdateDatabaseSchema(int version, QSqlDatabase &db) {
 
 void Database::UrlEncodeFilenameColumn(const QString &table, QSqlDatabase &db) {
 
-  QSqlQuery select(db);
+  SqlQuery select(db);
   select.prepare(QString("SELECT ROWID, filename FROM %1").arg(table));
-  QSqlQuery update(db);
+  SqlQuery update(db);
   update.prepare(QString("UPDATE %1 SET filename=:filename WHERE ROWID=:id").arg(table));
-  select.exec();
-  if (CheckErrors(select)) return;
+  if (!select.Exec()) {
+    ReportErrors(select);
+  }
 
   while (select.next()) {
     const int rowid = select.value(0).toInt();
@@ -374,10 +384,11 @@ void Database::UrlEncodeFilenameColumn(const QString &table, QSqlDatabase &db) {
 
     const QUrl url = QUrl::fromLocalFile(filename);
 
-    update.bindValue(":filename", url.toEncoded());
-    update.bindValue(":id", rowid);
-    update.exec();
-    CheckErrors(update);
+    update.BindValue(":filename", url.toEncoded());
+    update.BindValue(":id", rowid);
+    if (!update.Exec()) {
+      ReportErrors(update);
+    }
   }
 
 }
@@ -386,9 +397,12 @@ void Database::ExecSchemaCommandsFromFile(QSqlDatabase &db, const QString &filen
 
   // Open and read the database schema
   QFile schema_file(filename);
-  if (!schema_file.open(QIODevice::ReadOnly))
-    qFatal("Couldn't open schema file %s", filename.toUtf8().constData());
+  if (!schema_file.open(QIODevice::ReadOnly)) {
+    qFatal("Couldn't open schema file %s for reading: %s", filename.toUtf8().constData(), schema_file.errorString().toUtf8().constData());
+    return;
+  }
   ExecSchemaCommands(db, QString::fromUtf8(schema_file.readAll()), schema_version, in_transaction);
+  schema_file.close();
 
 }
 
@@ -428,20 +442,27 @@ void Database::ExecSongTablesCommands(QSqlDatabase &db, const QStringList &song_
         qLog(Info) << "Updating" << table << "for" << kMagicAllSongsTables;
         QString new_command(command);
         new_command.replace(kMagicAllSongsTables, table);
-        QSqlQuery query(db.exec(new_command));
-        if (CheckErrors(query))
+        SqlQuery query(db);
+        query.prepare(new_command);
+        if (!query.Exec()) {
+          ReportErrors(query);
           qFatal("Unable to update music collection database");
+        }
       }
     }
     else {
-      QSqlQuery query(db.exec(command));
-      if (CheckErrors(query)) qFatal("Unable to update music collection database");
+      SqlQuery query(db);
+      query.prepare(command);
+      if (!query.Exec()) {
+        ReportErrors(query);
+        qFatal("Unable to update music collection database");
+      }
     }
   }
 
 }
 
-QStringList Database::SongsTables(QSqlDatabase &db, int schema_version) const {
+QStringList Database::SongsTables(QSqlDatabase &db, const int schema_version) {
 
   Q_UNUSED(schema_version);
 
@@ -453,15 +474,19 @@ QStringList Database::SongsTables(QSqlDatabase &db, int schema_version) const {
   }
 
   // look for the tables in attached dbs
-  for (const QString &key : attached_databases_.keys()) {
-    QSqlQuery q(db);
+  QStringList keys = attached_databases_.keys();
+  for (const QString &key : keys) {
+    SqlQuery q(db);
     q.prepare(QString("SELECT NAME FROM %1.sqlite_master WHERE type='table' AND name='songs' OR name LIKE '%songs'").arg(key));
-    if (q.exec()) {
+    if (q.Exec()) {
       while (q.next()) {
         QString tab_name = key + "." + q.value(0).toString();
         ret << tab_name;
       }
     }
+    else {
+      ReportErrors(q);
+    }
   }
 
   ret << "playlist_items";
@@ -470,44 +495,49 @@ QStringList Database::SongsTables(QSqlDatabase &db, int schema_version) const {
 
 }
 
-bool Database::CheckErrors(const QSqlQuery &query) {
+void Database::ReportErrors(const SqlQuery &query) {
 
-  QSqlError last_error = query.lastError();
-  if (last_error.isValid()) {
-    qLog(Error) << "db error: " << last_error;
-    qLog(Error) << "faulty query: " << query.lastQuery();
-    qLog(Error) << "bound values: " << query.boundValues();
-
-    return true;
+  const QSqlError sql_error = query.lastError();
+  if (sql_error.isValid()) {
+    qLog(Error) << "Unable to execute SQL query: " << sql_error;
+    qLog(Error) << "Failed query: " << query.LastQuery();
+    QString error;
+    error += "Unable to execute SQL query: " + sql_error.text() + "<br />";
+    error += "Failed query: " + query.LastQuery();
+    emit Error(error);
   }
 
-  return false;
-
 }
 
-bool Database::IntegrityCheck(QSqlDatabase db) {
+bool Database::IntegrityCheck(const QSqlDatabase &db) {
 
   qLog(Debug) << "Starting database integrity check";
-  int task_id = app_->task_manager()->StartTask(tr("Integrity check"));
+  const int task_id = app_->task_manager()->StartTask(tr("Integrity check"));
 
   bool ok = false;
   bool error_reported = false;
   // Ask for 10 error messages at most.
-  QSqlQuery q(QString("PRAGMA integrity_check(10)"), db);
-  while (q.next()) {
-    QString message = q.value(0).toString();
+  SqlQuery q(db);
+  q.prepare("PRAGMA integrity_check(10)");
+  if (q.Exec()) {
+    while (q.next()) {
+      QString message = q.value(0).toString();
 
-    // If no errors are found, a single row with the value "ok" is returned
-    if (message == "ok") {
-      ok = true;
-      break;
-    }
-    else {
-      if (!error_reported) { app_->AddError(tr("Database corruption detected.")); }
-      app_->AddError("Database: " + message);
-      error_reported = true;
+      // If no errors are found, a single row with the value "ok" is returned
+      if (message == "ok") {
+        ok = true;
+        break;
+      }
+      else {
+        if (!error_reported) { app_->AddError(tr("Database corruption detected.")); }
+        app_->AddError("Database: " + message);
+        error_reported = true;
+      }
     }
   }
+  else {
+    ReportErrors(q);
+  }
 
   app_->task_manager()->SetTaskFinished(task_id);
 
@@ -517,7 +547,7 @@ bool Database::IntegrityCheck(QSqlDatabase db) {
 
 void Database::DoBackup() {
 
-  QSqlDatabase db(this->Connect());
+  QSqlDatabase db(Connect());
 
   if (!db.isOpen()) return;
 
@@ -531,7 +561,7 @@ void Database::DoBackup() {
 
 }
 
-bool Database::OpenDatabase(const QString &filename, sqlite3 **connection) const {
+bool Database::OpenDatabase(const QString &filename, sqlite3 **connection) {
 
   int ret = sqlite3_open(filename.toUtf8(), connection);
   if (ret != 0) {
@@ -557,7 +587,7 @@ void Database::BackupFile(const QString &filename) {
   sqlite3 *source_connection = nullptr;
   sqlite3 *dest_connection = nullptr;
 
-  BOOST_SCOPE_EXIT((&source_connection)(&dest_connection)(task_id)(app_)) {
+  BOOST_SCOPE_EXIT((&source_connection)(&dest_connection)(task_id)(app_)) {  // clazy:exclude=rule-of-three NOLINT(google-explicit-constructor)
     // Harmless to call sqlite3_close() with a nullptr pointer.
     sqlite3_close(source_connection);
     sqlite3_close(dest_connection);
