@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@
 #  include <QRecursiveMutex>
 #endif
 
+#include "sqlquery.h"
+
 class QThread;
 class Application;
 
@@ -65,7 +67,7 @@ class Database : public QObject {
   void ExitAsync();
   QSqlDatabase Connect();
   void Close();
-  bool CheckErrors(const QSqlQuery &query);
+  void ReportErrors(const SqlQuery &query);
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
   QRecursiveMutex *Mutex() { return &mutex_; }
@@ -85,7 +87,8 @@ class Database : public QObject {
 
  signals:
   void ExitFinished();
-  void Error(QString message);
+  void Error(QString error);
+  void Errors(QStringList errors);
 
  private slots:
   void Exit();
@@ -94,7 +97,7 @@ class Database : public QObject {
   void DoBackup();
 
  private:
-  int SchemaVersion(QSqlDatabase *db);
+  static int SchemaVersion(QSqlDatabase *db);
   void UpdateMainSchema(QSqlDatabase *db);
 
   void ExecSchemaCommandsFromFile(QSqlDatabase &db, const QString &filename, int schema_version, bool in_transaction = false);
@@ -102,10 +105,10 @@ class Database : public QObject {
 
   void UpdateDatabaseSchema(int version, QSqlDatabase &db);
   void UrlEncodeFilenameColumn(const QString &table, QSqlDatabase &db);
-  QStringList SongsTables(QSqlDatabase &db, int schema_version) const;
-  bool IntegrityCheck(QSqlDatabase db);
+  QStringList SongsTables(QSqlDatabase &db, const int schema_version);
+  bool IntegrityCheck(const QSqlDatabase &db);
   void BackupFile(const QString &filename);
-  bool OpenDatabase(const QString &filename, sqlite3 **connection) const;
+  static bool OpenDatabase(const QString &filename, sqlite3 **connection);
 
   Application *app_;
 
@@ -140,6 +143,8 @@ class Database : public QObject {
 };
 
 class MemoryDatabase : public Database {
+  Q_OBJECT
+
  public:
   explicit MemoryDatabase(Application *app, QObject *parent = nullptr)
       : Database(app, parent, ":memory:") {}
