@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2013, 2017-2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2013, 2017-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,8 +62,12 @@ QIcon IconLoader::Load(const QString &name, const int fixed_size, const int min_
   }
 
   QList<int> sizes;
-  if (fixed_size == 0) { sizes << 22 << 32 << 48 << 64; }
-  else sizes << fixed_size;
+  if (fixed_size == 0) {
+    sizes << 22 << 32 << 48 << 64;
+  }
+  else {
+    sizes << fixed_size;
+  }
 
   if (system_icons_) {
     IconMapper::IconProperties icon_prop;
@@ -75,7 +79,7 @@ QIcon IconLoader::Load(const QString &name, const int fixed_size, const int min_
     if (icon_prop.allow_system_icon) {
       ret = QIcon::fromTheme(name);
       if (ret.isNull()) {
-        for (QString alt_name : icon_prop.names) {
+        for (const QString &alt_name : icon_prop.names) {
           ret = QIcon::fromTheme(alt_name);
           if (!ret.isNull()) break;
         }
@@ -129,6 +133,10 @@ QIcon IconLoader::Load(const QString &name, const int fixed_size, const int min_
     if (QFile::exists(filename)) ret.addFile(filename, QSize(s, s));
   }
 
+  if (ret.isNull() && !system_icons_ && !custom_icons_) {
+    qLog(Error) << "Couldn't load icon" << name;
+  }
+
   return ret;
 
 }
