@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2013-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2013-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -84,7 +84,7 @@ class OrganizeDialog;
 class PlaylistListContainer;
 class QueueView;
 class SystemTrayIcon;
-#if defined(HAVE_GSTREAMER) && defined(HAVE_CHROMAPRINT)
+#ifdef HAVE_MUSICBRAINZ
 class TagFetcher;
 #endif
 class TrackSelectionDialog;
@@ -100,12 +100,13 @@ class Windows7ThumbBar;
 #endif
 class AddStreamDialog;
 class LastFMImportDialog;
+class RadioViewContainer;
 
 class MainWindow : public QMainWindow, public PlatformInterface {
   Q_OBJECT
 
  public:
-  explicit MainWindow(Application *app, SystemTrayIcon *tray_icon, OSDBase *osd, const CommandlineOptions &options, QWidget *parent = nullptr);
+  explicit MainWindow(Application *app, std::shared_ptr<SystemTrayIcon> tray_icon, OSDBase *osd, const CommandlineOptions &options, QWidget *parent = nullptr);
   ~MainWindow() override;
 
   static const char *kSettingsGroup;
@@ -149,7 +150,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void ForceShowOSD(const Song &song, const bool toggle);
 
   void PlaylistMenuHidden();
-  void PlaylistRightClick(const QPoint &global_pos, const QModelIndex &idx);
+  void PlaylistRightClick(const QPoint global_pos, const QModelIndex &index);
   void PlaylistCurrentChanged(const QModelIndex &current);
   void PlaylistViewSelectionModelChanged();
   void PlaylistPlay();
@@ -158,7 +159,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void PlaylistQueuePlayNext();
   void PlaylistSkip();
   void PlaylistRemoveCurrent();
-  void PlaylistEditFinished(const QModelIndex &idx);
+  void PlaylistEditFinished(const int playlist_id, const QModelIndex &idx);
   void PlaylistClearCurrent();
   void RescanSongs();
   void EditTracks();
@@ -200,7 +201,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void ToggleShowHide();
   void ToggleHide();
 
-  void Seeked(const qlonglong microseconds);
+  void Seeked(const qint64 microseconds);
   void UpdateTrackPosition();
   void UpdateTrackSliderPosition();
 
@@ -242,12 +243,11 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void SavePlaybackStatus();
   void LoadPlaybackStatus();
   void ResumePlayback();
-  void ResumePlaybackSeek(const int playback_position);
 
   void Exit();
   void DoExit();
 
-  void HandleNotificationPreview(const OSDBase::Behaviour type, QString line1, QString line2);
+  void HandleNotificationPreview(const OSDBase::Behaviour type, const QString &line1, const QString &line2);
 
   void ShowConsole();
 
@@ -272,6 +272,8 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 
   void PlaylistDelete();
 
+  void FocusSearchField();
+
  public slots:
   void CommandlineOptionsReceived(const quint32 instanceId, const QByteArray &string_options);
   void Raise();
@@ -280,7 +282,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 
   void SaveSettings();
 
-  void ApplyAddBehaviour(const BehaviourSettingsPage::AddBehaviour b, MimeData *mimedata) const;
+  static void ApplyAddBehaviour(const BehaviourSettingsPage::AddBehaviour b, MimeData *mimedata);
   void ApplyPlayBehaviour(const BehaviourSettingsPage::PlayBehaviour b, MimeData *mimedata) const;
 
   void CheckFullRescanRevisions();
@@ -299,7 +301,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 #endif
 
   Application *app_;
-  SystemTrayIcon *tray_icon_;
+  std::shared_ptr<SystemTrayIcon> tray_icon_;
   OSDBase *osd_;
   Lazy<About> about_dialog_;
   Lazy<Console> console_;
@@ -327,7 +329,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 #endif
   Lazy<AddStreamDialog> add_stream_dialog_;
 
-#if defined(HAVE_GSTREAMER) && defined(HAVE_CHROMAPRINT)
+#ifdef HAVE_MUSICBRAINZ
   std::unique_ptr<TagFetcher> tag_fetcher_;
 #endif
   std::unique_ptr<TrackSelectionDialog> track_selection_dialog_;
@@ -339,6 +341,8 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   InternetTabsView *tidal_view_;
   InternetTabsView *qobuz_view_;
 
+  RadioViewContainer *radio_view_;
+
   LastFMImportDialog *lastfm_import_dialog_;
 
   QAction *collection_show_all_;
@@ -360,7 +364,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 #endif
   QAction *playlist_delete_;
   QAction *playlist_queue_;
-  QAction* playlist_queue_play_next_;
+  QAction *playlist_queue_play_next_;
   QAction *playlist_skip_;
   QAction *playlist_add_to_another_;
   QList<QAction*> playlistitem_actions_;
@@ -390,6 +394,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   Song song_;
   Song song_playing_;
   AlbumCoverImageResult album_cover_;
+  bool exit_;
   int exit_count_;
   bool delete_files_;
 
