@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,10 +101,9 @@ const char *Mpris2::kServiceName = "org.mpris.MediaPlayer2.strawberry";
 const char *Mpris2::kFreedesktopPath = "org.freedesktop.DBus.Properties";
 
 Mpris2::Mpris2(Application *app, QObject *parent)
-  : QObject(parent),
-  app_(app),
-  app_name_(QCoreApplication::applicationName())
-  {
+    : QObject(parent),
+      app_(app),
+      app_name_(QCoreApplication::applicationName()) {
 
   new Mpris2Root(this);
   new Mpris2TrackList(this);
@@ -133,8 +133,9 @@ Mpris2::Mpris2(Application *app, QObject *parent)
 
   app_name_[0] = app_name_[0].toUpper();
 
-  if (!QGuiApplication::desktopFileName().isEmpty())
+  if (!QGuiApplication::desktopFileName().isEmpty()) {
     desktop_files_ << QGuiApplication::desktopFileName();
+  }
 
   QStringList domain_split = QCoreApplication::organizationDomain().split(".");
   std::reverse(domain_split.begin(), domain_split.end());
@@ -142,7 +143,7 @@ Mpris2::Mpris2(Application *app, QObject *parent)
   desktop_files_ << QCoreApplication::applicationName().toLower();
   desktop_file_ = desktop_files_.first();
 
-  data_dirs_ = QString(getenv("XDG_DATA_DIRS")).split(":");
+  data_dirs_ = QString(qgetenv("XDG_DATA_DIRS")).split(":");
   data_dirs_.append("/usr/local/share");
   data_dirs_.append("/usr/share");
 
@@ -297,7 +298,7 @@ QStringList Mpris2::SupportedMimeTypes() const {
 
 void Mpris2::Raise() { emit RaiseMainWindow(); }
 
-void Mpris2::Quit() { qApp->quit(); }
+void Mpris2::Quit() { QCoreApplication::quit(); }
 
 QString Mpris2::PlaybackStatus() const {
   return PlaybackStatus(app_->player()->GetState());
@@ -411,9 +412,9 @@ void Mpris2::AlbumCoverLoaded(const Song &song, const AlbumCoverLoaderResult &re
 
 double Mpris2::Volume() const { return app_->player()->GetVolume() / 100.0; }
 
-void Mpris2::SetVolume(double value) { app_->player()->SetVolume(value * 100); }
+void Mpris2::SetVolume(double value) { app_->player()->SetVolume(static_cast<int>(value * 100)); }
 
-qlonglong Mpris2::Position() const {
+qint64 Mpris2::Position() const {
   return app_->player()->engine()->position_nanosec() / kNsecPerUsec;
 }
 
@@ -478,20 +479,24 @@ void Mpris2::Play() {
   }
 }
 
-void Mpris2::Seek(qlonglong offset) {
+void Mpris2::Seek(qint64 offset) {
+
   if (CanSeek()) {
     app_->player()->SeekTo(app_->player()->engine()->position_nanosec() / kNsecPerSec + offset / kUsecPerSec);
   }
+
 }
 
-void Mpris2::SetPosition(const QDBusObjectPath &trackId, qlonglong offset) {
+void Mpris2::SetPosition(const QDBusObjectPath &trackId, qint64 offset) {
+
   if (CanSeek() && trackId.path() == current_track_id() && offset >= 0) {
     offset *= kNsecPerUsec;
 
-    if(offset < app_->player()->GetCurrentItem()->Metadata().length_nanosec()) {
+    if (offset < app_->player()->GetCurrentItem()->Metadata().length_nanosec()) {
       app_->player()->SeekTo(offset / kNsecPerSec);
     }
   }
+
 }
 
 void Mpris2::OpenUri(const QString &uri) {
@@ -546,7 +551,7 @@ QDBusObjectPath MakePlaylistPath(int id) {
   return QDBusObjectPath(QString("/org/strawberrymusicplayer/strawberry/PlaylistId/%1").arg(id));
 }
 
-}
+}  // namespace
 
 MaybePlaylist Mpris2::ActivePlaylist() const {
 
@@ -589,8 +594,10 @@ MprisPlaylistList Mpris2::GetPlaylists(quint32 index, quint32 max_count, const Q
 
   Q_UNUSED(order);
 
+  QList<Playlist*> playlists = app_->playlist_manager()->GetAllPlaylists();
   MprisPlaylistList ret;
-  for (Playlist *p : app_->playlist_manager()->GetAllPlaylists()) {
+  ret.reserve(playlists.count());
+  for (Playlist *p : playlists) {
     MprisPlaylist mpris_playlist;
     mpris_playlist.id = MakePlaylistPath(p->id());
     mpris_playlist.name = app_->playlist_manager()->GetPlaylistName(p->id());
