@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,25 +73,25 @@ const char *Player::kSettingsGroup = "Player";
 
 Player::Player(Application *app, QObject *parent)
     : PlayerInterface(parent),
-    app_(app),
-    engine_(nullptr),
+      app_(app),
+      engine_(nullptr),
 #ifdef HAVE_GSTREAMER
-    gst_startup_(new GstStartup(this)),
+      gst_startup_(new GstStartup(this)),
 #endif
-    analyzer_(nullptr),
-    equalizer_(nullptr),
-    stream_change_type_(Engine::First),
-    autoscroll_(Playlist::AutoScroll_Maybe),
-    last_state_(Engine::Empty),
-    nb_errors_received_(0),
-    volume_before_mute_(100),
-    last_pressed_previous_(QDateTime::currentDateTime()),
-    continue_on_error_(false),
-    greyout_(true),
-    menu_previousmode_(BehaviourSettingsPage::PreviousBehaviour_DontRestart),
-    seek_step_sec_(10),
-    volume_control_(true)
-    {
+      analyzer_(nullptr),
+      equalizer_(nullptr),
+      stream_change_type_(Engine::First),
+      autoscroll_(Playlist::AutoScroll_Maybe),
+      last_state_(Engine::Empty),
+      nb_errors_received_(0),
+      volume_before_mute_(100),
+      last_pressed_previous_(QDateTime::currentDateTime()),
+      continue_on_error_(false),
+      greyout_(true),
+      menu_previousmode_(BehaviourSettingsPage::PreviousBehaviour_DontRestart),
+      seek_step_sec_(10),
+      volume_control_(true),
+      play_offset_nanosec_(0) {
 
   settings_.beginGroup(kSettingsGroup);
 
@@ -111,7 +111,7 @@ Engine::EngineType Player::CreateEngine(Engine::EngineType enginetype) {
 
   Engine::EngineType use_enginetype(Engine::None);
 
-  for (int i = 0 ; use_enginetype == Engine::None ; i++) {
+  for (int i = 0; use_enginetype == Engine::None; i++) {
     switch(enginetype) {
       case Engine::None:
 #ifdef HAVE_GSTREAMER
@@ -126,7 +126,7 @@ Engine::EngineType Player::CreateEngine(Engine::EngineType enginetype) {
 #ifdef HAVE_VLC
       case Engine::VLC:
         use_enginetype=Engine::VLC;
-        engine_.reset(new VLCEngine(app_->task_manager()));
+        engine_ = std::make_shared<VLCEngine>(app_->task_manager());
         break;
 #endif
       default:
@@ -159,7 +159,7 @@ void Player::Init() {
 
   QSettings s;
 
-  if (!engine_.get()) {
+  if (!engine_) {
     s.beginGroup(BackendSettingsPage::kSettingsGroup);
     Engine::EngineType enginetype = Engine::EngineTypeFromName(s.value("engine", EngineName(Engine::GStreamer)).toString().toLower());
     s.endGroup();
@@ -333,11 +333,12 @@ void Player::HandleLoadResult(const UrlHandler::LoadResult &result) {
       }
 
       if (is_current) {
-        qLog(Debug) << "Playing song" << item->Metadata().title() << result.stream_url_;
-        engine_->Play(result.stream_url_, result.original_url_, stream_change_type_, song.has_cue(), song.beginning_nanosec(), song.end_nanosec());
+        qLog(Debug) << "Playing song" << item->Metadata().title() << result.stream_url_ << "position" << play_offset_nanosec_;
+        engine_->Play(result.stream_url_, result.original_url_, stream_change_type_, song.has_cue(), song.beginning_nanosec(), song.end_nanosec(), play_offset_nanosec_);
         current_item_ = item;
+        play_offset_nanosec_ = 0;
       }
-      else if (is_next) {
+      else if (is_next && !item->Metadata().is_module_music()) {
         qLog(Debug) << "Preloading next song" << next_item->Metadata().title() << result.stream_url_;
         engine_->StartPreloading(result.stream_url_, next_item->Url(), song.has_cue(), song.beginning_nanosec(), song.end_nanosec());
       }
@@ -359,6 +360,9 @@ void Player::Next() { NextInternal(Engine::Manual, Playlist::AutoScroll_Always);
 
 void Player::NextInternal(const Engine::TrackChangeFlags change, const Playlist::AutoScroll autoscroll) {
 
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
+
   if (HandleStopAfter(autoscroll)) return;
 
   NextItem(change, autoscroll);
@@ -367,6 +371,9 @@ void Player::NextInternal(const Engine::TrackChangeFlags change, const Playlist:
 
 void Player::NextItem(const Engine::TrackChangeFlags change, const Playlist::AutoScroll autoscroll) {
 
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
+
   Playlist *active_playlist = app_->playlist_manager()->active();
 
   // If we received too many errors in auto change, with repeat enabled, we stop
@@ -394,7 +401,7 @@ void Player::NextItem(const Engine::TrackChangeFlags change, const Playlist::Aut
     return;
   }
 
-  PlayAt(i, change, autoscroll, false, true);
+  PlayAt(i, 0, change, autoscroll, false, true);
 
 }
 
@@ -402,7 +409,11 @@ void Player::PlayPlaylist(const QString &playlist_name) {
   PlayPlaylistInternal(Engine::Manual, Playlist::AutoScroll_Always, playlist_name);
 }
 
-void Player::PlayPlaylistInternal(Engine::TrackChangeFlags change, const Playlist::AutoScroll autoscroll, const QString &playlist_name) {
+void Player::PlayPlaylistInternal(const Engine::TrackChangeFlags change, const Playlist::AutoScroll autoscroll, const QString &playlist_name) {
+
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
+
   Playlist *playlist = nullptr;
   for (Playlist *p : app_->playlist_manager()->GetAllPlaylists()) {
     if (playlist_name == app_->playlist_manager()->GetPlaylistName(p->id())) {
@@ -424,7 +435,8 @@ void Player::PlayPlaylistInternal(Engine::TrackChangeFlags change, const Playlis
   if (i == -1) i = app_->playlist_manager()->active()->last_played_row();
   if (i == -1) i = 0;
 
-  PlayAt(i, change, autoscroll, true);
+  PlayAt(i, 0, change, autoscroll, true);
+
 }
 
 
@@ -442,6 +454,7 @@ bool Player::HandleStopAfter(const Playlist::AutoScroll autoscroll) {
     Stop(true);
     return true;
   }
+
   return false;
 
 }
@@ -458,11 +471,11 @@ void Player::TrackEnded() {
 
 }
 
-void Player::PlayPause(Playlist::AutoScroll autoscroll) {
+void Player::PlayPause(const quint64 offset_nanosec, const Playlist::AutoScroll autoscroll) {
 
   switch (engine_->state()) {
     case Engine::Paused:
-      engine_->Unpause();
+      UnPause();
       emit Resumed();
       break;
 
@@ -471,6 +484,8 @@ void Player::PlayPause(Playlist::AutoScroll autoscroll) {
         Stop();
       }
       else {
+        pause_time_ = QDateTime::currentDateTime();
+        play_offset_nanosec_ = engine_->position_nanosec();
         engine_->Pause();
       }
       break;
@@ -479,31 +494,63 @@ void Player::PlayPause(Playlist::AutoScroll autoscroll) {
     case Engine::Empty:
     case Engine::Error:
     case Engine::Idle: {
+      pause_time_ = QDateTime();
+      play_offset_nanosec_ = offset_nanosec;
       app_->playlist_manager()->SetActivePlaylist(app_->playlist_manager()->current_id());
       if (app_->playlist_manager()->active()->rowCount() == 0) break;
       int i = app_->playlist_manager()->active()->current_row();
       if (i == -1) i = app_->playlist_manager()->active()->last_played_row();
       if (i == -1) i = 0;
-      PlayAt(i, Engine::First, autoscroll, true);
+      PlayAt(i, offset_nanosec, Engine::First, autoscroll, true);
       break;
     }
   }
 
 }
 
+void Player::UnPause() {
+
+  if (current_item_ && pause_time_.isValid()) {
+    const Song &song = current_item_->Metadata();
+    if (url_handlers_.contains(song.url().scheme()) && song.stream_url_can_expire()) {
+      const quint64 time = QDateTime::currentDateTime().toSecsSinceEpoch() - pause_time_.toSecsSinceEpoch();
+      if (time >= 30) { // Stream URL might be expired.
+        qLog(Debug) << "Re-requesting stream URL for" << song.url();
+        play_offset_nanosec_ = engine_->position_nanosec();
+        HandleLoadResult(url_handlers_[song.url().scheme()]->StartLoading(song.url()));
+        return;
+      }
+    }
+  }
+
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
+
+  engine_->Unpause();
+
+}
+
 void Player::RestartOrPrevious() {
 
-  if (engine_->position_nanosec() < 8 * kNsecPerSec) return Previous();
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
+
+  if (engine_->position_nanosec() < 8 * kNsecPerSec) {
+    Previous();
+    return;
+  }
 
   SeekTo(0);
 
 }
 
-void Player::Stop(bool stop_after) {
+void Player::Stop(const bool stop_after) {
 
   engine_->Stop(stop_after);
   app_->playlist_manager()->active()->set_current_row(-1);
   current_item_.reset();
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
 
 }
 
@@ -515,12 +562,16 @@ bool Player::PreviousWouldRestartTrack() const {
 
   // Check if it has been over two seconds since previous button was pressed
   return menu_previousmode_ == BehaviourSettingsPage::PreviousBehaviour_Restart && last_pressed_previous_.isValid() && last_pressed_previous_.secsTo(QDateTime::currentDateTime()) >= 2;
+
 }
 
 void Player::Previous() { PreviousItem(Engine::Manual); }
 
 void Player::PreviousItem(const Engine::TrackChangeFlags change) {
 
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = 0;
+
   const bool ignore_repeat_track = change & Engine::Manual;
 
   if (menu_previousmode_ == BehaviourSettingsPage::PreviousBehaviour_Restart) {
@@ -528,7 +579,7 @@ void Player::PreviousItem(const Engine::TrackChangeFlags change) {
     QDateTime now = QDateTime::currentDateTime();
     if (last_pressed_previous_.isValid() && last_pressed_previous_.secsTo(now) >= 2) {
       last_pressed_previous_ = now;
-      PlayAt(app_->playlist_manager()->active()->current_row(), change, Playlist::AutoScroll_Always, false, true);
+      PlayAt(app_->playlist_manager()->active()->current_row(), 0, change, Playlist::AutoScroll_Always, false, true);
       return;
     }
     last_pressed_previous_ = now;
@@ -538,11 +589,11 @@ void Player::PreviousItem(const Engine::TrackChangeFlags change) {
   app_->playlist_manager()->active()->set_current_row(i, Playlist::AutoScroll_Always, false);
   if (i == -1) {
     Stop();
-    PlayAt(i, change, Playlist::AutoScroll_Always, true);
+    PlayAt(i, 0, change, Playlist::AutoScroll_Always, true);
     return;
   }
 
-  PlayAt(i, change, Playlist::AutoScroll_Always, false);
+  PlayAt(i, 0, change, Playlist::AutoScroll_Always, false);
 
 }
 
@@ -557,9 +608,13 @@ void Player::EngineStateChanged(const Engine::State state) {
 
   switch (state) {
     case Engine::Paused:
+      pause_time_ = QDateTime::currentDateTime();
+      play_offset_nanosec_ = engine_->position_nanosec();
       emit Paused();
       break;
     case Engine::Playing:
+      pause_time_ = QDateTime();
+      play_offset_nanosec_ = 0;
       emit Playing();
       break;
     case Engine::Error:
@@ -567,18 +622,21 @@ void Player::EngineStateChanged(const Engine::State state) {
       // fallthrough
     case Engine::Empty:
     case Engine::Idle:
+      pause_time_ = QDateTime();
+      play_offset_nanosec_ = 0;
       emit Stopped();
       break;
   }
+
   last_state_ = state;
 
 }
 
-void Player::SetVolume(const int value) {
+void Player::SetVolume(const uint value) {
 
-  int old_volume = engine_->volume();
+  uint old_volume = engine_->volume();
 
-  int volume = qBound(0, value, 100);
+  uint volume = qBound(0U, value, 100U);
   settings_.setValue("volume", volume);
   engine_->SetVolume(volume);
 
@@ -588,9 +646,12 @@ void Player::SetVolume(const int value) {
 
 }
 
-int Player::GetVolume() const { return engine_->volume(); }
+uint Player::GetVolume() const { return engine_->volume(); }
 
-void Player::PlayAt(const int index, Engine::TrackChangeFlags change, const Playlist::AutoScroll autoscroll, const bool reshuffle, const bool force_inform) {
+void Player::PlayAt(const int index, const quint64 offset_nanosec, Engine::TrackChangeFlags change, const Playlist::AutoScroll autoscroll, const bool reshuffle, const bool force_inform) {
+
+  pause_time_ = QDateTime();
+  play_offset_nanosec_ = offset_nanosec;
 
   if (current_item_ && change == Engine::Manual && engine_->position_nanosec() != engine_->length_nanosec()) {
     emit TrackSkipped(current_item_);
@@ -613,15 +674,17 @@ void Player::PlayAt(const int index, Engine::TrackChangeFlags change, const Play
 
   if (url_handlers_.contains(url.scheme())) {
     // It's already loading
-    if (loading_async_.contains(url)) return;
+    if (loading_async_.contains(url)) {
+      return;
+    }
 
     stream_change_type_ = change;
     autoscroll_ = autoscroll;
     HandleLoadResult(url_handlers_[url.scheme()]->StartLoading(url));
   }
   else {
-    qLog(Debug) << "Playing song" << current_item_->Metadata().title() << url;
-    engine_->Play(url, current_item_->Url(), change, current_item_->Metadata().has_cue(), current_item_->effective_beginning_nanosec(), current_item_->effective_end_nanosec());
+    qLog(Debug) << "Playing song" << current_item_->Metadata().title() << url << "position" << offset_nanosec;
+    engine_->Play(url, current_item_->Url(), change, current_item_->Metadata().has_cue(), current_item_->effective_beginning_nanosec(), current_item_->effective_end_nanosec(), offset_nanosec);
   }
 
 }
@@ -633,7 +696,7 @@ void Player::CurrentMetadataChanged(const Song &metadata) {
 
 }
 
-void Player::SeekTo(const int seconds) {
+void Player::SeekTo(const quint64 seconds) {
 
   const qint64 length_nanosec = engine_->length_nanosec();
 
@@ -642,7 +705,7 @@ void Player::SeekTo(const int seconds) {
     return;
   }
 
-  const qint64 nanosec = qBound(0ll, qint64(seconds) * kNsecPerSec, length_nanosec);
+  const qint64 nanosec = qBound(0LL, static_cast<qint64>(seconds) * kNsecPerSec, length_nanosec);
   engine_->Seek(nanosec);
 
   qLog(Debug) << "Track seeked to" << nanosec << "ns - updating scrobble point";
@@ -703,7 +766,7 @@ void Player::Mute() {
 
   if (!volume_control_) return;
 
-  const int current_volume = engine_->volume();
+  const uint current_volume = engine_->volume();
 
   if (current_volume == 0) {
     SetVolume(volume_before_mute_);
@@ -717,17 +780,17 @@ void Player::Mute() {
 
 void Player::Pause() { engine_->Pause(); }
 
-void Player::Play() {
+void Player::Play(const quint64 offset_nanosec) {
 
   switch (GetState()) {
     case Engine::Playing:
-      SeekTo(0);
+      SeekTo(offset_nanosec);
       break;
     case Engine::Paused:
-      engine_->Unpause();
+      UnPause();
       break;
     default:
-      PlayPause();
+      PlayPause(offset_nanosec);
       break;
   }
 
@@ -793,6 +856,12 @@ void Player::TrackAboutToEnd() {
     }
   }
 
+  // Preloading any format while currently playing module music is broken in GStreamer.
+  // See: https://gitlab.freedesktop.org/gstreamer/gstreamer/-/issues/769
+  if (current_item_ && current_item_->Metadata().is_module_music()) {
+    return;
+  }
+
   engine_->StartPreloading(url, next_item->Url(), next_item->Metadata().has_cue(), next_item->effective_beginning_nanosec(), next_item->effective_end_nanosec());
 
 }
