@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +23,9 @@
 
 #include "settingsprovider.h"
 
-SettingsProvider::SettingsProvider() {}
+SettingsProvider::SettingsProvider() = default;
 
-DefaultSettingsProvider::DefaultSettingsProvider() {}
+DefaultSettingsProvider::DefaultSettingsProvider() = default;
 
 void DefaultSettingsProvider::set_group(const char *group) {
   while (!backend_.group().isEmpty()) backend_.endGroup();
