@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@
 #include "song.h"
 #include "songloader.h"
 #include "tagreaderclient.h"
+#include "database.h"
 #include "engine/enginetype.h"
 #include "engine/enginebase.h"
 #include "collection/collectionbackend.h"
@@ -65,13 +66,11 @@
 #  include "device/cddasongloader.h"
 #endif
 
-using std::placeholders::_1;
-
 QSet<QString> SongLoader::sRawUriSchemes;
 const int SongLoader::kDefaultTimeout = 5000;
 
-SongLoader::SongLoader(CollectionBackendInterface *collection, const Player *player, QObject *parent) :
-      QObject(parent),
+SongLoader::SongLoader(CollectionBackendInterface *collection, const Player *player, QObject *parent)
+    : QObject(parent),
       timeout_timer_(new QTimer(this)),
       playlist_parser_(new PlaylistParser(collection, this)),
       cue_parser_(new CueParser(collection, this)),
@@ -113,6 +112,8 @@ SongLoader::~SongLoader() {
 
 SongLoader::Result SongLoader::Load(const QUrl &url) {
 
+  if (url.isEmpty()) return Error;
+
   url_ = url;
 
   if (url_.isLocalFile()) {
@@ -159,22 +160,33 @@ SongLoader::Result SongLoader::LoadFilenamesBlocking() {
 SongLoader::Result SongLoader::LoadLocalPartial(const QString &filename) {
 
   qLog(Debug) << "Fast Loading local file" << filename;
+
+  QFileInfo fileinfo(filename);
+
+  if (!fileinfo.exists()) {
+    errors_ << tr("File %1 does not exist.").arg(filename);
+    return Error;
+  }
+
   // First check to see if it's a directory - if so we can load all the songs inside right away.
-  if (QFileInfo(filename).isDir()) {
+  if (fileinfo.isDir()) {
     LoadLocalDirectory(filename);
     return Success;
   }
-  Song song(Song::Source_LocalFile);
-  song.InitFromFilePartial(filename);
-  if (song.is_valid()) {
-    songs_ << song;
-    return Success;
-  }
-  else {
-    errors_ << song.error();
-    return Error;
+
+  // Assume it's just a normal file
+  if (TagReaderClient::Instance()->IsMediaFileBlocking(filename) || Song::kAcceptedExtensions.contains(fileinfo.suffix(), Qt::CaseInsensitive)) {
+    Song song(Song::Source_LocalFile);
+    song.InitFromFilePartial(filename, fileinfo);
+    if (song.is_valid()) {
+      songs_ << song;
+      return Success;
+    }
   }
 
+  errors_ << QObject::tr("File %1 is not recognized as a valid audio file.").arg(filename);
+  return Error;
+
 }
 
 SongLoader::Result SongLoader::LoadAudioCD() {
@@ -224,11 +236,14 @@ SongLoader::Result SongLoader::LoadLocal(const QString &filename) {
   // Search in the database.
   QUrl url = QUrl::fromLocalFile(filename);
 
-  CollectionQuery query;
+  QMutexLocker l(collection_->db()->Mutex());
+  QSqlDatabase db(collection_->db()->Connect());
+
+  CollectionQuery query(db, collection_->songs_table(), collection_->fts_table());
   query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
   query.AddWhere("url", url.toEncoded());
 
-  if (collection_->ExecQuery(&query) && query.Next()) {
+  if (query.Exec() && query.Next()) {
     // We may have many results when the file has many sections
     do {
       Song song(Song::Source_Collection);
@@ -237,8 +252,7 @@ SongLoader::Result SongLoader::LoadLocal(const QString &filename) {
       if (song.is_valid()) {
         songs_ << song;
       }
-    }
-    while (query.Next());
+    } while (query.Next());
 
     return Success;
   }
@@ -251,8 +265,15 @@ SongLoader::Result SongLoader::LoadLocal(const QString &filename) {
 
 SongLoader::Result SongLoader::LoadLocalAsync(const QString &filename) {
 
+  QFileInfo fileinfo(filename);
+
+  if (!fileinfo.exists()) {
+    errors_ << tr("File %1 does not exist.").arg(filename);
+    return Error;
+  }
+
   // First check to see if it's a directory - if so we will load all the songs inside right away.
-  if (QFileInfo(filename).isDir()) {
+  if (fileinfo.isDir()) {
     LoadLocalDirectory(filename);
     return Success;
   }
@@ -260,62 +281,71 @@ SongLoader::Result SongLoader::LoadLocalAsync(const QString &filename) {
   // It's a local file, so check if it looks like a playlist. Read the first few bytes.
   QFile file(filename);
   if (!file.open(QIODevice::ReadOnly)) {
-    errors_ << tr("Could not open file %1").arg(filename);
+    errors_ << tr("Could not open file %1 for reading: %2").arg(filename, file.errorString());
     return Error;
   }
   QByteArray data(file.read(PlaylistParser::kMagicSize));
+  file.close();
 
   ParserBase *parser = playlist_parser_->ParserForMagic(data);
   if (!parser) {
     // Check the file extension as well, maybe the magic failed, or it was a basic M3U file which is just a plain list of filenames.
-    parser = playlist_parser_->ParserForExtension(QFileInfo(filename).suffix().toLower());
+    parser = playlist_parser_->ParserForExtension(fileinfo.suffix().toLower());
   }
 
-  if (parser) { // It's a playlist!
+  if (parser) {  // It's a playlist!
     qLog(Debug) << "Parsing using" << parser->name();
     LoadPlaylist(parser, filename);
     return Success;
   }
 
-  // Check if it's a cue file
-  QString matching_cue = filename.section('.', 0, -2) + ".cue";
+  // Check if it's a CUE file
+  QString matching_cue = CueParser::FindCueFilename(filename);
   if (QFile::exists(matching_cue)) {
-    // it's a cue - create virtual tracks
+    // It's a CUE - create virtual tracks
     QFile cue(matching_cue);
-    cue.open(QIODevice::ReadOnly);
-
-    SongList song_list = cue_parser_->Load(&cue, matching_cue, QDir(filename.section('/', 0, -2)));
-    for (const Song &song : song_list) {
-      if (song.is_valid()) songs_ << song;
+    if (cue.open(QIODevice::ReadOnly)) {
+      const SongList songs = cue_parser_->Load(&cue, matching_cue, QDir(filename.section('/', 0, -2)));
+      cue.close();
+      for (const Song &song : songs) {
+        if (song.is_valid()) songs_ << song;
+      }
+      return Success;
+    }
+    else {
+      errors_ << tr("Could not open CUE file %1 for reading: %2").arg(matching_cue, cue.errorString());
+      return Error;
     }
-    return Success;
   }
 
   // Assume it's just a normal file
-  Song song(Song::Source_LocalFile);
-  song.InitFromFilePartial(filename);
-  if (song.is_valid()) {
-    songs_ << song;
-    return Success;
-  }
-  else {
-    errors_ << song.error();
-    return Error;
+  if (TagReaderClient::Instance()->IsMediaFileBlocking(filename) || Song::kAcceptedExtensions.contains(fileinfo.suffix(), Qt::CaseInsensitive)) {
+    Song song(Song::Source_LocalFile);
+    song.InitFromFilePartial(filename, fileinfo);
+    if (song.is_valid()) {
+      songs_ << song;
+      return Success;
+    }
   }
 
+  errors_ << QObject::tr("File %1 is not recognized as a valid audio file.").arg(filename);
+  return Error;
+
 }
 
 void SongLoader::LoadMetadataBlocking() {
+
   for (int i = 0; i < songs_.size(); i++) {
     EffectiveSongLoad(&songs_[i]);
   }
+
 }
 
 void SongLoader::EffectiveSongLoad(Song *song) {
 
-  if (!song) return;
+  if (!song || !song->url().isLocalFile()) return;
 
-  if (song->filetype() != Song::FileType_Unknown) {
+  if (song->init_from_file() && song->filetype() != Song::FileType_Unknown) {
     // Maybe we loaded the metadata already, for example from a cuesheet.
     return;
   }
@@ -326,7 +356,7 @@ void SongLoader::EffectiveSongLoad(Song *song) {
     *song = collection_song;
   }
   else {
-    // it's a normal media file
+    // It's a normal media file
     QString filename = song->url().toLocalFile();
     TagReaderClient::Instance()->ReadFileBlocking(filename, song);
   }
@@ -334,9 +364,16 @@ void SongLoader::EffectiveSongLoad(Song *song) {
 }
 
 void SongLoader::LoadPlaylist(ParserBase *parser, const QString &filename) {
+
   QFile file(filename);
-  file.open(QIODevice::ReadOnly);
-  songs_ = parser->Load(&file, filename, QFileInfo(filename).path());
+  if (file.open(QIODevice::ReadOnly)) {
+    songs_ = parser->Load(&file, filename, QFileInfo(filename).path());
+    file.close();
+  }
+  else {
+    errors_ << tr("Could not open playlist file %1 for reading: %2").arg(filename, file.errorString());
+  }
+
 }
 
 static bool CompareSongs(const Song &left, const Song &right) {
@@ -372,12 +409,14 @@ void SongLoader::LoadLocalDirectory(const QString &filename) {
 }
 
 void SongLoader::AddAsRawStream() {
-  Song song(Song::Source_Stream);
+
+  Song song(Song::SourceFromURL(url_));
   song.set_valid(true);
   song.set_filetype(Song::FileType_Stream);
   song.set_url(url_);
   song.set_title(url_.toString());
   songs_ << song;
+
 }
 
 void SongLoader::Timeout() {
@@ -402,8 +441,10 @@ void SongLoader::StopTypefind() {
 
     // Parse the playlist
     QBuffer buf(&buffer_);
-    buf.open(QIODevice::ReadOnly);
-    songs_ = parser_->Load(&buf);
+    if (buf.open(QIODevice::ReadOnly)) {
+      songs_ = parser_->Load(&buf);
+      buf.close();
+    }
 
   }
   else if (success_) {
@@ -428,15 +469,15 @@ SongLoader::Result SongLoader::LoadRemote() {
   // Otherwise wait to get 512 bytes of data and do magic on it - if the magic fails then we don't know what it is so return failure.
   // If the magic succeeds then we know for sure it's a playlist - so read the rest of the file, parse the playlist and return success.
 
-  timeout_timer_->start(timeout_);
+  ScheduleTimeoutAsync();
 
   // Create the pipeline - it gets unreffed if it goes out of scope
-  std::shared_ptr<GstElement> pipeline(gst_pipeline_new(nullptr), std::bind(&gst_object_unref, _1));
+  std::shared_ptr<GstElement> pipeline(gst_pipeline_new(nullptr), std::bind(&gst_object_unref, std::placeholders::_1));
 
   // Create the source element automatically based on the URL
   GstElement *source = gst_element_make_from_uri(GST_URI_SRC, url_.toEncoded().constData(), nullptr, nullptr);
   if (!source) {
-    errors_ << tr("Couldn't create gstreamer source element for %1").arg(url_.toString());
+    errors_ << tr("Couldn't create GStreamer source element for %1").arg(url_.toString());
     return Error;
   }
   g_object_set(source, "ssl-strict", FALSE, nullptr);
@@ -453,6 +494,7 @@ SongLoader::Result SongLoader::LoadRemote() {
   CHECKED_GCONNECT(typefind, "have-type", &TypeFound, this);
   gst_bus_set_sync_handler(bus, BusCallbackSync, this, nullptr);
   gst_bus_add_watch(bus, BusCallback, this);
+  gst_object_unref(bus);
 
   // Add a probe to the sink so we can capture the data if it's a playlist
   GstPad *pad = gst_element_get_static_pad(fakesink, "sink");
@@ -475,7 +517,7 @@ SongLoader::Result SongLoader::LoadRemote() {
 #endif
 
 #ifdef HAVE_GSTREAMER
-void SongLoader::TypeFound(GstElement *, uint, GstCaps *caps, void *self) {
+void SongLoader::TypeFound(GstElement*, uint, GstCaps *caps, void *self) {
 
   SongLoader *instance = static_cast<SongLoader*>(self);
 
@@ -501,15 +543,16 @@ GstPadProbeReturn SongLoader::DataReady(GstPad*, GstPadProbeInfo *info, gpointer
 
   SongLoader *instance = reinterpret_cast<SongLoader*>(self);
 
-  if (instance->state_ == Finished)
+  if (instance->state_ == Finished) {
     return GST_PAD_PROBE_OK;
+  }
 
   GstBuffer *buffer = gst_pad_probe_info_get_buffer(info);
   GstMapInfo map;
   gst_buffer_map(buffer, &map, GST_MAP_READ);
 
   // Append the data to the buffer
-  instance->buffer_.append(reinterpret_cast<const char*>(map.data), map.size);
+  instance->buffer_.append(reinterpret_cast<const char*>(map.data), static_cast<qint64>(map.size));
   qLog(Debug) << "Received total" << instance->buffer_.size() << "bytes";
   gst_buffer_unmap(buffer, &map);
 
@@ -523,7 +566,7 @@ GstPadProbeReturn SongLoader::DataReady(GstPad*, GstPadProbeInfo *info, gpointer
 #endif
 
 #ifdef HAVE_GSTREAMER
-gboolean SongLoader::BusCallback(GstBus *, GstMessage *msg, gpointer self) {
+gboolean SongLoader::BusCallback(GstBus*, GstMessage *msg, gpointer self) {
 
   SongLoader *instance = reinterpret_cast<SongLoader*>(self);
 
@@ -541,7 +584,7 @@ gboolean SongLoader::BusCallback(GstBus *, GstMessage *msg, gpointer self) {
 #endif
 
 #ifdef HAVE_GSTREAMER
-GstBusSyncReply SongLoader::BusCallbackSync(GstBus *, GstMessage *msg, gpointer self) {
+GstBusSyncReply SongLoader::BusCallbackSync(GstBus*, GstMessage *msg, gpointer self) {
 
   SongLoader *instance = reinterpret_cast<SongLoader*>(self);
 
@@ -566,12 +609,12 @@ void SongLoader::ErrorMessageReceived(GstMessage *msg) {
 
   if (state_ == Finished) return;
 
-  GError *error;
-  gchar *debugs;
+  GError *error = nullptr;
+  gchar *debugs = nullptr;
 
   gst_message_parse_error(msg, &error, &debugs);
-  qLog(Error) << __PRETTY_FUNCTION__ << error->message;
-  qLog(Error) << __PRETTY_FUNCTION__ << debugs;
+  qLog(Error) << error->message;
+  qLog(Error) << debugs;
 
   QString message_str = error->message;
 
@@ -671,7 +714,25 @@ void SongLoader::StopTypefindAsync(bool success) {
   state_ = Finished;
   success_ = success;
 
-  metaObject()->invokeMethod(this, "StopTypefind", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(this, "StopTypefind", Qt::QueuedConnection);
 
 }
 #endif
+
+
+void SongLoader::ScheduleTimeoutAsync() {
+
+  if (QThread::currentThread() == thread()) {
+    ScheduleTimeout();
+  }
+  else {
+    QMetaObject::invokeMethod(this, "ScheduleTimeout", Qt::QueuedConnection);
+  }
+
+}
+
+void SongLoader::ScheduleTimeout() {
+
+  timeout_timer_->start(timeout_);
+
+}
