@@ -57,14 +57,15 @@ class UrlHandler : public QObject {
       Error,
     };
 
-    explicit LoadResult(const QUrl &original_url = QUrl(), const Type type = NoMoreTracks, const QUrl &stream_url = QUrl(), const Song::FileType filetype = Song::FileType_Stream, const int samplerate = -1, const int bit_depth = -1, const qint64 length_nanosec = -1, const QString error = QString());
+    explicit LoadResult(const QUrl &original_url = QUrl(), const Type type = NoMoreTracks, const QUrl &stream_url = QUrl(), const Song::FileType filetype = Song::FileType_Stream, const int samplerate = -1, const int bit_depth = -1, const qint64 length_nanosec = -1, const QString &error = QString());
 
-    explicit LoadResult(const QUrl &original_url, const Type type, const QString error);
+    explicit LoadResult(const QUrl &original_url, const Type type, const QString &error);
 
     // The url that the playlist item has in Url().
     // Might be something unplayable like lastfm://...
     QUrl original_url_;
 
+    // Result type
     Type type_;
 
     // The actual url to something that gstreamer can play.
