@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,31 +78,30 @@ const char *AlbumCoverChoiceController::kAllFilesFilter = QT_TR_NOOP("All files
 
 QSet<QString> *AlbumCoverChoiceController::sImageExtensions = nullptr;
 
-AlbumCoverChoiceController::AlbumCoverChoiceController(QWidget *parent) :
-    QWidget(parent),
-    app_(nullptr),
-    cover_searcher_(nullptr),
-    cover_fetcher_(nullptr),
-    save_file_dialog_(nullptr),
-    cover_from_url_dialog_(nullptr),
-    cover_from_file_(nullptr),
-    cover_to_file_(nullptr),
-    cover_from_url_(nullptr),
-    search_for_cover_(nullptr),
-    separator1_(nullptr),
-    unset_cover_(nullptr),
-    delete_cover_(nullptr),
-    clear_cover_(nullptr),
-    separator2_(nullptr),
-    show_cover_(nullptr),
-    search_cover_auto_(nullptr),
-    save_cover_type_(CollectionSettingsPage::SaveCoverType_Cache),
-    save_cover_filename_(CollectionSettingsPage::SaveCoverFilename_Hash),
-    cover_overwrite_(false),
-    cover_lowercase_(true),
-    cover_replace_spaces_(true),
-    save_embedded_cover_override_(false)
-    {
+AlbumCoverChoiceController::AlbumCoverChoiceController(QWidget *parent)
+    : QWidget(parent),
+      app_(nullptr),
+      cover_searcher_(nullptr),
+      cover_fetcher_(nullptr),
+      save_file_dialog_(nullptr),
+      cover_from_url_dialog_(nullptr),
+      cover_from_file_(nullptr),
+      cover_to_file_(nullptr),
+      cover_from_url_(nullptr),
+      search_for_cover_(nullptr),
+      separator1_(nullptr),
+      unset_cover_(nullptr),
+      delete_cover_(nullptr),
+      clear_cover_(nullptr),
+      separator2_(nullptr),
+      show_cover_(nullptr),
+      search_cover_auto_(nullptr),
+      save_cover_type_(CollectionSettingsPage::SaveCoverType_Cache),
+      save_cover_filename_(CollectionSettingsPage::SaveCoverFilename_Pattern),
+      cover_overwrite_(false),
+      cover_lowercase_(true),
+      cover_replace_spaces_(true),
+      save_embedded_cover_override_(false) {
 
   cover_from_file_ = new QAction(IconLoader::Load("document-open"), tr("Load cover from disk..."), this);
   cover_to_file_ = new QAction(IconLoader::Load("document-save"), tr("Save cover to disk..."), this);
@@ -126,7 +125,7 @@ AlbumCoverChoiceController::AlbumCoverChoiceController(QWidget *parent) :
 
 }
 
-AlbumCoverChoiceController::~AlbumCoverChoiceController() {}
+AlbumCoverChoiceController::~AlbumCoverChoiceController() = default;
 
 void AlbumCoverChoiceController::Init(Application *app) {
 
@@ -146,7 +145,7 @@ void AlbumCoverChoiceController::ReloadSettings() {
   QSettings s;
   s.beginGroup(CollectionSettingsPage::kSettingsGroup);
   save_cover_type_ = CollectionSettingsPage::SaveCoverType(s.value("save_cover_type", CollectionSettingsPage::SaveCoverType_Cache).toInt());
-  save_cover_filename_ = CollectionSettingsPage::SaveCoverFilename(s.value("save_cover_filename", CollectionSettingsPage::SaveCoverFilename_Hash).toInt());
+  save_cover_filename_ = CollectionSettingsPage::SaveCoverFilename(s.value("save_cover_filename", CollectionSettingsPage::SaveCoverFilename_Pattern).toInt());
   cover_pattern_ = s.value("cover_pattern", "%albumartist-%album").toString();
   cover_overwrite_ = s.value("cover_overwrite", false).toBool();
   cover_lowercase_ = s.value("cover_lowercase", false).toBool();
@@ -183,12 +182,20 @@ AlbumCoverImageResult AlbumCoverChoiceController::LoadImageFromFile(Song *song)
   if (file.open(QIODevice::ReadOnly)) {
     result.image_data = file.readAll();
     file.close();
-    if (!result.image_data.isEmpty()) {
+    if (result.image_data.isEmpty()) {
+      qLog(Error) << "Cover file" << cover_file << "is empty.";
+      emit Error(tr("Cover file %1 is empty.").arg(cover_file));
+    }
+    else {
       result.mime_type = Utilities::MimeTypeFromData(result.image_data);
       result.image.loadFromData(result.image_data);
       result.cover_url = QUrl::fromLocalFile(cover_file);
     }
   }
+  else {
+    qLog(Error) << "Failed to open cover file" << cover_file << "for reading:" << file.errorString();
+    emit Error(tr("Failed to open cover file %1 for reading: %2").arg(cover_file, file.errorString()));
+  }
 
   return result;
 
@@ -205,7 +212,7 @@ QUrl AlbumCoverChoiceController::LoadCoverFromFile(Song *song) {
   if (QImage(cover_file).isNull()) return QUrl();
 
   switch(get_save_album_cover_type()) {
-      case CollectionSettingsPage::SaveCoverType_Embedded:
+    case CollectionSettingsPage::SaveCoverType_Embedded:
       if (song->save_embedded_cover_supported()) {
         SaveCoverEmbeddedAutomatic(*song, cover_file);
         return QUrl::fromLocalFile(Song::kEmbeddedCover);
@@ -216,7 +223,6 @@ QUrl AlbumCoverChoiceController::LoadCoverFromFile(Song *song) {
       QUrl cover_url = QUrl::fromLocalFile(cover_file);
       SaveArtManualToSong(song, cover_url);
       return cover_url;
-      break;
     }
   }
 
@@ -251,15 +257,25 @@ void AlbumCoverChoiceController::SaveCoverToFileManual(const Song &song, const A
     fileinfo.setFile(save_filename);
   }
 
-  if (result.is_jpeg() && fileinfo.completeSuffix().toLower() == "jpg") {
+  if (result.is_jpeg() && fileinfo.completeSuffix().compare("jpg", Qt::CaseInsensitive) == 0) {
     QFile file(save_filename);
     if (file.open(QIODevice::WriteOnly)) {
-      file.write(result.image_data);
+      if (file.write(result.image_data) <= 0) {
+        qLog(Error) << "Failed writing cover to file" << save_filename << file.errorString();
+        emit Error(tr("Failed writing cover to file %1: %2").arg(save_filename, file.errorString()));
+      }
       file.close();
     }
+    else {
+      qLog(Error) << "Failed to open cover file" << save_filename << "for writing:" << file.errorString();
+      emit Error(tr("Failed to open cover file %1 for writing: %2").arg(save_filename, file.errorString()));
+    }
   }
   else {
-    result.image.save(save_filename);
+    if (!result.image.save(save_filename)) {
+      qLog(Error) << "Failed writing cover to file" << save_filename;
+      emit Error(tr("Failed writing cover to file %1.").arg(save_filename));
+    }
   }
 
 }
@@ -377,24 +393,34 @@ bool AlbumCoverChoiceController::DeleteCover(Song *song, const bool manually_uns
   bool success = true;
 
   if (!art_automatic.isEmpty()) {
-    if (QFile::exists(art_automatic)) {
-      if (QFile::remove(art_automatic)) {
+    QFile file(art_automatic);
+    if (file.exists()) {
+      if (file.remove()) {
         song->clear_art_automatic();
         if (art_automatic == art_manual) song->clear_art_manual();
       }
-      else success = false;
+      else {
+        success = false;
+        qLog(Error) << "Failed to delete cover file" << art_automatic << file.errorString();
+        emit Error(tr("Failed to delete cover file %1: %2").arg(art_automatic, file.errorString()));
+      }
     }
     else song->clear_art_automatic();
   }
   else song->clear_art_automatic();
 
   if (!art_manual.isEmpty()) {
-    if (QFile::exists(art_manual)) {
-      if (QFile::remove(art_manual)) {
+    QFile file(art_manual);
+    if (file.exists()) {
+      if (file.remove()) {
         song->clear_art_manual();
         if (art_automatic == art_manual) song->clear_art_automatic();
       }
-      else success = false;
+      else {
+        success = false;
+        qLog(Error) << "Failed to delete cover file" << art_manual << file.errorString();
+        emit Error(tr("Failed to delete cover file %1: %2").arg(art_manual, file.errorString()));
+      }
     }
     else song->clear_art_manual();
   }
@@ -454,13 +480,13 @@ void AlbumCoverChoiceController::ShowCover(const Song &song, const QPixmap &pixm
 
   // Resize differently if monitor is in portrait mode
   if (desktop_width < desktop_height) {
-    const int new_width = static_cast<double>(desktop_width) * 0.95;
+    const int new_width = static_cast<int>(static_cast<double>(desktop_width) * 0.95);
     if (new_width < pixmap.width()) {
       label->setPixmap(pixmap.scaledToWidth(new_width, Qt::SmoothTransformation));
     }
   }
   else {
-    const int new_height = static_cast<double>(desktop_height) * 0.85;
+    const int new_height = static_cast<int>(static_cast<double>(desktop_height) * 0.85);
     if (new_height < pixmap.height()) {
       label->setPixmap(pixmap.scaledToHeight(new_height, Qt::SmoothTransformation));
     }
@@ -476,9 +502,9 @@ void AlbumCoverChoiceController::ShowCover(const Song &song, const QPixmap &pixm
 
 }
 
-qint64 AlbumCoverChoiceController::SearchCoverAutomatically(const Song &song) {
+quint64 AlbumCoverChoiceController::SearchCoverAutomatically(const Song &song) {
 
-  qint64 id = cover_fetcher_->FetchAlbumCover(song.effective_albumartist(), song.album(), song.title(), true);
+  quint64 id = cover_fetcher_->FetchAlbumCover(song.effective_albumartist(), song.album(), song.title(), true);
 
   cover_fetching_tasks_[id] = song;
 
@@ -508,9 +534,12 @@ void AlbumCoverChoiceController::SaveArtAutomaticToSong(Song *song, const QUrl &
   if (!song->is_valid()) return;
 
   song->set_art_automatic(art_automatic);
+  if (song->has_embedded_cover()) {
+    song->clear_art_manual();
+  }
 
   if (song->source() == Song::Source_Collection) {
-    app_->collection_backend()->UpdateAutomaticAlbumArtAsync(song->effective_albumartist(), song->album(), art_automatic);
+    app_->collection_backend()->UpdateAutomaticAlbumArtAsync(song->effective_albumartist(), song->album(), art_automatic, song->has_embedded_cover());
   }
 
   if (*song == app_->current_albumcover_loader()->last_song()) {
@@ -535,6 +564,8 @@ void AlbumCoverChoiceController::SaveArtManualToSong(Song *song, const QUrl &art
     case Song::Source_CDDA:
     case Song::Source_Device:
     case Song::Source_Stream:
+    case Song::Source_RadioParadise:
+    case Song::Source_SomaFM:
     case Song::Source_Unknown:
       break;
     case Song::Source_Tidal:
@@ -596,9 +627,19 @@ QUrl AlbumCoverChoiceController::SaveCoverToFileAutomatic(const Song::Source sou
   QUrl cover_url;
   if (result.is_jpeg()) {
     if (file.open(QIODevice::WriteOnly)) {
-      if (file.write(result.image_data)) cover_url = QUrl::fromLocalFile(filepath);
+      if (file.write(result.image_data) > 0) {
+        cover_url = QUrl::fromLocalFile(filepath);
+      }
+      else {
+        qLog(Error) << "Failed to write cover to file" << file.fileName() << file.errorString();
+        emit Error(tr("Failed to write cover to file %1: %2").arg(file.fileName(), file.errorString()));
+      }
       file.close();
     }
+    else {
+      qLog(Error) << "Failed to open cover file" << file.fileName() << "for writing:" << file.errorString();
+      emit Error(tr("Failed to open cover file %1 for writing: %2").arg(file.fileName(), file.errorString()));
+    }
   }
   else {
     if (result.image.save(filepath, "JPG")) cover_url = QUrl::fromLocalFile(filepath);
@@ -617,23 +658,24 @@ void AlbumCoverChoiceController::SaveCoverEmbeddedAutomatic(const Song &song, co
     QFuture<SongList> future = QtConcurrent::run(app_->collection_backend(), &CollectionBackend::GetAlbumSongs, song.effective_albumartist(), song.effective_album(), QueryOptions());
 #endif
     QFutureWatcher<SongList> *watcher = new QFutureWatcher<SongList>();
-    watcher->setFuture(future);
     QObject::connect(watcher, &QFutureWatcher<SongList>::finished, this, [=]() {
       SongList songs = watcher->result();
       watcher->deleteLater();
       QList<QUrl> urls;
+      urls.reserve(songs.count());
       for (const Song &s : songs) urls << s.url();
       if (result.is_jpeg()) {
-        qint64 id = app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, result.image_data);
+        quint64 id = app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, result.image_data);
         QMutexLocker l(&mutex_cover_save_tasks_);
         cover_save_tasks_.insert(id, song);
       }
       else {
-        qint64 id = app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, result.image);
+        quint64 id = app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, result.image);
         QMutexLocker l(&mutex_cover_save_tasks_);
         cover_save_tasks_.insert(id, song);
       }
     });
+    watcher->setFuture(future);
   }
   else {
     if (result.is_jpeg()) {
@@ -661,16 +703,17 @@ void AlbumCoverChoiceController::SaveCoverEmbeddedAutomatic(const Song &song, co
     QFuture<SongList> future = QtConcurrent::run(app_->collection_backend(), &CollectionBackend::GetAlbumSongs, song.effective_albumartist(), song.effective_album(), QueryOptions());
 #endif
     QFutureWatcher<SongList> *watcher = new QFutureWatcher<SongList>();
-    watcher->setFuture(future);
     QObject::connect(watcher, &QFutureWatcher<SongList>::finished, this, [=]() {
       SongList songs = watcher->result();
       watcher->deleteLater();
       QList<QUrl> urls;
+      urls.reserve(songs.count());
       for (const Song &s : songs) urls << s.url();
-      qint64 id = app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, cover_filename);
+      quint64 id = app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, cover_filename);
       QMutexLocker l(&mutex_cover_save_tasks_);
       cover_save_tasks_.insert(id, song);
     });
+    watcher->setFuture(future);
   }
   else {
     app_->album_cover_loader()->SaveEmbeddedCoverAsync(song.url().toLocalFile(), cover_filename);
@@ -678,7 +721,7 @@ void AlbumCoverChoiceController::SaveCoverEmbeddedAutomatic(const Song &song, co
 
 }
 
-void AlbumCoverChoiceController::SaveCoverEmbeddedAutomatic(const QList<QUrl> urls, const QImage &image) {
+void AlbumCoverChoiceController::SaveCoverEmbeddedAutomatic(const QList<QUrl> &urls, const QImage &image) {
 
   app_->album_cover_loader()->SaveEmbeddedCoverAsync(urls, image);
 
