@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <QList>
 #include <QMap>
 #include <QMultiMap>
+#include <QHash>
 #include <QString>
 #include <QUrl>
 #include <QImage>
@@ -73,7 +74,8 @@ AlbumCoverFetcherSearch::~AlbumCoverFetcherSearch() {
 
 void AlbumCoverFetcherSearch::TerminateSearch() {
 
-  for (quint64 id : pending_requests_.keys()) {
+  QList<int> ids = pending_requests_.keys();
+  for (const int id : ids) {
     pending_requests_.take(id)->CancelSearch(id);
   }
 
@@ -84,7 +86,7 @@ void AlbumCoverFetcherSearch::TerminateSearch() {
 void AlbumCoverFetcherSearch::Start(CoverProviders *cover_providers) {
 
   // Ignore Radio Paradise "commercial" break.
-  if (request_.artist.toLower() == "commercial-free" && request_.title.toLower() == "listener-supported") {
+  if (request_.artist.compare("commercial-free", Qt::CaseInsensitive) == 0 && request_.title.compare("listener-supported", Qt::CaseInsensitive) == 0) {
     TerminateSearch();
     return;
   }
@@ -140,27 +142,27 @@ void AlbumCoverFetcherSearch::ProviderSearchResults(const int id, const CoverPro
 void AlbumCoverFetcherSearch::ProviderSearchResults(CoverProvider *provider, const CoverProviderSearchResults &results) {
 
   CoverProviderSearchResults results_copy(results);
-  for (int i = 0 ; i < results_copy.count() ; ++i) {
+  for (int i = 0; i < results_copy.count(); ++i) {
 
     results_copy[i].provider = provider->name();
     results_copy[i].score_provider = provider->quality();
 
-    QString request_artist = request_.artist.toLower();
-    QString request_album = request_.album.toLower();
-    QString result_artist = results_copy[i].artist.toLower();
-    QString result_album = results_copy[i].album.toLower();
+    const QString &request_artist = request_.artist;
+    const QString &request_album = request_.album;
+    const QString &result_artist = results_copy[i].artist;
+    const QString &result_album = results_copy[i].album;
 
-    if (result_artist == request_artist) {
+    if (result_artist.compare(request_artist, Qt::CaseInsensitive) == 0) {
       results_copy[i].score_match += 0.5;
     }
-    if (result_album == request_album) {
+    if (result_album.compare(request_album, Qt::CaseInsensitive) == 0) {
       results_copy[i].score_match += 0.5;
     }
-    if (result_artist != request_artist && result_album != request_album) {
+    if (result_artist.compare(request_artist, Qt::CaseInsensitive) != 0 && result_album.compare(request_album, Qt::CaseInsensitive) != 0) {
       results_copy[i].score_match -= 1.5;
     }
 
-    if (request_album.isEmpty() && result_artist != request_artist) {
+    if (request_album.isEmpty() && result_artist.compare(request_artist, Qt::CaseInsensitive) != 0) {
       results_copy[i].score_match -= 1;
     }
 
@@ -168,55 +170,55 @@ void AlbumCoverFetcherSearch::ProviderSearchResults(CoverProvider *provider, con
     // This is done since we can't match the album titles, and we want to prevent compilation or live albums from being picked before studio albums for streams.
     // TODO: Make these regular expressions.
     if (request_album.isEmpty() && (
-        result_album.contains("hits") ||
-        result_album.contains("greatest") ||
-        result_album.contains("best") ||
-        result_album.contains("collection") ||
-        result_album.contains("classics") ||
-        result_album.contains("singles") ||
-        result_album.contains("bootleg") ||
-        result_album.contains("live") ||
-        result_album.contains("concert") ||
-        result_album.contains("essential") ||
-        result_album.contains("ultimate") ||
-        result_album.contains("karaoke") ||
-        result_album.contains("mixtape") ||
-        result_album.contains("country rock") ||
-        result_album.contains("indie folk") ||
-        result_album.contains("soft rock") ||
-        result_album.contains("folk music") ||
-        result_album.contains("60's rock") ||
-        result_album.contains("60's romance") ||
-        result_album.contains("60s music") ||
-        result_album.contains("late 60s") ||
-        result_album.contains("the 60s") ||
-        result_album.contains("folk and blues") ||
-        result_album.contains("60 from the 60's") ||
-        result_album.contains("classic psychedelic") ||
-        result_album.contains("playlist: acoustic") ||
-        result_album.contains("90's rnb playlist") ||
-        result_album.contains("rock 80s") ||
-        result_album.contains("classic 80s") ||
-        result_album.contains("rock anthems") ||
-        result_album.contains("rock songs") ||
-        result_album.contains("rock 2019") ||
-        result_album.contains("guitar anthems") ||
-        result_album.contains("driving anthems") ||
-        result_album.contains("traffic jam jams") ||
-        result_album.contains("perfect background music") ||
-        result_album.contains("70's gold") ||
-        result_album.contains("rockfluence") ||
-        result_album.contains("acoustic dinner accompaniment") ||
-        result_album.contains("complete studio albums") ||
-        result_album.contains("mellow rock")
+        result_album.contains("hits", Qt::CaseInsensitive) ||
+        result_album.contains("greatest", Qt::CaseInsensitive) ||
+        result_album.contains("best", Qt::CaseInsensitive) ||
+        result_album.contains("collection", Qt::CaseInsensitive) ||
+        result_album.contains("classics", Qt::CaseInsensitive) ||
+        result_album.contains("singles", Qt::CaseInsensitive) ||
+        result_album.contains("bootleg", Qt::CaseInsensitive) ||
+        result_album.contains("live", Qt::CaseInsensitive) ||
+        result_album.contains("concert", Qt::CaseInsensitive) ||
+        result_album.contains("essential", Qt::CaseInsensitive) ||
+        result_album.contains("ultimate", Qt::CaseInsensitive) ||
+        result_album.contains("karaoke", Qt::CaseInsensitive) ||
+        result_album.contains("mixtape", Qt::CaseInsensitive) ||
+        result_album.contains("country rock", Qt::CaseInsensitive) ||
+        result_album.contains("indie folk", Qt::CaseInsensitive) ||
+        result_album.contains("soft rock", Qt::CaseInsensitive) ||
+        result_album.contains("folk music", Qt::CaseInsensitive) ||
+        result_album.contains("60's rock", Qt::CaseInsensitive) ||
+        result_album.contains("60's romance", Qt::CaseInsensitive) ||
+        result_album.contains("60s music", Qt::CaseInsensitive) ||
+        result_album.contains("late 60s", Qt::CaseInsensitive) ||
+        result_album.contains("the 60s", Qt::CaseInsensitive) ||
+        result_album.contains("folk and blues", Qt::CaseInsensitive) ||
+        result_album.contains("60 from the 60's", Qt::CaseInsensitive) ||
+        result_album.contains("classic psychedelic", Qt::CaseInsensitive) ||
+        result_album.contains("playlist: acoustic", Qt::CaseInsensitive) ||
+        result_album.contains("90's rnb playlist", Qt::CaseInsensitive) ||
+        result_album.contains("rock 80s", Qt::CaseInsensitive) ||
+        result_album.contains("classic 80s", Qt::CaseInsensitive) ||
+        result_album.contains("rock anthems", Qt::CaseInsensitive) ||
+        result_album.contains("rock songs", Qt::CaseInsensitive) ||
+        result_album.contains("rock 2019", Qt::CaseInsensitive) ||
+        result_album.contains("guitar anthems", Qt::CaseInsensitive) ||
+        result_album.contains("driving anthems", Qt::CaseInsensitive) ||
+        result_album.contains("traffic jam jams", Qt::CaseInsensitive) ||
+        result_album.contains("perfect background music", Qt::CaseInsensitive) ||
+        result_album.contains("70's gold", Qt::CaseInsensitive) ||
+        result_album.contains("rockfluence", Qt::CaseInsensitive) ||
+        result_album.contains("acoustic dinner accompaniment", Qt::CaseInsensitive) ||
+        result_album.contains("complete studio albums", Qt::CaseInsensitive) ||
+        result_album.contains("mellow rock", Qt::CaseInsensitive)
         )) {
       results_copy[i].score_match -= 1;
     }
-    else if (request_album.isEmpty() && result_album.contains("soundtrack")) {
+    else if (request_album.isEmpty() && result_album.contains("soundtrack", Qt::CaseInsensitive)) {
       results_copy[i].score_match -= 0.5;
     }
 
-    // Set the initial image quality score besed on the size returned by the API, this is recalculated when the image is received.
+    // Set the initial image quality score based on the size returned by the API, this is recalculated when the image is received.
     results_copy[i].score_quality += ScoreImage(results_copy[i].image_size);
 
   }
@@ -288,7 +290,7 @@ void AlbumCoverFetcherSearch::FetchMoreImages() {
     req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
 #endif
     QNetworkReply *image_reply = network_->get(req);
-    QObject::connect(image_reply, &QNetworkReply::finished, [this, image_reply]() { ProviderCoverFetchFinished(image_reply); });
+    QObject::connect(image_reply, &QNetworkReply::finished, this, [this, image_reply]() { ProviderCoverFetchFinished(image_reply); });
     pending_image_loads_[image_reply] = result;
     image_load_timeout_->AddReply(image_reply);
 
@@ -325,6 +327,9 @@ void AlbumCoverFetcherSearch::ProviderCoverFetchFinished(QNetworkReply *reply) {
   }
   else {
     QString mimetype = reply->header(QNetworkRequest::ContentTypeHeader).toString();
+    if (mimetype.contains(';')) {
+      mimetype = mimetype.left(mimetype.indexOf(';'));
+    }
     if (ImageUtils::SupportedImageMimeTypes().contains(mimetype, Qt::CaseInsensitive) || ImageUtils::SupportedImageFormats().contains(mimetype, Qt::CaseInsensitive)) {
       QByteArray image_data = reply->readAll();
       QString mime_type = Utilities::MimeTypeFromData(image_data);
@@ -352,7 +357,8 @@ void AlbumCoverFetcherSearch::ProviderCoverFetchFinished(QNetworkReply *reply) {
     float best_score = 0.0;
 
     if (!candidate_images_.isEmpty()) {
-      best_score = candidate_images_.keys().last();
+      QList<float> scores = candidate_images_.keys();
+      best_score = scores.last();
       qLog(Debug) << "Best image so far has a score of" << best_score;
     }
 
@@ -366,15 +372,15 @@ void AlbumCoverFetcherSearch::ProviderCoverFetchFinished(QNetworkReply *reply) {
 
 }
 
-float AlbumCoverFetcherSearch::ScoreImage(const QSize size) const {
+float AlbumCoverFetcherSearch::ScoreImage(const QSize size) {
 
   if (size.width() == 0 || size.height() == 0) return 0.0;
 
   // A 500x500px image scores 1.0, bigger scores higher
-  const float size_score = std::sqrt(float(size.width() * size.height())) / kTargetSize;
+  const float size_score = std::sqrt(static_cast<float>(size.width() * size.height())) / kTargetSize;
 
   // A 1:1 image scores 1.0, anything else scores less
-  const float aspect_score = 1.0 - float(std::max(size.width(), size.height()) - std::min(size.width(), size.height())) / std::max(size.height(), size.width());
+  const float aspect_score = static_cast<float>(1.0) - static_cast<float>(std::max(size.width(), size.height()) - std::min(size.width(), size.height())) / static_cast<float>(std::max(size.height(), size.width()));
 
   return size_score + aspect_score;
 
@@ -385,7 +391,8 @@ void AlbumCoverFetcherSearch::SendBestImage() {
   AlbumCoverImageResult result;
 
   if (!candidate_images_.isEmpty()) {
-    const CandidateImage best_image = candidate_images_.values().back();
+    QList<CandidateImage> candidate_images = candidate_images_.values();
+    const CandidateImage best_image = candidate_images.back();
     result = best_image.album_cover;
 
     qLog(Info) << "Using" << best_image.result.image_url << "from" << best_image.result.provider << "with score" << best_image.result.score();
@@ -411,7 +418,8 @@ void AlbumCoverFetcherSearch::Cancel() {
     TerminateSearch();
   }
   else if (!pending_image_loads_.isEmpty()) {
-    for (QNetworkReply *reply : pending_image_loads_.keys()) {
+    QList<QNetworkReply*> replies = pending_image_loads_.keys();
+    for (QNetworkReply *reply : replies) {
       QObject::disconnect(reply, &QNetworkReply::finished, this, nullptr);
       reply->abort();
       reply->deleteLater();
