@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <QString>
 #include <QRegularExpression>
 #include <QUrl>
+#include <QFile>
 #include <QImage>
 #include <QPixmap>
 #include <QPainter>
@@ -63,12 +64,11 @@ AlbumCoverLoader::AlbumCoverLoader(QObject *parent)
       save_image_async_id_(1),
       network_(new NetworkAccessManager(this)),
       save_cover_type_(CollectionSettingsPage::SaveCoverType_Cache),
-      save_cover_filename_(CollectionSettingsPage::SaveCoverFilename_Hash),
+      save_cover_filename_(CollectionSettingsPage::SaveCoverFilename_Pattern),
       cover_overwrite_(false),
       cover_lowercase_(true),
       cover_replace_spaces_(true),
-      original_thread_(nullptr)
-      {
+      original_thread_(nullptr) {
 
   original_thread_ = thread();
   ReloadSettings();
@@ -78,13 +78,13 @@ AlbumCoverLoader::AlbumCoverLoader(QObject *parent)
 void AlbumCoverLoader::ExitAsync() {
 
   stop_requested_ = true;
-  metaObject()->invokeMethod(this, "Exit", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(this, "Exit", Qt::QueuedConnection);
 
 }
 
 void AlbumCoverLoader::Exit() {
 
-  assert(QThread::currentThread() == thread());
+  Q_ASSERT(QThread::currentThread() == thread());
   moveToThread(original_thread_);
   emit ExitFinished();
 
@@ -95,7 +95,7 @@ void AlbumCoverLoader::ReloadSettings() {
   QSettings s;
   s.beginGroup(CollectionSettingsPage::kSettingsGroup);
   save_cover_type_ = CollectionSettingsPage::SaveCoverType(s.value("save_cover_type", CollectionSettingsPage::SaveCoverType_Cache).toInt());
-  save_cover_filename_ = CollectionSettingsPage::SaveCoverFilename(s.value("save_cover_filename", CollectionSettingsPage::SaveCoverFilename_Hash).toInt());
+  save_cover_filename_ = CollectionSettingsPage::SaveCoverFilename(s.value("save_cover_filename", CollectionSettingsPage::SaveCoverFilename_Pattern).toInt());
   cover_pattern_ = s.value("cover_pattern", "%albumartist-%album").toString();
   cover_overwrite_ = s.value("cover_overwrite", false).toBool();
   cover_lowercase_ = s.value("cover_lowercase", false).toBool();
@@ -199,6 +199,8 @@ QString AlbumCoverLoader::CoverFilenameFromSource(const Song::Source source, con
     case Song::Source_CDDA:
     case Song::Source_Device:
     case Song::Source_Stream:
+    case Song::Source_SomaFM:
+    case Song::Source_RadioParadise:
     case Song::Source_Unknown:
       filename = Utilities::Sha1CoverHash(artist, album).toHex();
       break;
@@ -232,7 +234,7 @@ void AlbumCoverLoader::CancelTask(const quint64 id) {
   QMutexLocker l(&mutex_load_image_async_);
   for (QQueue<Task>::iterator it = tasks_.begin(); it != tasks_.end(); ++it) {
     if (it->id == id) {
-      tasks_.erase(it);
+      tasks_.erase(it);  // clazy:exclude=strict-iterators
       break;
     }
   }
@@ -244,7 +246,7 @@ void AlbumCoverLoader::CancelTasks(const QSet<quint64> &ids) {
   QMutexLocker l(&mutex_load_image_async_);
   for (QQueue<Task>::iterator it = tasks_.begin(); it != tasks_.end();) {
     if (ids.contains(it->id)) {
-      it = tasks_.erase(it);
+      it = tasks_.erase(it);  // clazy:exclude=strict-iterators
     }
     else {
       ++it;
@@ -308,7 +310,7 @@ quint64 AlbumCoverLoader::EnqueueTask(Task &task) {
     tasks_.enqueue(task);
   }
 
-  metaObject()->invokeMethod(this, "ProcessTasks", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(this, "ProcessTasks", Qt::QueuedConnection);
 
   return task.id;
 
@@ -378,7 +380,7 @@ AlbumCoverLoader::TryLoadResult AlbumCoverLoader::TryLoadImage(Task *task) {
   }
 
   // For local files and streams initialize art if found.
-  if ((task->song.source() == Song::Source_LocalFile || task->song.source() == Song::Source_Stream) && !task->song.art_manual_is_valid() && !task->song.art_automatic_is_valid()) {
+  if ((task->song.source() == Song::Source_LocalFile || task->song.is_radio()) && !task->song.art_manual_is_valid() && !task->song.art_automatic_is_valid()) {
     switch (task->state) {
       case State_None:
         break;
@@ -418,11 +420,11 @@ AlbumCoverLoader::TryLoadResult AlbumCoverLoader::TryLoadImage(Task *task) {
       QByteArray image_data = TagReaderClient::Instance()->LoadEmbeddedArtBlocking(task->song.url().toLocalFile());
       if (!image_data.isEmpty()) {
         QImage image;
-        if (task->options.get_image_ && image.loadFromData(image_data)) {
-          return TryLoadResult(false, true, AlbumCoverLoaderResult::Type_Embedded, AlbumCoverImageResult(cover_url, QString(), image_data, image));
+        if (!image_data.isEmpty() && task->options.get_image_ && image.loadFromData(image_data)) {
+          return TryLoadResult(false, !image.isNull(), AlbumCoverLoaderResult::Type_Embedded, AlbumCoverImageResult(cover_url, QString(), image_data, image));
         }
         else {
-          return TryLoadResult(false, true, AlbumCoverLoaderResult::Type_Embedded, AlbumCoverImageResult(cover_url, QString(), image_data, image));
+          return TryLoadResult(false, !image_data.isEmpty(), AlbumCoverLoaderResult::Type_Embedded, AlbumCoverImageResult(cover_url, QString(), image_data, image));
         }
       }
     }
@@ -438,27 +440,38 @@ AlbumCoverLoader::TryLoadResult AlbumCoverLoader::TryLoadImage(Task *task) {
             return TryLoadResult(false, !image.isNull(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
           }
           else {
-            return TryLoadResult(false, !image.isNull(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
+            return TryLoadResult(false, !image_data.isEmpty(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
           }
         }
         else {
-          qLog(Error) << "Failed to open album cover file" << cover_url;
+          qLog(Error) << "Failed to open cover file" << cover_url << "for reading" << file.errorString();
         }
       }
+      else {
+        qLog(Error) << "Cover file" << cover_url << "does not exist";
+      }
     }
     else if (cover_url.scheme().isEmpty()) {  // Assume a local file with no scheme.
       QFile file(cover_url.path());
-      if (file.exists() && file.open(QIODevice::ReadOnly)) {
-        QByteArray image_data = file.readAll();
-        file.close();
-        QImage image;
-        if (!image_data.isEmpty() && task->options.get_image_ && image.loadFromData(image_data)) {
-          return TryLoadResult(false, !image.isNull(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
+      if (file.exists()) {
+        if (file.open(QIODevice::ReadOnly)) {
+          QByteArray image_data = file.readAll();
+          file.close();
+          QImage image;
+          if (!image_data.isEmpty() && task->options.get_image_ && image.loadFromData(image_data)) {
+            return TryLoadResult(false, !image.isNull(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
+          }
+          else {
+            return TryLoadResult(false, !image_data.isEmpty(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
+          }
         }
         else {
-          return TryLoadResult(false, !image.isNull(), type, AlbumCoverImageResult(cover_url, QString(), image_data, image.isNull() ? task->options.default_output_image_ : image));
+          qLog(Error) << "Failed to open cover file" << cover_url << "for reading" << file.errorString();
         }
       }
+      else {
+        qLog(Error) << "Cover file" << cover_url << "does not exist";
+      }
     }
     else if (network_->supportedSchemes().contains(cover_url.scheme())) {  // Remote URL
       qLog(Debug) << "Loading remote cover from" << cover_url;
@@ -469,7 +482,7 @@ AlbumCoverLoader::TryLoadResult AlbumCoverLoader::TryLoadImage(Task *task) {
       request.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
 #endif
       QNetworkReply *reply = network_->get(request);
-      QObject::connect(reply, &QNetworkReply::finished, [this, reply, cover_url]() { RemoteFetchFinished(reply, cover_url); });
+      QObject::connect(reply, &QNetworkReply::finished, this, [this, reply, cover_url]() { RemoteFetchFinished(reply, cover_url); });
 
       remote_tasks_.insert(reply, *task);
       return TryLoadResult(true, false, type, AlbumCoverImageResult(cover_url));
@@ -501,7 +514,7 @@ void AlbumCoverLoader::RemoteFetchFinished(QNetworkReply *reply, const QUrl &cov
 #endif
     request.setUrl(redirect.toUrl());
     QNetworkReply *redirected_reply = network_->get(request);
-    QObject::connect(redirected_reply, &QNetworkReply::finished, [this, redirected_reply, redirect]() { RemoteFetchFinished(redirected_reply, redirect.toUrl()); });
+    QObject::connect(redirected_reply, &QNetworkReply::finished, this, [this, redirected_reply, redirect]() { RemoteFetchFinished(redirected_reply, redirect.toUrl()); });
 
     remote_tasks_.insert(redirected_reply, task);
     return;
@@ -512,21 +525,16 @@ void AlbumCoverLoader::RemoteFetchFinished(QNetworkReply *reply, const QUrl &cov
     QByteArray image_data = reply->readAll();
     QString mime_type = Utilities::MimeTypeFromData(image_data);
     QImage image;
-    if (task.options.get_image_data_) {
-      if (image.loadFromData(image_data)) {
-        QImage image_scaled;
-        QImage image_thumbnail;
-        if (task.options.scale_output_image_) image_scaled = ImageUtils::ScaleAndPad(image, task.options.scale_output_image_, task.options.pad_output_image_, task.options.desired_height_);
-        if (task.options.create_thumbnail_) image_thumbnail =  ImageUtils::CreateThumbnail(image, task.options.pad_thumbnail_image_, task.options.thumbnail_size_);
-        emit AlbumCoverLoaded(task.id, AlbumCoverLoaderResult(true, task.type, AlbumCoverImageResult(cover_url, mime_type, (task.options.get_image_data_ ? image_data : QByteArray()), image), image_scaled, image_thumbnail, task.art_updated));
-        return;
-      }
-      else {
-        qLog(Error) << "Unable to load album cover image" << cover_url;
-      }
+    if (image.loadFromData(image_data)) {
+      QImage image_scaled;
+      QImage image_thumbnail;
+      if (task.options.scale_output_image_) image_scaled = ImageUtils::ScaleAndPad(image, task.options.scale_output_image_, task.options.pad_output_image_, task.options.desired_height_);
+      if (task.options.create_thumbnail_) image_thumbnail = ImageUtils::CreateThumbnail(image, task.options.pad_thumbnail_image_, task.options.thumbnail_size_);
+      emit AlbumCoverLoaded(task.id, AlbumCoverLoaderResult(true, task.type, AlbumCoverImageResult(cover_url, mime_type, (task.options.get_image_data_ ? image_data : QByteArray()), image), image_scaled, image_thumbnail, task.art_updated));
+      return;
     }
     else {
-      emit AlbumCoverLoaded(task.id, AlbumCoverLoaderResult(true, task.type, AlbumCoverImageResult(cover_url, mime_type, image_data, QImage()), QImage(), QImage(), task.art_updated));
+      qLog(Error) << "Unable to load album cover image" << cover_url;
     }
   }
   else {
@@ -537,61 +545,61 @@ void AlbumCoverLoader::RemoteFetchFinished(QNetworkReply *reply, const QUrl &cov
 
 }
 
-qint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QString song_filename, const QString &cover_filename) {
+quint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QString &song_filename, const QString &cover_filename) {
 
   QMutexLocker l(&mutex_save_image_async_);
-  qint64 id = ++save_image_async_id_;
-  metaObject()->invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(qint64, id), Q_ARG(QString, song_filename), Q_ARG(QString, cover_filename));
+  quint64 id = ++save_image_async_id_;
+  QMetaObject::invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(quint64, id), Q_ARG(QString, song_filename), Q_ARG(QString, cover_filename));
   return id;
 
 }
 
-qint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QString song_filename, const QImage &image) {
+quint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QString &song_filename, const QImage &image) {
 
   QMutexLocker l(&mutex_save_image_async_);
-  qint64 id = ++save_image_async_id_;
-  metaObject()->invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(qint64, id), Q_ARG(QString, song_filename), Q_ARG(QImage, image));
+  quint64 id = ++save_image_async_id_;
+  QMetaObject::invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(quint64, id), Q_ARG(QString, song_filename), Q_ARG(QImage, image));
   return id;
 
 }
 
-qint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QString song_filename, const QByteArray &image_data) {
+quint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QString &song_filename, const QByteArray &image_data) {
 
   QMutexLocker l(&mutex_save_image_async_);
-  qint64 id = ++save_image_async_id_;
-  metaObject()->invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(qint64, id), Q_ARG(QString, song_filename), Q_ARG(QByteArray, image_data));
+  quint64 id = ++save_image_async_id_;
+  QMetaObject::invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(quint64, id), Q_ARG(QString, song_filename), Q_ARG(QByteArray, image_data));
   return id;
 
 }
 
-qint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QList<QUrl> urls, const QString &cover_filename) {
+quint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QList<QUrl> &urls, const QString &cover_filename) {
 
   QMutexLocker l(&mutex_save_image_async_);
-  qint64 id = ++save_image_async_id_;
-  metaObject()->invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(qint64, id), Q_ARG(QList<QUrl>, urls), Q_ARG(QString, cover_filename));
+  quint64 id = ++save_image_async_id_;
+  QMetaObject::invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(quint64, id), Q_ARG(QList<QUrl>, urls), Q_ARG(QString, cover_filename));
   return id;
 
 }
 
-qint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QList<QUrl> urls, const QImage &image) {
+quint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QList<QUrl> &urls, const QImage &image) {
 
   QMutexLocker l(&mutex_save_image_async_);
-  qint64 id = ++save_image_async_id_;
-  metaObject()->invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(qint64, id), Q_ARG(QList<QUrl>, urls), Q_ARG(QImage, image));
+  quint64 id = ++save_image_async_id_;
+  QMetaObject::invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(quint64, id), Q_ARG(QList<QUrl>, urls), Q_ARG(QImage, image));
   return id;
 
 }
 
-qint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QList<QUrl> urls, const QByteArray &image_data) {
+quint64 AlbumCoverLoader::SaveEmbeddedCoverAsync(const QList<QUrl> &urls, const QByteArray &image_data) {
 
   QMutexLocker l(&mutex_save_image_async_);
-  qint64 id = ++save_image_async_id_;
-  metaObject()->invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(qint64, id), Q_ARG(QList<QUrl>, urls), Q_ARG(QByteArray, image_data));
+  quint64 id = ++save_image_async_id_;
+  QMetaObject::invokeMethod(this, "SaveEmbeddedCover", Qt::QueuedConnection, Q_ARG(quint64, id), Q_ARG(QList<QUrl>, urls), Q_ARG(QByteArray, image_data));
   return id;
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QString song_filename, const QByteArray &image_data) {
+void AlbumCoverLoader::SaveEmbeddedCover(const quint64 id, const QString &song_filename, const QByteArray &image_data) {
 
   TagReaderReply *reply = TagReaderClient::Instance()->SaveEmbeddedArt(song_filename, image_data);
   tagreader_save_embedded_art_requests_.insert(id, reply);
@@ -600,7 +608,7 @@ void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QString song_fil
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QString song_filename, const QImage &image) {
+void AlbumCoverLoader::SaveEmbeddedCover(const quint64 id, const QString &song_filename, const QImage &image) {
 
   QByteArray image_data;
 
@@ -616,11 +624,17 @@ void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QString song_fil
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QString song_filename, const QString &cover_filename) {
+void AlbumCoverLoader::SaveEmbeddedCover(const quint64 id, const QString &song_filename, const QString &cover_filename) {
 
   QFile file(cover_filename);
 
-  if (file.size() >= 209715200 || !file.open(QIODevice::ReadOnly)) { // Max 200 MB.
+  if (file.size() >= 209715200) {  // Max 200 MB.
+    emit SaveEmbeddedCoverAsyncFinished(id, false, false);
+    return;
+  }
+
+  if (!file.open(QIODevice::ReadOnly)) {
+    qLog(Error) << "Failed to open cover file" << cover_filename << "for reading:" << file.errorString();
     emit SaveEmbeddedCoverAsyncFinished(id, false, false);
     return;
   }
@@ -632,7 +646,7 @@ void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QString song_fil
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QList<QUrl> urls, const QImage &image) {
+void AlbumCoverLoader::SaveEmbeddedCover(const quint64 id, const QList<QUrl> &urls, const QImage &image) {
 
   if (image.isNull()) {
     for (const QUrl &url : urls) {
@@ -657,11 +671,17 @@ void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QList<QUrl> urls
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QList<QUrl> urls, const QString &cover_filename) {
+void AlbumCoverLoader::SaveEmbeddedCover(const quint64 id, const QList<QUrl> &urls, const QString &cover_filename) {
 
   QFile file(cover_filename);
 
-  if (file.size() >= 209715200 || !file.open(QIODevice::ReadOnly)) { // Max 200 MB.
+  if (file.size() >= 209715200) {  // Max 200 MB.
+    emit SaveEmbeddedCoverAsyncFinished(id, false, false);
+    return;
+  }
+
+  if (!file.open(QIODevice::ReadOnly)) {
+    qLog(Error) << "Failed to open cover file" << cover_filename  << "for reading:" << file.errorString();
     emit SaveEmbeddedCoverAsyncFinished(id, false, false);
     return;
   }
@@ -672,7 +692,7 @@ void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QList<QUrl> urls
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QList<QUrl> urls, const QByteArray &image_data) {
+void AlbumCoverLoader::SaveEmbeddedCover(const quint64 id, const QList<QUrl> &urls, const QByteArray &image_data) {
 
   for (const QUrl &url : urls) {
     SaveEmbeddedCover(id, url.toLocalFile(), image_data);
@@ -680,7 +700,7 @@ void AlbumCoverLoader::SaveEmbeddedCover(const qint64 id, const QList<QUrl> urls
 
 }
 
-void AlbumCoverLoader::SaveEmbeddedArtFinished(const qint64 id, TagReaderReply *reply, const bool cleared) {
+void AlbumCoverLoader::SaveEmbeddedArtFinished(const quint64 id, TagReaderReply *reply, const bool cleared) {
 
   if (tagreader_save_embedded_art_requests_.contains(id)) {
     tagreader_save_embedded_art_requests_.remove(id, reply);
@@ -690,6 +710,6 @@ void AlbumCoverLoader::SaveEmbeddedArtFinished(const qint64 id, TagReaderReply *
     emit SaveEmbeddedCoverAsyncFinished(id, reply->is_successful(), cleared);
   }
 
-  metaObject()->invokeMethod(reply, "deleteLater", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(reply, "deleteLater", Qt::QueuedConnection);
 
 }
