@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,9 +33,10 @@
 #include "albumcoverexport.h"
 #include "coverexportrunnable.h"
 
-CoverExportRunnable::CoverExportRunnable(const AlbumCoverExport::DialogResult &dialog_result, const Song &song) :
-    dialog_result_(dialog_result),
-    song_(song) {}
+CoverExportRunnable::CoverExportRunnable(const AlbumCoverExport::DialogResult &dialog_result, const Song &song, QObject *parent)
+    : QObject(parent),
+      dialog_result_(dialog_result),
+      song_(song) {}
 
 void CoverExportRunnable::run() {
 
@@ -45,10 +47,12 @@ void CoverExportRunnable::run() {
     EmitCoverSkipped();
   }
   else {
-    if (dialog_result_.RequiresCoverProcessing())
+    if (dialog_result_.RequiresCoverProcessing()) {
       ProcessAndExportCover();
-    else
+    }
+    else {
       ExportCover();
+    }
   }
 
 }
@@ -117,7 +121,7 @@ void CoverExportRunnable::ProcessAndExportCover() {
   QString dir = song_.url().toLocalFile().section('/', 0, -2);
   QString extension = cover_path.section('.', -1);
 
-  QString new_file = dir + '/' + dialog_result_.fileName_ + '.' + (cover_path == Song::kEmbeddedCover ? "jpg" : extension);
+  QString new_file = dir + '/' + dialog_result_.filename_ + '.' + (cover_path == Song::kEmbeddedCover ? "jpg" : extension);
 
   // If the file exists, do not override!
   if (dialog_result_.overwrite_ == AlbumCoverExport::OverwriteMode_None && QFile::exists(new_file)) {
@@ -146,10 +150,12 @@ void CoverExportRunnable::ProcessAndExportCover() {
     }
   }
 
-  if (cover.save(new_file))
+  if (cover.save(new_file)) {
     EmitCoverExported();
-  else
+  }
+  else {
     EmitCoverSkipped();
+  }
 
 }
 
@@ -161,7 +167,7 @@ void CoverExportRunnable::ExportCover() {
   QString dir = song_.url().toLocalFile().section('/', 0, -2);
   QString extension = cover_path.section('.', -1);
 
-  QString new_file = dir + '/' + dialog_result_.fileName_ + '.' + (cover_path == Song::kEmbeddedCover ? "jpg" : extension);
+  QString new_file = dir + '/' + dialog_result_.filename_ + '.' + (cover_path == Song::kEmbeddedCover ? "jpg" : extension);
 
   // If the file exists, do not override!
   if (dialog_result_.overwrite_ == AlbumCoverExport::OverwriteMode_None && QFile::exists(new_file)) {
