@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2019-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 
 #include "config.h"
 
+#include <memory>
+
 #include <QtGlobal>
 #include <QObject>
 #include <QDir>
@@ -41,8 +43,7 @@ CurrentAlbumCoverLoader::CurrentAlbumCoverLoader(Application *app, QObject *pare
     : QObject(parent),
       app_(app),
       temp_file_pattern_(QDir::tempPath() + "/strawberry-cover-XXXXXX.jpg"),
-      id_(0)
-  {
+      id_(0) {
 
   options_.get_image_data_ = true;
   options_.get_image_ = true;
@@ -78,24 +79,24 @@ void CurrentAlbumCoverLoader::TempAlbumCoverLoaded(const quint64 id, AlbumCoverL
   id_ = 0;
 
   if (!result.album_cover.image.isNull()) {
-    temp_cover_.reset(new QTemporaryFile(temp_file_pattern_));
+    temp_cover_ = std::make_unique<QTemporaryFile>(temp_file_pattern_);
     temp_cover_->setAutoRemove(true);
     if (temp_cover_->open()) {
       if (result.album_cover.image.save(temp_cover_->fileName(), "JPEG")) {
         result.temp_cover_url = QUrl::fromLocalFile(temp_cover_->fileName());
       }
       else {
-        qLog(Error) << "Unable to save cover image to" << temp_cover_->fileName();
+        qLog(Error) << "Failed to save cover image to" << temp_cover_->fileName() << temp_cover_->errorString();
       }
     }
     else {
-      qLog(Error) << "Unable to open" << temp_cover_->fileName();
+      qLog(Error) << "Failed to open" << temp_cover_->fileName() << temp_cover_->errorString();
     }
   }
 
   QUrl thumbnail_url;
   if (!result.image_thumbnail.isNull()) {
-    temp_cover_thumbnail_.reset(new QTemporaryFile(temp_file_pattern_));
+    temp_cover_thumbnail_ = std::make_unique<QTemporaryFile>(temp_file_pattern_);
     temp_cover_thumbnail_->setAutoRemove(true);
     if (temp_cover_thumbnail_->open()) {
       if (result.image_thumbnail.save(temp_cover_thumbnail_->fileName(), "JPEG")) {
