@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2012, Martin Björklund <mbj4668@gmail.com>
- * Copyright 2016, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2016-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,6 @@
 #include <QUrl>
 #include <QUrlQuery>
 #include <QTimer>
-#include <QNetworkAccessManager>
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QJsonDocument>
@@ -61,9 +60,9 @@ const char *DiscogsCoverProvider::kAccessKeyB64 = "dGh6ZnljUGJlZ1NEeXBuSFFxSVk="
 const char *DiscogsCoverProvider::kSecretKeyB64 = "ZkFIcmlaSER4aHhRSlF2U3d0bm5ZVmdxeXFLWUl0UXI=";
 const int DiscogsCoverProvider::kRequestsDelay = 1000;
 
-DiscogsCoverProvider::DiscogsCoverProvider(Application *app, QObject *parent) :
-  JsonCoverProvider("Discogs", false, false, 0.0, false, false, app, parent),
-  network_(new NetworkAccessManager(this)), timer_flush_requests_(new QTimer(this)) {
+DiscogsCoverProvider::DiscogsCoverProvider(Application *app, NetworkAccessManager *network, QObject *parent)
+    : JsonCoverProvider("Discogs", false, false, 0.0, false, false, app, network, parent),
+      timer_flush_requests_(new QTimer(this)) {
 
   timer_flush_requests_->setInterval(kRequestsDelay);
   timer_flush_requests_->setSingleShot(false);
@@ -144,7 +143,7 @@ void DiscogsCoverProvider::SendSearchRequest(std::shared_ptr<DiscogsCoverSearchC
   }
 
   QNetworkReply *reply = CreateRequest(QUrl(kUrlSearch), params);
-  QObject::connect(reply, &QNetworkReply::finished, [this, reply, search]() { HandleSearchReply(reply, search->id); });
+  QObject::connect(reply, &QNetworkReply::finished, this, [this, reply, search]() { HandleSearchReply(reply, search->id); });
 
 }
 
@@ -268,15 +267,17 @@ void DiscogsCoverProvider::HandleSearchReply(QNetworkReply *reply, const int id)
     return;
   }
 
-  QJsonArray array_results;
-  if (value_results.isArray()) {
-    array_results = value_results.toArray();
+  if (!value_results.isArray()) {
+    Error("Missing results array.", value_results);
+    EndSearch(search);
+    return;
   }
 
-  for (const QJsonValue value_result : array_results) {
+  QJsonArray array_results = value_results.toArray();
+  for (QJsonValueRef value_result : array_results) {
 
     if (!value_result.isObject()) {
-      Error("Invalid Json reply, results value is not a object.", value_result);
+      Error("Invalid Json reply, results value is not a object.");
       continue;
     }
     QJsonObject obj_result = value_result.toObject();
@@ -284,7 +285,7 @@ void DiscogsCoverProvider::HandleSearchReply(QNetworkReply *reply, const int id)
       Error("Invalid Json reply, results value object is missing ID, title or resource_url.", obj_result);
       continue;
     }
-    quint64 release_id = obj_result["id"].toDouble();
+    quint64 release_id = obj_result["id"].toInt();
     QUrl resource_url(obj_result["resource_url"].toString());
     QString title = obj_result["title"].toString();
 
@@ -293,7 +294,7 @@ void DiscogsCoverProvider::HandleSearchReply(QNetworkReply *reply, const int id)
       if (title_splitted.count() == 2) {
         QString artist = title_splitted.first();
         title = title_splitted.last();
-        if (artist.toLower() != search->artist.toLower() && title.toLower() != search->album.toLower()) continue;
+        if (artist.compare(search->artist, Qt::CaseInsensitive) != 0 && title.compare(search->album, Qt::CaseInsensitive) != 0) continue;
       }
     }
 
@@ -328,10 +329,10 @@ void DiscogsCoverProvider::StartReleaseRequest(std::shared_ptr<DiscogsCoverSearc
 
 }
 
-void DiscogsCoverProvider::SendReleaseRequest(const DiscogsCoverReleaseContext release) {
+void DiscogsCoverProvider::SendReleaseRequest(const DiscogsCoverReleaseContext &release) {
 
   QNetworkReply *reply = CreateRequest(release.url);
-  QObject::connect(reply, &QNetworkReply::finished, [this, reply, release]() { HandleReleaseReply(reply, release.search_id, release.id); } );
+  QObject::connect(reply, &QNetworkReply::finished, this, [this, reply, release]() { HandleReleaseReply(reply, release.search_id, release.id); });
 
 }
 
@@ -380,9 +381,9 @@ void DiscogsCoverProvider::HandleReleaseReply(QNetworkReply *reply, const int se
   QJsonArray array_artists = value_artists.toArray();
   int i = 0;
   QString artist;
-  for (const QJsonValue value_artist : array_artists) {
+  for (const QJsonValueRef value_artist : array_artists) {
     if (!value_artist.isObject()) {
-      Error("Invalid Json reply, atists array value is not a object.", value_artist);
+      Error("Invalid Json reply, atists array value is not a object.");
       continue;
     }
     QJsonObject obj_artist = value_artist.toObject();
@@ -421,14 +422,14 @@ void DiscogsCoverProvider::HandleReleaseReply(QNetworkReply *reply, const int se
     return;
   }
 
-  for (const QJsonValue value_image : array_images) {
+  for (const QJsonValueRef value_image : array_images) {
 
     if (!value_image.isObject()) {
-      Error("Invalid Json reply, images array value is not an object.", value_image);
+      Error("Invalid Json reply, images array value is not an object.");
       continue;
     }
     QJsonObject obj_image = value_image.toObject();
-    if (!obj_image.contains("type") || !obj_image.contains("resource_url") || !obj_image.contains("width") || !obj_image.contains("height") ) {
+    if (!obj_image.contains("type") || !obj_image.contains("resource_url") || !obj_image.contains("width") || !obj_image.contains("height")) {
       Error("Invalid Json reply, images array value object is missing type, resource_url, width or height.", obj_image);
       continue;
     }
@@ -439,7 +440,7 @@ void DiscogsCoverProvider::HandleReleaseReply(QNetworkReply *reply, const int se
     int width = obj_image["width"].toInt();
     int height = obj_image["height"].toInt();
     if (width < 300 || height < 300) continue;
-    const float aspect_score = 1.0 - float(std::max(width, height) - std::min(width, height)) / std::max(height, width);
+    const float aspect_score = static_cast<float>(1.0) - static_cast<float>(std::max(width, height) - std::min(width, height)) / static_cast<float>(std::max(height, width));
     if (aspect_score < 0.85) continue;
     CoverProviderSearchResult result;
     result.artist = artist;
