@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,8 +38,8 @@
 #include "devicemanager.h"
 #include "deviceinfo.h"
 
-ConnectedDevice::ConnectedDevice(const QUrl &url, DeviceLister *lister, const QString &unique_id, DeviceManager *manager, Application *app, int database_id, bool first_time)
-    : QObject(manager),
+ConnectedDevice::ConnectedDevice(const QUrl &url, DeviceLister *lister, const QString &unique_id, DeviceManager *manager, Application *app, const int database_id, const bool first_time, QObject *parent)
+    : QObject(parent),
       app_(app),
       url_(url),
       first_time_(first_time),
@@ -63,11 +63,12 @@ ConnectedDevice::ConnectedDevice(const QUrl &url, DeviceLister *lister, const QS
   }
 
   backend_->Init(app_->database(),
+                 app_->task_manager(),
                  Song::Source_Device,
                  QString("device_%1_songs").arg(database_id),
+                 QString("device_%1_fts").arg(database_id),
                  QString("device_%1_directories").arg(database_id),
-                 QString("device_%1_subdirectories").arg(database_id),
-                 QString("device_%1_fts").arg(database_id));
+                 QString("device_%1_subdirectories").arg(database_id));
 
   // Create the model
   model_ = new CollectionModel(backend_, app_, this);
@@ -80,7 +81,8 @@ ConnectedDevice::~ConnectedDevice() {
 
 void ConnectedDevice::InitBackendDirectory(const QString &mount_point, const bool first_time, const bool rewrite_path) {
 
-  if (first_time || backend_->GetAllDirectories().isEmpty()) {
+  QList<Directory> directories = backend_->GetAllDirectories();
+  if (first_time || directories.isEmpty()) {
     backend_->AddDirectory(mount_point);
   }
   else {
@@ -91,7 +93,7 @@ void ConnectedDevice::InitBackendDirectory(const QString &mount_point, const boo
       // This can be done entirely in sqlite so it's relatively fast...
 
       // Get the directory it was mounted at last time.  Devices only have one directory (the root).
-      Directory dir = backend_->GetAllDirectories()[0];
+      Directory dir = directories[0];
       if (dir.path != mount_point) {
         // The directory is different, commence the munging.
         qLog(Info) << "Changing path from" << dir.path << "to" << mount_point;
