@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,8 +31,8 @@
 #include "devicestatefiltermodel.h"
 
 DeviceStateFilterModel::DeviceStateFilterModel(QObject *parent, DeviceManager::State state)
-  : QSortFilterProxyModel(parent),
-    state_(state) {
+    : QSortFilterProxyModel(parent),
+      state_(state) {
 
   QObject::connect(this, &DeviceStateFilterModel::rowsInserted, this, &DeviceStateFilterModel::ProxyRowCountChanged);
   QObject::connect(this, &DeviceStateFilterModel::rowsRemoved, this, &DeviceStateFilterModel::ProxyRowCountChanged);
@@ -43,7 +44,7 @@ bool DeviceStateFilterModel::filterAcceptsRow(int row, const QModelIndex&) const
   return sourceModel()->index(row, 0).data(DeviceManager::Role_State).toInt() != state_ && sourceModel()->index(row, 0).data(DeviceManager::Role_CopyMusic).toBool();
 }
 
-void DeviceStateFilterModel::ProxyRowCountChanged(QModelIndex, int, int) {
+void DeviceStateFilterModel::ProxyRowCountChanged(const QModelIndex&, const int, const int) {
 
   emit IsEmptyChanged(rowCount() == 0);
 
