@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +21,6 @@
 
 #include "config.h"
 
-#include <cassert>
-
 #include <QThread>
 #include <QString>
 #include <QUrl>
@@ -40,11 +39,11 @@
 
 class DeviceLister;
 
-FilesystemDevice::FilesystemDevice(const QUrl &url, DeviceLister *lister, const QString &unique_id, DeviceManager *manager, Application *app, int database_id, bool first_time)
+FilesystemDevice::FilesystemDevice(const QUrl &url, DeviceLister *lister, const QString &unique_id, DeviceManager *manager, Application *app, const int database_id, const bool first_time, QObject *parent)
       : FilesystemMusicStorage(url.toLocalFile()),
-      ConnectedDevice(url, lister, unique_id, manager, app, database_id, first_time),
-      watcher_(new CollectionWatcher(Song::Source_Device)), watcher_thread_(new QThread(this))
-  {
+        ConnectedDevice(url, lister, unique_id, manager, app, database_id, first_time, parent),
+        watcher_(new CollectionWatcher(Song::Source_Device)),
+        watcher_thread_(new QThread(this)) {
 
   watcher_->moveToThread(watcher_thread_);
   watcher_thread_->start(QThread::IdlePriority);
@@ -84,12 +83,12 @@ bool FilesystemDevice::Init() {
 }
 
 void FilesystemDevice::CloseAsync() {
-  metaObject()->invokeMethod(this, "Close", Qt::QueuedConnection);
+  QMetaObject::invokeMethod(this, "Close", Qt::QueuedConnection);
 }
 
 void FilesystemDevice::Close() {
 
-  assert(QThread::currentThread() == thread());
+  Q_ASSERT(QThread::currentThread() == thread());
 
   wait_for_exit_ << backend_ << watcher_;
 
@@ -105,7 +104,7 @@ void FilesystemDevice::Close() {
 
 void FilesystemDevice::ExitFinished() {
 
-  QObject *obj = qobject_cast<QObject*>(sender());
+  QObject *obj = sender();
   if (!obj) return;
   QObject::disconnect(obj, nullptr, this, nullptr);
   qLog(Debug) << obj << "successfully exited.";
