@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,8 @@
 
 #include "config.h"
 
+#include <memory>
+
 #include <gpod/itdb.h>
 
 #include <QObject>
@@ -32,6 +35,7 @@
 #include <QString>
 #include <QStringList>
 #include <QUrl>
+#include <QTemporaryFile>
 
 #include "core/song.h"
 #include "core/musicstorage.h"
@@ -47,39 +51,40 @@ class GPodDevice : public ConnectedDevice, public virtual MusicStorage {
   Q_OBJECT
 
  public:
-  Q_INVOKABLE GPodDevice(const QUrl &url, DeviceLister *lister, const QString &unique_id, DeviceManager *manager, Application *app, const int database_id, const bool first_time);
-  ~GPodDevice();
+  Q_INVOKABLE GPodDevice(const QUrl &url, DeviceLister *lister, const QString &unique_id, DeviceManager *manager, Application *app, const int database_id, const bool first_time, QObject *parent = nullptr);
+  ~GPodDevice() override;
 
-  bool Init();
-  void ConnectAsync();
-  void Close();
-  bool IsLoading() { return loader_; }
+  bool Init() override;
+  void ConnectAsync() override;
+  void Close() override;
+  bool IsLoading() override { return loader_; }
   QObject *Loader() { return loader_; }
 
   static QStringList url_schemes() { return QStringList() << "ipod"; }
 
-  bool GetSupportedFiletypes(QList<Song::FileType> *ret);
+  bool GetSupportedFiletypes(QList<Song::FileType> *ret) override;
 
-  bool StartCopy(QList<Song::FileType> *supported_types);
-  bool CopyToStorage(const CopyJob &job);
-  void FinishCopy(bool success);
+  bool StartCopy(QList<Song::FileType> *supported_filetypes) override;
+  bool CopyToStorage(const CopyJob &job) override;
+  void FinishCopy(bool success) override;
 
-  void StartDelete();
-  bool DeleteFromStorage(const DeleteJob &job);
-  void FinishDelete(bool success);
+  void StartDelete() override;
+  bool DeleteFromStorage(const DeleteJob &job) override;
+  void FinishDelete(bool success) override;
 
  protected slots:
-  void LoadFinished(Itdb_iTunesDB *db, bool success);
-  void LoaderError(const QString& message);
+  void LoadFinished(Itdb_iTunesDB *db, const bool success);
+  void LoaderError(const QString &message);
 
  protected:
   Itdb_Track *AddTrackToITunesDb(const Song &metadata);
   void AddTrackToModel(Itdb_Track *track, const QString &prefix);
   bool RemoveTrackFromITunesDb(const QString &path, const QString &relative_to = QString());
-  virtual void FinaliseDatabase() {}
 
  private:
-  void WriteDatabase(bool success);
+  void Start();
+  void Finish(const bool success);
+  bool WriteDatabase();
 
  protected:
   GPodLoader *loader_;
@@ -93,6 +98,7 @@ class GPodDevice : public ConnectedDevice, public virtual MusicStorage {
   QMutex db_busy_;
   SongList songs_to_add_;
   SongList songs_to_remove_;
+  QList<std::shared_ptr<QTemporaryFile>> cover_files_;
 };
 
-#endif // GPODDEVICE_H
+#endif  // GPODDEVICE_H
