@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,8 +36,8 @@
 #include "core/taskmanager.h"
 #include "gpodloader.h"
 
-GPodLoader::GPodLoader(const QString &mount_point, TaskManager *task_manager, CollectionBackend *backend, std::shared_ptr<ConnectedDevice> device)
-    : QObject(nullptr),
+GPodLoader::GPodLoader(const QString &mount_point, TaskManager *task_manager, CollectionBackend *backend, std::shared_ptr<ConnectedDevice> device, QObject *parent)
+    : QObject(parent),
       device_(device),
       mount_point_(mount_point),
       type_(Song::FileType_Unknown),
@@ -46,7 +47,7 @@ GPodLoader::GPodLoader(const QString &mount_point, TaskManager *task_manager, Co
   original_thread_ = thread();
 }
 
-GPodLoader::~GPodLoader() {}
+GPodLoader::~GPodLoader() = default;
 
 void GPodLoader::LoadDatabase() {
 
