@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2016, Valeriy Malov <jazzvoid@gmail.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +21,9 @@
 
 #include "config.h"
 
+#include <memory>
+#include <utility>
+
 #include <dbus/objectmanager.h>
 #include <dbus/udisks2block.h>
 #include <dbus/udisks2drive.h>
@@ -49,11 +53,9 @@
 
 #include "udisks2lister.h"
 
-constexpr char Udisks2Lister::udisks2_service_[];
+Udisks2Lister::Udisks2Lister(QObject *parent) : DeviceLister(parent) {}
 
-Udisks2Lister::Udisks2Lister() {}
-
-Udisks2Lister::~Udisks2Lister() {}
+Udisks2Lister::~Udisks2Lister() = default;
 
 QStringList Udisks2Lister::DeviceUniqueIDs() {
   QReadLocker locker(&device_data_lock_);
@@ -109,7 +111,7 @@ QVariantMap Udisks2Lister::DeviceHardwareInfo(const QString &id) {
 
   QVariantMap result;
 
-  const auto &data = device_data_[id];
+  const PartitionData &data = device_data_[id];
   result[QT_TR_NOOP("D-Bus path")] = data.dbus_path;
   result[QT_TR_NOOP("Serial number")] = data.serial;
   result[QT_TR_NOOP("Mount points")] = data.mount_paths.join(", ");
@@ -160,12 +162,13 @@ void Udisks2Lister::UnmountDevice(const QString &id) {
       auto eject_result = drive.Eject(QVariantMap());
       eject_result.waitForFinished();
 
-      if (eject_result.isError())
+      if (eject_result.isError()) {
         qLog(Warning) << "Failed to eject " << id << ": " << eject_result.error();
+      }
     }
 
     device_data_.remove(id);
-    DeviceRemoved(id);
+    emit DeviceRemoved(id);
   }
 
 }
@@ -178,7 +181,7 @@ void Udisks2Lister::UpdateDeviceFreeSpace(const QString &id) {
 
 bool Udisks2Lister::Init() {
 
-  udisks2_interface_.reset(new OrgFreedesktopDBusObjectManagerInterface(udisks2_service_, "/org/freedesktop/UDisks2", QDBusConnection::systemBus()));
+  udisks2_interface_ = std::make_unique<OrgFreedesktopDBusObjectManagerInterface>(udisks2_service_, "/org/freedesktop/UDisks2", QDBusConnection::systemBus());
 
   QDBusPendingReply<ManagedObjectList> reply = udisks2_interface_->GetManagedObjects();
   reply.waitForFinished();
@@ -189,8 +192,9 @@ bool Udisks2Lister::Init() {
     return false;
   }
 
-  for (const QDBusObjectPath &path : reply.value().keys()) {
-    auto partition_data = ReadPartitionData(path);
+  QList<QDBusObjectPath> paths = reply.value().keys();
+  for (const QDBusObjectPath &path : paths) {
+    PartitionData partition_data = ReadPartitionData(path);
 
     if (!partition_data.dbus_path.isEmpty()) {
       QWriteLocker locker(&device_data_lock_);
@@ -198,7 +202,8 @@ bool Udisks2Lister::Init() {
     }
   }
 
-  for (const auto &id : device_data_.keys()) {
+  QStringList ids = device_data_.keys();
+  for (const QString &id : ids) {
     emit DeviceAdded(id);
   }
 
@@ -268,7 +273,7 @@ void Udisks2Lister::RemoveDevice(const QDBusObjectPath &device_path) {
 
   QWriteLocker locker(&device_data_lock_);
   QString id;
-  for (const auto &data : device_data_) {
+  for (const PartitionData &data : std::as_const(device_data_)) {
     if (data.dbus_path == device_path.path()) {
       id = data.unique_id();
       break;
@@ -279,7 +284,8 @@ void Udisks2Lister::RemoveDevice(const QDBusObjectPath &device_path) {
 
   qLog(Debug) << "UDisks2 device removed: " << device_path.path();
   device_data_.remove(id);
-  DeviceRemoved(id);
+
+  emit DeviceRemoved(id);
 
 }
 
@@ -299,14 +305,16 @@ QList<QDBusObjectPath> Udisks2Lister::GetMountedPartitionsFromDBusArgument(const
 
 }
 
-void Udisks2Lister::JobCompleted(bool success, const QString &message) {
+void Udisks2Lister::JobCompleted(const bool success, const QString &message) {
 
   Q_UNUSED(message);
 
-  auto job = qobject_cast<OrgFreedesktopUDisks2JobInterface*>(sender());
+  OrgFreedesktopUDisks2JobInterface *job = qobject_cast<OrgFreedesktopUDisks2JobInterface*>(sender());
   QDBusObjectPath jobPath(job->path());
 
-  if (!job->isValid() || !success || !mounting_jobs_.contains(jobPath)) return;
+  if (!job->isValid() || !success || !mounting_jobs_.contains(jobPath)) {
+    return;
+  }
 
   qLog(Debug) << "Pending Job Completed | Path = " << job->path() << " | Mount? = " << mounting_jobs_[jobPath].is_mount << " | Success = " << success;
 
@@ -319,20 +327,21 @@ void Udisks2Lister::JobCompleted(bool success, const QString &message) {
 
 }
 
-void Udisks2Lister::HandleFinishedMountJob(const Udisks2Lister::PartitionData &partition_data) {
+void Udisks2Lister::HandleFinishedMountJob(const PartitionData &partition_data) {
 
   qLog(Debug) << "UDisks2 mount job finished: Drive = " << partition_data.dbus_drive_path << " | Partition = " << partition_data.dbus_path;
   QWriteLocker locker(&device_data_lock_);
   device_data_[partition_data.unique_id()] = partition_data;
-  DeviceAdded(partition_data.unique_id());
+
+  emit DeviceAdded(partition_data.unique_id());
 
 }
 
-void Udisks2Lister::HandleFinishedUnmountJob(const Udisks2Lister::PartitionData &partition_data, const QDBusObjectPath &mounted_object) {
+void Udisks2Lister::HandleFinishedUnmountJob(const PartitionData &partition_data, const QDBusObjectPath &mounted_object) {
 
   QWriteLocker locker(&device_data_lock_);
   QString id;
-  for (auto &data : device_data_) {
+  for (PartitionData &data : device_data_) {
     if (data.mount_paths.contains(mounted_object.path())) {
       qLog(Debug) << "UDisks2 umount job finished, found corresponding device: Drive = " << data.dbus_drive_path << " | Partition = " << data.dbus_path;
       data.mount_paths.removeOne(mounted_object.path());
@@ -344,7 +353,7 @@ void Udisks2Lister::HandleFinishedUnmountJob(const Udisks2Lister::PartitionData
   if (!id.isEmpty()) {
     qLog(Debug) << "Partition " << partition_data.dbus_path << " has no more mount points, removing it from device list";
     device_data_.remove(id);
-    DeviceRemoved(id);
+    emit DeviceRemoved(id);
   }
 
 }
@@ -379,7 +388,7 @@ Udisks2Lister::PartitionData Udisks2Lister::ReadPartitionData(const QDBusObjectP
 
       for (const QByteArray &p : filesystem.mountPoints()) {
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)  // Workaround a bytearray to string conversion issue with Qt 6
-        QString mountpoint = QByteArray(p.data(), strlen(p.data()));
+        QString mountpoint = QByteArray(p.data(), static_cast<qint64>(strlen(p.data())));
 #else
         QString mountpoint = p;
 #endif
