@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,9 +45,7 @@
 #include <QUrl>
 
 class QTimerEvent;
-class GstEngine;
 class GstBufferConsumer;
-class GstElementDeleter;
 
 namespace Engine {
 struct SimpleMetaBundle;
@@ -58,7 +56,7 @@ class GstEnginePipeline : public QObject {
   Q_OBJECT
 
  public:
-  explicit GstEnginePipeline(GstEngine *engine);
+  explicit GstEnginePipeline(QObject *parent = nullptr);
   ~GstEnginePipeline() override;
 
   // Globally unique across all pipelines.
@@ -69,14 +67,15 @@ class GstEnginePipeline : public QObject {
   void set_volume_enabled(const bool enabled);
   void set_stereo_balancer_enabled(const bool enabled);
   void set_equalizer_enabled(const bool enabled);
-  void set_replaygain(const bool enabled, const int mode, const float preamp, const bool compression);
-  void set_buffer_duration_nanosec(const qint64 duration_nanosec);
+  void set_replaygain(const bool enabled, const int mode, const double preamp, const double fallbackgain, const bool compression);
+  void set_buffer_duration_nanosec(const quint64 duration_nanosec);
   void set_buffer_low_watermark(const double value);
   void set_buffer_high_watermark(const double value);
   void set_proxy_settings(const QString &address, const bool authentication, const QString &user, const QString &pass);
+  void set_channels(const bool enabled, const int channels);
 
   // Creates the pipeline, returns false on error
-  bool InitFromUrl(const QByteArray &stream_url, const QUrl original_url, const qint64 end_nanosec);
+  bool InitFromUrl(const QByteArray &stream_url, const QUrl &original_url, const qint64 end_nanosec, QString &error);
 
   // GstBufferConsumers get fed audio data.  Thread-safe.
   void AddBufferConsumer(GstBufferConsumer *consumer);
@@ -86,7 +85,7 @@ class GstEnginePipeline : public QObject {
   // Control the music playback
   QFuture<GstStateChangeReturn> SetState(const GstState state);
   Q_INVOKABLE bool Seek(const qint64 nanosec);
-  void SetVolume(const int percent);
+  void SetVolume(const uint percent);
   void SetStereoBalance(const float value);
   void SetEqualizerParams(const int preamp, const QList<int> &band_gains);
 
@@ -96,7 +95,7 @@ class GstEnginePipeline : public QObject {
   void SetNextUrl(const QByteArray &stream_url, const QUrl &original_url, qint64 beginning_nanosec, qint64 end_nanosec);
   bool has_next_valid_url() const { return !next_stream_url_.isEmpty(); }
 
-  void SetSourceDevice(QString device) { source_device_ = device; }
+  void SetSourceDevice(const QString &device) { source_device_ = device; }
 
   // Get information about the music playback
   QByteArray stream_url() const { return stream_url_; }
@@ -124,11 +123,11 @@ class GstEnginePipeline : public QObject {
   void SetVolumeModifier(qreal mod);
 
  signals:
+  void Error(int pipeline_id, QString message, const int domain, const int error_code);
+
   void EndOfStreamReached(int pipeline_id, bool has_next_track);
   void MetadataFound(int pipeline_id, const Engine::SimpleMetaBundle &bundle);
-  // This indicates an error, delegated from GStreamer, in the pipeline.
-  // The message, domain and error_code are related to GStreamer's GError.
-  void Error(int pipeline_id, QString message, int domain, int error_code);
+
   void FaderFinished();
 
   void BufferingStarted();
@@ -139,11 +138,12 @@ class GstEnginePipeline : public QObject {
   void timerEvent(QTimerEvent*) override;
 
  private:
-  bool InitAudioBin();
+  GstElement *CreateElement(const QString &factory_name, const QString &name, GstElement *bin, QString &error) const;
+  bool InitAudioBin(QString &error);
 
   // Static callbacks.  The GstEnginePipeline instance is passed in the last argument.
   static GstPadProbeReturn EventHandoffCallback(GstPad*, GstPadProbeInfo*, gpointer);
-  static void SourceSetupCallback(GstPlayBin*, GParamSpec* pspec, gpointer);
+  static void SourceSetupCallback(GstPlayBin*, GParamSpec *pspec, gpointer);
   static void NewPadCallback(GstElement*, GstPad*, gpointer);
   static GstPadProbeReturn PlaybinProbe(GstPad*, GstPadProbeInfo*, gpointer);
   static GstPadProbeReturn HandoffCallback(GstPad*, GstPadProbeInfo*, gpointer);
@@ -160,8 +160,8 @@ class GstEnginePipeline : public QObject {
   void StreamStatusMessageReceived(GstMessage*);
   void StreamStartMessageReceived();
 
-  QString ParseStrTag(GstTagList *list, const char *tag) const;
-  guint ParseUIntTag(GstTagList *list, const char *tag) const;
+  static QString ParseStrTag(GstTagList *list, const char *tag);
+  static guint ParseUIntTag(GstTagList *list, const char *tag);
 
   void UpdateVolume();
   void UpdateStereoBalance();
@@ -176,10 +176,6 @@ class GstEnginePipeline : public QObject {
   static const int kEqBandCount;
   static const int kEqBandFrequencies[];
 
-  static GstElementDeleter *sElementDeleter;
-
-  GstEngine *engine_;
-
   // Using == to compare two pipelines is a bad idea, because new ones often get created in the same address as old ones.  This ID will be unique for each pipeline.
   // Threading warning: access to the static ID field isn't protected by a mutex because all pipeline creation is currently done in the main thread.
   static int sId;
@@ -205,7 +201,8 @@ class GstEnginePipeline : public QObject {
 
   // ReplayGain
   int rg_mode_;
-  float rg_preamp_;
+  double rg_preamp_;
+  double rg_fallbackgain_;
   bool rg_compression_;
 
   // Buffering
@@ -220,6 +217,10 @@ class GstEnginePipeline : public QObject {
   QString proxy_user_;
   QString proxy_pass_;
 
+  // Channels
+  bool channels_enabled_;
+  int channels_;
+
   // These get called when there is a new audio buffer available
   QList<GstBufferConsumer*> buffer_consumers_;
   QMutex buffer_consumers_mutex_;
@@ -265,7 +266,7 @@ class GstEnginePipeline : public QObject {
   // Complete the transition to the next song when it starts playing
   bool next_uri_set_;
 
-  int volume_percent_;
+  uint volume_percent_;
   qreal volume_modifier_;
 
   std::unique_ptr<QTimeLine> fader_;
@@ -283,13 +284,12 @@ class GstEnginePipeline : public QObject {
   int pad_added_cb_id_;
   int notify_source_cb_id_;
   int about_to_finish_cb_id_;
-  int bus_cb_id_;
 
   QThreadPool set_state_threadpool_;
 
-  GstSegment last_playbin_segment_;
+  GstSegment last_playbin_segment_{};
 
-  bool unsupported_analyzer_;
+  bool logged_unsupported_analyzer_format_;
 
 };
 
