@@ -35,7 +35,7 @@
 namespace {
 
 template <typename T>
-std::unique_ptr<T> GetProperty(const AudioDeviceID& device_id, const AudioObjectPropertyAddress& address, UInt32* size_bytes_out = nullptr) {
+std::unique_ptr<T> GetProperty(const AudioDeviceID &device_id, const AudioObjectPropertyAddress &address, UInt32 *size_bytes_out = nullptr) {
 
   UInt32 size_bytes = 0;
   OSStatus status = AudioObjectGetPropertyDataSize(device_id, &address, 0, NULL, &size_bytes);
@@ -62,9 +62,7 @@ std::unique_ptr<T> GetProperty(const AudioDeviceID& device_id, const AudioObject
 }  // namespace
 
 
-MacOsDeviceFinder::MacOsDeviceFinder()
-    : DeviceFinder("osxaudio", { "osxaudio", "osx", "osxaudiosink"} ) {
-}
+MacOsDeviceFinder::MacOsDeviceFinder() : DeviceFinder("osxaudio", { "osxaudio", "osx", "osxaudiosink" }) {}
 
 QList<DeviceFinder::Device> MacOsDeviceFinder::ListDevices() {
 
@@ -78,7 +76,7 @@ QList<DeviceFinder::Device> MacOsDeviceFinder::ListDevices() {
 
   UInt32 device_size_bytes = 0;
   std::unique_ptr<AudioDeviceID> devices = GetProperty<AudioDeviceID>(kAudioObjectSystemObject, address, &device_size_bytes);
-  if (!devices.get()) {
+  if (!devices) {
     return ret;
   }
   const UInt32 device_count = device_size_bytes / sizeof(AudioDeviceID);
@@ -91,7 +89,7 @@ QList<DeviceFinder::Device> MacOsDeviceFinder::ListDevices() {
     address.mSelector = kAudioDevicePropertyDeviceNameCFString;
     std::unique_ptr<CFStringRef> device_name = GetProperty<CFStringRef>(id, address);
     ScopedCFTypeRef<CFStringRef> scoped_device_name(*device_name.get());
-    if (!device_name.get()) {
+    if (!device_name) {
       continue;
     }
 
