@@ -52,11 +52,11 @@ class Equalizer : public QDialog {
     Params();
     Params(int g0, int g1, int g2, int g3, int g4, int g5, int g6, int g7, int g8, int g9, int pre = 0);
 
-    bool operator ==(const Params &other) const;
-    bool operator !=(const Params &other) const;
+    bool operator==(const Params &other) const;
+    bool operator!=(const Params &other) const;
 
     int preamp;
-    int gain[kBands];
+    int gain[kBands]{};
   };
 
   bool is_equalizer_enabled() const;
@@ -81,7 +81,7 @@ class Equalizer : public QDialog {
   void EqualizerEnabledChangedSlot(const bool enabled);
   void EqualizerParametersChangedSlot();
   void PresetChanged(const QString &name);
-  void PresetChanged(int index);
+  void PresetChanged(const int index);
   void SavePreset();
   void DelPreset();
   void Save();
@@ -100,7 +100,7 @@ class Equalizer : public QDialog {
   QString last_preset_;
 
   EqualizerSlider *preamp_;
-  EqualizerSlider *gain_[kBands];
+  EqualizerSlider *gain_[kBands]{};
 
   QMap<QString, Params> presets_;
 };
