@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,14 +48,14 @@
 
 class GlobalShortcutsBackendMacOSPrivate : boost::noncopyable {
  public:
-  explicit GlobalShortcutsBackendMacOSPrivate(GlobalShortcutsBackendMacOS* backend)
+  explicit GlobalShortcutsBackendMacOSPrivate(GlobalShortcutsBackendMacOS *backend)
       : global_monitor_(nil), local_monitor_(nil), backend_(backend) {}
 
   bool Register() {
-    global_monitor_ = [NSEvent addGlobalMonitorForEventsMatchingMask:NSEventMaskKeyDown handler:^(NSEvent* event) {
+    global_monitor_ = [NSEvent addGlobalMonitorForEventsMatchingMask:NSEventMaskKeyDown handler:^(NSEvent *event) {
       HandleKeyEvent(event);
     } ];
-    local_monitor_ = [NSEvent addLocalMonitorForEventsMatchingMask:NSEventMaskKeyDown handler:^(NSEvent* event) {
+    local_monitor_ = [NSEvent addLocalMonitorForEventsMatchingMask:NSEventMaskKeyDown handler:^(NSEvent *event) {
       // Filter event if we handle it as a global shortcut.
       return HandleKeyEvent(event) ? nil : event;
     } ];
@@ -67,18 +68,18 @@ class GlobalShortcutsBackendMacOSPrivate : boost::noncopyable {
   }
 
  private:
-  bool HandleKeyEvent(NSEvent* event) {
+  bool HandleKeyEvent(NSEvent *event) {
     QKeySequence sequence = mac::KeySequenceFromNSEvent(event);
     return backend_->KeyPressed(sequence);
   }
 
   id global_monitor_;
   id local_monitor_;
-  GlobalShortcutsBackendMacOS* backend_;
+  GlobalShortcutsBackendMacOS *backend_;
 };
 
-GlobalShortcutsBackendMacOS::GlobalShortcutsBackendMacOS(GlobalShortcutsManager* parent)
-    : GlobalShortcutsBackend(parent),
+GlobalShortcutsBackendMacOS::GlobalShortcutsBackendMacOS(GlobalShortcutsManager *manager, QObject *parent)
+    : GlobalShortcutsBackend(manager, GlobalShortcutsBackend::Type_MacOS, parent),
       p_(new GlobalShortcutsBackendMacOSPrivate(this)) {}
 
 GlobalShortcutsBackendMacOS::~GlobalShortcutsBackendMacOS() {}
@@ -88,7 +89,8 @@ bool GlobalShortcutsBackendMacOS::DoRegister() {
   // Always enable media keys.
   mac::SetShortcutHandler(this);
 
-  for (const GlobalShortcutsManager::Shortcut& shortcut : manager_->shortcuts().values()) {
+  QList<GlobalShortcutsManager::Shortcut> shortcuts = manager_->shortcuts().values();
+  for (const GlobalShortcutsManager::Shortcut &shortcut : shortcuts) {
     shortcuts_[shortcut.action->shortcut()] = shortcut.action;
   }
 
@@ -103,7 +105,7 @@ void GlobalShortcutsBackendMacOS::DoUnregister() {
 
 }
 
-void GlobalShortcutsBackendMacOS::MacMediaKeyPressed(int key) {
+void GlobalShortcutsBackendMacOS::MacMediaKeyPressed(const int key) {
 
   switch (key) {
     case NX_KEYTYPE_PLAY:
@@ -119,12 +121,12 @@ void GlobalShortcutsBackendMacOS::MacMediaKeyPressed(int key) {
 
 }
 
-bool GlobalShortcutsBackendMacOS::KeyPressed(const QKeySequence& sequence) {
+bool GlobalShortcutsBackendMacOS::KeyPressed(const QKeySequence &sequence) {
 
   if (sequence.isEmpty()) {
     return false;
   }
-  QAction* action = shortcuts_[sequence];
+  QAction *action = shortcuts_[sequence];
   if (action) {
     action->trigger();
     return true;
@@ -133,7 +135,7 @@ bool GlobalShortcutsBackendMacOS::KeyPressed(const QKeySequence& sequence) {
 
 }
 
-bool GlobalShortcutsBackendMacOS::IsAccessibilityEnabled() const {
+bool GlobalShortcutsBackendMacOS::IsAccessibilityEnabled() {
 
   NSDictionary *options = @{(id)kAXTrustedCheckOptionPrompt: @YES};
   return AXIsProcessTrustedWithOptions((CFDictionaryRef)options);
@@ -144,12 +146,12 @@ void GlobalShortcutsBackendMacOS::ShowAccessibilityDialog() {
 
   NSArray *paths = NSSearchPathForDirectoriesInDomains(NSPreferencePanesDirectory, NSSystemDomainMask, YES);
   if ([paths count] == 1) {
-    SBSystemPreferencesApplication* system_prefs = [SBApplication applicationWithBundleIdentifier:@"com.apple.systempreferences"];
+    SBSystemPreferencesApplication *system_prefs = [SBApplication applicationWithBundleIdentifier:@"com.apple.systempreferences"];
     [system_prefs activate];
 
-    SBElementArray* panes = [system_prefs panes];
-    SBSystemPreferencesPane* security_pane = nil;
-    for (SBSystemPreferencesPane* pane : panes) {
+    SBElementArray *panes = [system_prefs panes];
+    SBSystemPreferencesPane *security_pane = nil;
+    for (SBSystemPreferencesPane *pane : panes) {
       if ([ [pane id] isEqualToString:@"com.apple.preference.security"]) {
         security_pane = pane;
         break;
@@ -157,11 +159,12 @@ void GlobalShortcutsBackendMacOS::ShowAccessibilityDialog() {
     }
     [system_prefs setCurrentPane:security_pane];
 
-    SBElementArray* anchors = [security_pane anchors];
-    for (SBSystemPreferencesAnchor* anchor : anchors) {
+    SBElementArray *anchors = [security_pane anchors];
+    for (SBSystemPreferencesAnchor *anchor : anchors) {
       if ([ [anchor name] isEqualToString:@"Privacy_Accessibility"]) {
         [anchor reveal];
       }
     }
   }
+
 }
