@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This code was part of Clementine (GlobalSearch)
  * Copyright 2012, David Sansome <me@davidsansome.com>
- * Copyright 2018-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include <QUrl>
 #include <QImage>
 #include <QPixmap>
+#include <QPixmapCache>
 #include <QPainter>
 #include <QPalette>
 #include <QColor>
@@ -87,9 +88,6 @@
 #include "ui_internetsearchview.h"
 #include "settings/appearancesettingspage.h"
 
-using std::placeholders::_1;
-using std::placeholders::_2;
-
 const int InternetSearchView::kSwapModelsTimeoutMsec = 250;
 const int InternetSearchView::kDelayedSearchTimeoutMs = 200;
 const int InternetSearchView::kArtHeight = 32;
@@ -227,7 +225,7 @@ void InternetSearchView::ReloadSettings() {
 
   // Internet search settings
 
-  search_type_ = InternetSearchView::SearchType(s.value("type", int(InternetSearchView::SearchType_Artists)).toInt());
+  search_type_ = InternetSearchView::SearchType(s.value("type", static_cast<int>(InternetSearchView::SearchType_Artists)).toInt());
   switch (search_type_) {
     case InternetSearchView::SearchType_Artists:
       ui_->radiobutton_search_artists->setChecked(true);
@@ -243,9 +241,9 @@ void InternetSearchView::ReloadSettings() {
   int group_by_version = s.value("search_group_by_version", 0).toInt();
   if (group_by_version == 1 && s.contains("search_group_by1") && s.contains("search_group_by2") && s.contains("search_group_by3")) {
     SetGroupBy(CollectionModel::Grouping(
-        CollectionModel::GroupBy(s.value("search_group_by1", int(CollectionModel::GroupBy_AlbumArtist)).toInt()),
-        CollectionModel::GroupBy(s.value("search_group_by2", int(CollectionModel::GroupBy_AlbumDisc)).toInt()),
-        CollectionModel::GroupBy(s.value("search_group_by3", int(CollectionModel::GroupBy_None)).toInt())));
+        CollectionModel::GroupBy(s.value("search_group_by1", static_cast<int>(CollectionModel::GroupBy_AlbumArtist)).toInt()),
+        CollectionModel::GroupBy(s.value("search_group_by2", static_cast<int>(CollectionModel::GroupBy_AlbumDisc)).toInt()),
+        CollectionModel::GroupBy(s.value("search_group_by3", static_cast<int>(CollectionModel::GroupBy_None)).toInt())));
   }
   else {
     SetGroupBy(CollectionModel::Grouping(CollectionModel::GroupBy(CollectionModel::GroupBy_AlbumArtist), CollectionModel::GroupBy(CollectionModel::GroupBy_AlbumDisc), CollectionModel::GroupBy(CollectionModel::GroupBy_None)));
@@ -255,6 +253,7 @@ void InternetSearchView::ReloadSettings() {
   s.beginGroup(AppearanceSettingsPage::kSettingsGroup);
   int iconsize = s.value(AppearanceSettingsPage::kIconSizeConfigureButtons, 20).toInt();
   s.endGroup();
+
   ui_->settings->setIconSize(QSize(iconsize, iconsize));
   ui_->search->setIconSize(iconsize);
 
@@ -263,7 +262,10 @@ void InternetSearchView::ReloadSettings() {
 void InternetSearchView::showEvent(QShowEvent *e) {
 
   QWidget::showEvent(e);
+
+#ifndef Q_OS_MACOS
   FocusSearchField();
+#endif
 
 }
 
@@ -314,38 +316,41 @@ bool InternetSearchView::SearchKeyEvent(QKeyEvent *e) {
 
 bool InternetSearchView::ResultsContextMenuEvent(QContextMenuEvent *e) {
 
-  context_menu_ = new QMenu(this);
-  context_actions_ << context_menu_->addAction( IconLoader::Load("media-playback-start"), tr("Append to current playlist"), this, &InternetSearchView::AddSelectedToPlaylist);
-  context_actions_ << context_menu_->addAction( IconLoader::Load("media-playback-start"), tr("Replace current playlist"), this, &InternetSearchView::LoadSelected);
-  context_actions_ << context_menu_->addAction( IconLoader::Load("document-new"), tr("Open in new playlist"), this, &InternetSearchView::OpenSelectedInNewPlaylist);
+  if (!context_menu_) {
+    context_menu_ = new QMenu(this);
+    context_actions_ << context_menu_->addAction(IconLoader::Load("media-playback-start"), tr("Append to current playlist"), this, &InternetSearchView::AddSelectedToPlaylist);
+    context_actions_ << context_menu_->addAction(IconLoader::Load("media-playback-start"), tr("Replace current playlist"), this, &InternetSearchView::LoadSelected);
+    context_actions_ << context_menu_->addAction(IconLoader::Load("document-new"), tr("Open in new playlist"), this, &InternetSearchView::OpenSelectedInNewPlaylist);
 
-  context_menu_->addSeparator();
-  context_actions_ << context_menu_->addAction(IconLoader::Load("go-next"), tr("Queue track"), this, &InternetSearchView::AddSelectedToPlaylistEnqueue);
-
-  context_menu_->addSeparator();
-
-  if (service_->artists_collection_model() || service_->albums_collection_model() || service_->songs_collection_model()) {
-    if (service_->artists_collection_model()) {
-      context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to artists"), this, &InternetSearchView::AddArtists);
-    }
-    if (service_->albums_collection_model()) {
-      context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to albums"), this, &InternetSearchView::AddAlbums);
-    }
-    if (service_->songs_collection_model()) {
-      context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to songs"), this, &InternetSearchView::AddSongs);
-    }
     context_menu_->addSeparator();
+    context_actions_ << context_menu_->addAction(IconLoader::Load("go-next"), tr("Queue track"), this, &InternetSearchView::AddSelectedToPlaylistEnqueue);
+
+    context_menu_->addSeparator();
+
+    if (service_->artists_collection_model() || service_->albums_collection_model() || service_->songs_collection_model()) {
+      if (service_->artists_collection_model()) {
+        context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to artists"), this, &InternetSearchView::AddArtists);
+      }
+      if (service_->albums_collection_model()) {
+        context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to albums"), this, &InternetSearchView::AddAlbums);
+      }
+      if (service_->songs_collection_model()) {
+        context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to songs"), this, &InternetSearchView::AddSongs);
+      }
+      context_menu_->addSeparator();
+    }
+
+    if (ui_->results->selectionModel() && ui_->results->selectionModel()->selectedRows().length() == 1) {
+      context_actions_ << context_menu_->addAction(IconLoader::Load("search"), tr("Search for this"), this, &InternetSearchView::SearchForThis);
+    }
+
+    context_menu_->addSeparator();
+    context_menu_->addMenu(tr("Group by"))->addActions(group_by_actions_->actions());
+
+    context_menu_->addAction(IconLoader::Load("configure"), tr("Configure %1...").arg(Song::TextForSource(service_->source())), this, &InternetSearchView::OpenSettingsDialog);
+
   }
 
-  if (ui_->results->selectionModel() && ui_->results->selectionModel()->selectedRows().length() == 1) {
-    context_actions_ << context_menu_->addAction(IconLoader::Load("search"), tr("Search for this"), this, &InternetSearchView::SearchForThis);
-  }
-
-  context_menu_->addSeparator();
-  context_menu_->addMenu(tr("Group by"))->addActions(group_by_actions_->actions());
-
-  context_menu_->addAction(IconLoader::Load("configure"), tr("Configure %1...").arg(Song::TextForSource(service_->source())), this, &InternetSearchView::OpenSettingsDialog);
-
   const bool enable_context_actions = ui_->results->selectionModel() && ui_->results->selectionModel()->hasSelection();
 
   for (QAction *action : context_actions_) {
@@ -363,7 +368,7 @@ void InternetSearchView::timerEvent(QTimerEvent *e) {
   QMap<int, DelayedSearch>::iterator it = delayed_searches_.find(e->timerId());
   if (it != delayed_searches_.end()) {
     SearchAsync(it.value().id_, it.value().query_, it.value().type_);
-    delayed_searches_.erase(it);
+    delayed_searches_.erase(it);  // clazy:exclude=strict-iterators
     return;
   }
 
@@ -440,7 +445,7 @@ QStringList InternetSearchView::TokenizeQuery(const QString &query) {
     (*it).remove(')');
     (*it).remove('"');
 
-    const int colon = (*it).indexOf(":");
+    const qint64 colon = (*it).indexOf(":");
     if (colon != -1) {
       (*it).remove(0, colon + 1);
     }
@@ -482,7 +487,7 @@ void InternetSearchView::SearchAsync(const int id, const QString &query, const S
 
 }
 
-void InternetSearchView::SearchDone(const int service_id, const SongList &songs, const QString &error) {
+void InternetSearchView::SearchDone(const int service_id, const SongMap &songs, const QString &error) {
 
   if (!pending_searches_.contains(service_id)) return;
 
@@ -496,6 +501,7 @@ void InternetSearchView::SearchDone(const int service_id, const SongList &songs,
   }
 
   ResultList results;
+  results.reserve(songs.count());
   for (const Song &song : songs) {
     Result result;
     result.metadata_ = song;
@@ -503,7 +509,7 @@ void InternetSearchView::SearchDone(const int service_id, const SongList &songs,
   }
 
   // Load cached pixmaps into the results
-  for (InternetSearchView::ResultList::iterator it = results.begin() ; it != results.end() ; ++it) {
+  for (InternetSearchView::ResultList::iterator it = results.begin(); it != results.end(); ++it) {
     it->pixmap_cache_key_ = PixmapCacheKey(*it);
   }
 
@@ -517,7 +523,7 @@ void InternetSearchView::CancelSearch(const int id) {
   for (it = delayed_searches_.begin(); it != delayed_searches_.end(); ++it) {
     if (it.value().id_ == id) {
       killTimer(it.key());
-      delayed_searches_.erase(it);
+      delayed_searches_.erase(it);  // clazy:exclude=strict-iterators
       return;
     }
   }
@@ -588,10 +594,10 @@ MimeData *InternetSearchView::SelectedMimeData() {
   QModelIndexList indexes = ui_->results->selectionModel()->selectedRows();
   if (indexes.isEmpty()) {
     // There's nothing selected - take the first thing in the model that isn't a divider.
-    for (int i = 0 ; i < front_proxy_->rowCount() ; ++i) {
+    for (int i = 0; i < front_proxy_->rowCount(); ++i) {
       QModelIndex idx = front_proxy_->index(i, 0);
       if (!idx.data(CollectionModel::Role_IsDivider).toBool()) {
-        indexes << idx;
+        indexes << idx;  // clazy:exclude=reserve-candidates
         ui_->results->setCurrentIndex(idx);
         break;
       }
@@ -606,7 +612,7 @@ MimeData *InternetSearchView::SelectedMimeData() {
   // Get items for these indexes
   QList<QStandardItem*> items;
   for (const QModelIndex &idx : indexes) {
-    items << (front_model_->itemFromIndex(front_proxy_->mapToSource(idx)));
+    items << (front_model_->itemFromIndex(front_proxy_->mapToSource(idx)));  // clazy:exclude=reserve-candidates
   }
 
   // Get a MimeData for these items
@@ -652,6 +658,12 @@ void InternetSearchView::SearchForThis() {
   StartSearch(ui_->results->selectionModel()->selectedRows().first().data().toString());
 }
 
+bool InternetSearchView::SearchFieldHasFocus() const {
+
+  return ui_->search->hasFocus();
+
+}
+
 void InternetSearchView::FocusSearchField() {
 
   ui_->search->setFocus();
@@ -674,7 +686,7 @@ void InternetSearchView::GroupByClicked(QAction *action) {
 
   if (action->property("group_by").isNull()) {
     if (!group_by_dialog_) {
-      group_by_dialog_.reset(new GroupByDialog);
+      group_by_dialog_ = std::make_unique<GroupByDialog>();
       QObject::connect(group_by_dialog_.get(), &GroupByDialog::Accepted, this, &InternetSearchView::SetGroupBy);
     }
 
@@ -686,7 +698,7 @@ void InternetSearchView::GroupByClicked(QAction *action) {
 
 }
 
-void InternetSearchView::SetGroupBy(const CollectionModel::Grouping &g) {
+void InternetSearchView::SetGroupBy(const CollectionModel::Grouping g) {
 
   // Clear requests: changing "group by" on the models will cause all the items to be removed/added again,
   // so all the QModelIndex here will become invalid. New requests will be created for those
@@ -701,9 +713,9 @@ void InternetSearchView::SetGroupBy(const CollectionModel::Grouping &g) {
   QSettings s;
   s.beginGroup(service_->settings_group());
   s.setValue("search_group_by_version", 1);
-  s.setValue("search_group_by1", int(g.first));
-  s.setValue("search_group_by2", int(g.second));
-  s.setValue("search_group_by3", int(g.third));
+  s.setValue("search_group_by1", static_cast<int>(g.first));
+  s.setValue("search_group_by2", static_cast<int>(g.second));
+  s.setValue("search_group_by3", static_cast<int>(g.third));
   s.endGroup();
 
   // Make sure the correct action is checked.
@@ -717,7 +729,8 @@ void InternetSearchView::SetGroupBy(const CollectionModel::Grouping &g) {
   }
 
   // Check the advanced action
-  group_by_actions_->actions().last()->setChecked(true);
+  QList<QAction*> actions = group_by_actions_->actions();
+  actions.last()->setChecked(true);
 
 }
 
@@ -739,7 +752,7 @@ void InternetSearchView::SetSearchType(const InternetSearchView::SearchType type
 
   QSettings s;
   s.beginGroup(service_->settings_group());
-  s.setValue("type", int(search_type_));
+  s.setValue("type", static_cast<int>(search_type_));
   s.endGroup();
 
   TextEdited(ui_->search->text());
@@ -751,7 +764,7 @@ void InternetSearchView::AddArtists() {
   MimeData *mimedata = SelectedMimeData();
   if (!mimedata) return;
   if (const InternetSongMimeData *internet_song_data = qobject_cast<const InternetSongMimeData*>(mimedata)) {
-    emit AddArtistsSignal(internet_song_data->songs);
+    emit AddArtistsSignal(internet_song_data->songs.values());
   }
 
 }
@@ -761,7 +774,7 @@ void InternetSearchView::AddAlbums() {
   MimeData *mimedata = SelectedMimeData();
   if (!mimedata) return;
   if (const InternetSongMimeData *internet_song_data = qobject_cast<const InternetSongMimeData*>(mimedata)) {
-    emit AddAlbumsSignal(internet_song_data->songs);
+    emit AddAlbumsSignal(internet_song_data->songs.values());
   }
 
 }
@@ -791,7 +804,7 @@ QString InternetSearchView::PixmapCacheKey(const InternetSearchView::Result &res
 }
 
 bool InternetSearchView::FindCachedPixmap(const InternetSearchView::Result &result, QPixmap *pixmap) const {
-  return pixmap_cache_.find(result.pixmap_cache_key_, pixmap);
+  return QPixmapCache::find(result.pixmap_cache_key_, pixmap);
 }
 
 void InternetSearchView::LazyLoadAlbumCover(const QModelIndex &proxy_index) {
@@ -829,7 +842,7 @@ void InternetSearchView::LazyLoadAlbumCover(const QModelIndex &proxy_index) {
 
   // Walk down the item's children until we find a track
   QStandardItem *item_song = item_album;
-  while (item_song->rowCount()) {
+  while (item_song->rowCount() > 0) {
     item_song = item_song->child(0);
   }
 
@@ -837,7 +850,7 @@ void InternetSearchView::LazyLoadAlbumCover(const QModelIndex &proxy_index) {
   const InternetSearchView::Result result = item_song->data(InternetSearchModel::Role_Result).value<InternetSearchView::Result>();
 
   QPixmap cached_pixmap;
-  if (pixmap_cache_.find(result.pixmap_cache_key_, &cached_pixmap)) {
+  if (QPixmapCache::find(result.pixmap_cache_key_, &cached_pixmap)) {
     item_album->setData(cached_pixmap, Qt::DecorationRole);
   }
   else {
@@ -858,7 +871,7 @@ void InternetSearchView::AlbumCoverLoaded(const quint64 id, const AlbumCoverLoad
   if (albumcover_result.success && !albumcover_result.image_scaled.isNull()) {
     QPixmap pixmap = QPixmap::fromImage(albumcover_result.image_scaled);
     if (!pixmap.isNull()) {
-      pixmap_cache_.insert(key, pixmap);
+      QPixmapCache::insert(key, pixmap);
     }
     if (idx.isValid()) {
       QStandardItem *item = front_model_->itemFromIndex(idx);
