@@ -21,6 +21,8 @@
 #ifndef INTERNETSONGMIMEDATA_H
 #define INTERNETSONGMIMEDATA_H
 
+#include <QMap>
+
 #include "core/mimedata.h"
 #include "core/song.h"
 
@@ -30,10 +32,10 @@ class InternetSongMimeData : public MimeData {
   Q_OBJECT
 
  public:
-  explicit InternetSongMimeData(InternetService *_service) : service(_service) {}
+  explicit InternetSongMimeData(InternetService *_service, QObject* = nullptr) : service(_service) {}
 
   InternetService *service;
-  SongList songs;
+  SongMap songs;
 };
 
 #endif  // INTERNETSONGMIMEDATA_H
