@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <QtGlobal>
 #include <QWidget>
+#include <QMap>
 #include <QString>
 #include <QStackedWidget>
 #include <QContextMenuEvent>
@@ -46,24 +47,23 @@ InternetSongsView::InternetSongsView(Application *app, InternetService *service,
       service_(service),
       settings_group_(settings_group),
       settings_page_(settings_page),
-      ui_(new Ui_InternetCollectionViewContainer)
-      {
+      ui_(new Ui_InternetCollectionViewContainer) {
 
   ui_->setupUi(this);
 
   ui_->stacked->setCurrentWidget(ui_->internetcollection_page);
   ui_->view->Init(app_, service_->songs_collection_backend(), service_->songs_collection_model(), false);
   ui_->view->setModel(service_->songs_collection_sort_model());
-  ui_->view->SetFilter(ui_->filter);
-  ui_->filter->SetSettingsGroup(settings_group);
-  ui_->filter->SetCollectionModel(service_->songs_collection_model());
+  ui_->view->SetFilter(ui_->filter_widget);
+  ui_->filter_widget->SetSettingsGroup(settings_group);
+  ui_->filter_widget->Init(service_->songs_collection_model());
 
   QAction *action_configure = new QAction(IconLoader::Load("configure"), tr("Configure %1...").arg(Song::TextForSource(service_->source())), this);
   QObject::connect(action_configure, &QAction::triggered, this, &InternetSongsView::OpenSettingsDialog);
-  ui_->filter->AddMenuAction(action_configure);
+  ui_->filter_widget->AddMenuAction(action_configure);
 
   QObject::connect(ui_->view, &InternetCollectionView::GetSongs, this, &InternetSongsView::GetSongs);
-  QObject::connect(ui_->view, &InternetCollectionView::RemoveSongs, service_, &InternetService::RemoveSongs);
+  QObject::connect(ui_->view, &InternetCollectionView::RemoveSongs, service_, &InternetService::RemoveSongsByList);
 
   QObject::connect(ui_->refresh, &QPushButton::clicked, this, &InternetSongsView::GetSongs);
   QObject::connect(ui_->close, &QPushButton::clicked, this, &InternetSongsView::AbortGetSongs);
@@ -87,7 +87,7 @@ InternetSongsView::~InternetSongsView() { delete ui_; }
 
 void InternetSongsView::ReloadSettings() {
 
-  ui_->filter->ReloadSettings();
+  ui_->filter_widget->ReloadSettings();
   ui_->view->ReloadSettings();
 
 }
@@ -122,7 +122,7 @@ void InternetSongsView::AbortGetSongs() {
 
 }
 
-void InternetSongsView::SongsFinished(const SongList &songs, const QString &error) {
+void InternetSongsView::SongsFinished(const SongMap &songs, const QString &error) {
 
   if (songs.isEmpty() && !error.isEmpty()) {
     ui_->status->setText(error);
@@ -132,10 +132,9 @@ void InternetSongsView::SongsFinished(const SongList &songs, const QString &erro
     ui_->close->show();
   }
   else {
-    service_->songs_collection_backend()->DeleteAll();
     ui_->stacked->setCurrentWidget(ui_->internetcollection_page);
     ui_->status->clear();
-    service_->songs_collection_backend()->AddOrUpdateSongsAsync(songs);
+    service_->songs_collection_backend()->UpdateSongsBySongIDAsync(songs);
   }
 
 }
