@@ -2,7 +2,7 @@
  * This file was part of Clementine.
  * Copyright 2012, 2014, John Maguire <john.maguire@gmail.com>
  * Copyright 2014, Krzysztof Sobiecki <sobkas@gmail.com>
- * Copyright 2018-2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,8 +57,7 @@ LocalRedirectServer::LocalRedirectServer(QObject *parent)
     : QTcpServer(parent),
       https_(false),
       port_(0),
-      socket_(nullptr)
-      {}
+      socket_(nullptr) {}
 
 LocalRedirectServer::~LocalRedirectServer() {
   if (isListening()) close();
@@ -71,7 +70,7 @@ bool LocalRedirectServer::GenerateCertificate() {
     return false;
   }
 
-  gnutls_x509_privkey_t key;
+  gnutls_x509_privkey_t key = nullptr;
   if (int result = gnutls_x509_privkey_init(&key) != GNUTLS_E_SUCCESS) {
     error_ = QString("Failed to initialize the private key structure: %1").arg(gnutls_strerror(result));
     gnutls_global_deinit();
@@ -96,7 +95,7 @@ bool LocalRedirectServer::GenerateCertificate() {
     return false;
   }
 
-  QSslKey ssl_key(QByteArray(buffer, buffer_size), QSsl::Rsa);
+  QSslKey ssl_key(QByteArray(buffer, static_cast<qint64>(buffer_size)), QSsl::Rsa);
   if (ssl_key.isNull()) {
     error_ = QString("Failed to generate random private key.");
     gnutls_x509_privkey_deinit(key);
@@ -104,7 +103,7 @@ bool LocalRedirectServer::GenerateCertificate() {
     return false;
   }
 
-  gnutls_x509_crt_t crt;
+  gnutls_x509_crt_t crt = nullptr;
   if (int result = gnutls_x509_crt_init(&crt) != GNUTLS_E_SUCCESS) {
     error_ = QString("Failed to initialize an X.509 certificate structure: %1").arg(gnutls_strerror(result));
     gnutls_x509_privkey_deinit(key);
@@ -147,8 +146,8 @@ bool LocalRedirectServer::GenerateCertificate() {
     return false;
   }
   quint64 time = QDateTime::currentDateTime().toSecsSinceEpoch();
-  gnutls_x509_crt_set_activation_time(crt, time);
-  if (int result = gnutls_x509_crt_set_expiration_time(crt, time + 31536000L) != GNUTLS_E_SUCCESS) {
+  gnutls_x509_crt_set_activation_time(crt, static_cast<time_t>(time));
+  if (int result = gnutls_x509_crt_set_expiration_time(crt, static_cast<time_t>(time + 31536000L)) != GNUTLS_E_SUCCESS) {
     error_ = QString("Failed to set the activation time of the certificate: %1").arg(gnutls_strerror(result));
     gnutls_x509_privkey_deinit(key);
     gnutls_x509_crt_deinit(crt);
@@ -173,7 +172,7 @@ bool LocalRedirectServer::GenerateCertificate() {
     return false;
   }
 
-  gnutls_privkey_t pkey;
+  gnutls_privkey_t pkey = nullptr;
   if (int result = gnutls_privkey_init(&pkey) != GNUTLS_E_SUCCESS) {
     error_ = QString("Failed to initialize a private key object: %1").arg(gnutls_strerror(result));
     gnutls_x509_privkey_deinit(key);
@@ -221,7 +220,7 @@ bool LocalRedirectServer::GenerateCertificate() {
   gnutls_x509_privkey_deinit(key);
   gnutls_privkey_deinit(pkey);
 
-  QSslCertificate ssl_certificate(QByteArray(buffer, buffer_size));
+  QSslCertificate ssl_certificate(QByteArray(buffer, static_cast<qint64>(buffer_size)));
   if (ssl_certificate.isNull()) {
     error_ = "Failed to generate random client certificate.";
     gnutls_global_deinit();
@@ -342,11 +341,12 @@ void LocalRedirectServer::ReadyRead() {
 void LocalRedirectServer::WriteTemplate() const {
 
   QFile page_file(":/html/oauthsuccess.html");
-  page_file.open(QIODevice::ReadOnly);
+  if (!page_file.open(QIODevice::ReadOnly)) return;
   QString page_data = QString::fromUtf8(page_file.readAll());
+  page_file.close();
 
   QRegularExpression tr_regexp("tr\\(\"([^\"]+)\"\\)");
-  int offset = 0;
+  qint64 offset = 0;
   forever {
     QRegularExpressionMatch re_match = tr_regexp.match(page_data, offset);
     if (!re_match.hasMatch()) break;
@@ -360,13 +360,15 @@ void LocalRedirectServer::WriteTemplate() const {
   }
 
   QBuffer image_buffer;
-  image_buffer.open(QIODevice::ReadWrite);
-  QApplication::style()
-      ->standardIcon(QStyle::SP_DialogOkButton)
-      .pixmap(16)
-      .toImage()
-      .save(&image_buffer, "PNG");
-  page_data.replace("@IMAGE_DATA@", image_buffer.data().toBase64());
+  if (image_buffer.open(QIODevice::ReadWrite)) {
+    QApplication::style()
+        ->standardIcon(QStyle::SP_DialogOkButton)
+        .pixmap(16)
+        .toImage()
+        .save(&image_buffer, "PNG");
+    page_data.replace("@IMAGE_DATA@", image_buffer.data().toBase64());
+    image_buffer.close();
+  }
 
   socket_->write("HTTP/1.0 200 OK\r\n");
   socket_->write("Content-type: text/html;charset=UTF-8\r\n");
