@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2020-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,15 +46,15 @@ class GeniusLyricsProvider : public JsonLyricsProvider {
   Q_OBJECT
 
  public:
-  explicit GeniusLyricsProvider(QObject *parent = nullptr);
+  explicit GeniusLyricsProvider(NetworkAccessManager *network, QObject *parent = nullptr);
   ~GeniusLyricsProvider() override;
 
   bool IsAuthenticated() const override { return !access_token_.isEmpty(); }
   void Authenticate() override;
   void Deauthenticate() override { access_token_.clear(); }
 
-  bool StartSearch(const QString &artist, const QString &album, const QString &title, quint64 id) override;
-  void CancelSearch(const quint64 id) override;
+  bool StartSearch(const QString &artist, const QString &album, const QString &title, int id) override;
+  void CancelSearch(const int id) override;
 
  public:
   struct GeniusLyricsLyricContext {
@@ -76,13 +76,13 @@ class GeniusLyricsProvider : public JsonLyricsProvider {
   void RequestAccessToken(const QUrl &url, const QUrl &redirect_url);
   void AuthError(const QString &error = QString(), const QVariant &debug = QVariant());
   void Error(const QString &error, const QVariant &debug = QVariant()) override;
-  void EndSearch(std::shared_ptr<GeniusLyricsSearchContext> search, const GeniusLyricsLyricContext lyric = GeniusLyricsLyricContext());
+  void EndSearch(std::shared_ptr<GeniusLyricsSearchContext> search, const GeniusLyricsLyricContext &lyric = GeniusLyricsLyricContext());
 
  private slots:
-  void HandleLoginSSLErrors(QList<QSslError> ssl_errors);
+  void HandleLoginSSLErrors(const QList<QSslError> &ssl_errors);
   void RedirectArrived();
   void AccessTokenRequestFinished(QNetworkReply *reply);
-  void HandleSearchReply(QNetworkReply *reply, const quint64 id);
+  void HandleSearchReply(QNetworkReply *reply, const int id);
   void HandleLyricReply(QNetworkReply *reply, const int search_id, const QUrl &url);
 
  private:
@@ -95,7 +95,6 @@ class GeniusLyricsProvider : public JsonLyricsProvider {
   static const char *kUrlSearch;
 
  private:
-  NetworkAccessManager *network_;
   LocalRedirectServer *server_;
   QString code_verifier_;
   QString code_challenge_;
