@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <QString>
 #include <QUrl>
 #include <QUrlQuery>
-#include <QNetworkAccessManager>
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QXmlStreamReader>
@@ -42,7 +41,7 @@
 
 const char *LoloLyricsProvider::kUrlSearch = "http://api.lololyrics.com/0.5/getLyric";
 
-LoloLyricsProvider::LoloLyricsProvider(QObject *parent) : LyricsProvider("LoloLyrics", true, false, parent), network_(new NetworkAccessManager(this)) {}
+LoloLyricsProvider::LoloLyricsProvider(NetworkAccessManager *network, QObject *parent) : LyricsProvider("LoloLyrics", true, false, network, parent) {}
 
 LoloLyricsProvider::~LoloLyricsProvider() {
 
@@ -55,7 +54,7 @@ LoloLyricsProvider::~LoloLyricsProvider() {
 
 }
 
-bool LoloLyricsProvider::StartSearch(const QString &artist, const QString &album, const QString &title, const quint64 id) {
+bool LoloLyricsProvider::StartSearch(const QString &artist, const QString &album, const QString &title, const int id) {
 
   Q_UNUSED(album);
 
@@ -77,7 +76,7 @@ bool LoloLyricsProvider::StartSearch(const QString &artist, const QString &album
 #endif
   QNetworkReply *reply = network_->get(req);
   replies_ << reply;
-  QObject::connect(reply, &QNetworkReply::finished, [this, reply, id, artist, title]() { HandleSearchReply(reply, id, artist, title); });
+  QObject::connect(reply, &QNetworkReply::finished, this, [this, reply, id, artist, title]() { HandleSearchReply(reply, id, artist, title); });
 
   //qLog(Debug) << "LoloLyrics: Sending request for" << url;
 
@@ -85,9 +84,9 @@ bool LoloLyricsProvider::StartSearch(const QString &artist, const QString &album
 
 }
 
-void LoloLyricsProvider::CancelSearch(const quint64 id) { Q_UNUSED(id); }
+void LoloLyricsProvider::CancelSearch(const int id) { Q_UNUSED(id); }
 
-void LoloLyricsProvider::HandleSearchReply(QNetworkReply *reply, const quint64 id, const QString &artist, const QString &title) {
+void LoloLyricsProvider::HandleSearchReply(QNetworkReply *reply, const int id, const QString &artist, const QString &title) {
 
   if (!replies_.contains(reply)) return;
   replies_.removeAll(reply);
@@ -136,13 +135,13 @@ void LoloLyricsProvider::HandleSearchReply(QNetworkReply *reply, const quint64 i
         }
       }
       else if (type == QXmlStreamReader::EndElement) {
-         if (name == "result") {
-           if (!result.lyrics.isEmpty()) {
-             result.lyrics = Utilities::DecodeHtmlEntities(result.lyrics);
-             results << result;
-           }
-           result = LyricsSearchResult();
-         }
+        if (name == "result") {
+          if (!result.lyrics.isEmpty()) {
+            result.lyrics = Utilities::DecodeHtmlEntities(result.lyrics);
+            results << result;
+          }
+          result = LyricsSearchResult();
+        }
       }
     }
   }
