@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +47,8 @@ LyricsFetcherSearch::LyricsFetcherSearch(const LyricsSearchRequest &request, QOb
 
 void LyricsFetcherSearch::TerminateSearch() {
 
-  for (int id : pending_requests_.keys()) {
+  QList<int> keys = pending_requests_.keys();
+  for (const int id : keys) {
     pending_requests_.take(id)->CancelSearch(id);
   }
   AllProvidersFinished();
@@ -57,7 +58,7 @@ void LyricsFetcherSearch::TerminateSearch() {
 void LyricsFetcherSearch::Start(LyricsProviders *lyrics_providers) {
 
   // Ignore Radio Paradise "commercial" break.
-  if (request_.artist.toLower() == "commercial-free" && request_.title.toLower() == "listener-supported") {
+  if (request_.artist.compare("commercial-free", Qt::CaseInsensitive) == 0 && request_.title.compare("listener-supported", Qt::CaseInsensitive) == 0) {
     TerminateSearch();
     return;
   }
@@ -77,26 +78,26 @@ void LyricsFetcherSearch::Start(LyricsProviders *lyrics_providers) {
 
 }
 
-void LyricsFetcherSearch::ProviderSearchFinished(const quint64 id, const LyricsSearchResults &results) {
+void LyricsFetcherSearch::ProviderSearchFinished(const int id, const LyricsSearchResults &results) {
 
   if (!pending_requests_.contains(id)) return;
   LyricsProvider *provider = pending_requests_.take(id);
 
   LyricsSearchResults results_copy(results);
   float higest_score = 0.0;
-  for (int i = 0 ; i < results_copy.count() ; ++i) {
+  for (int i = 0; i < results_copy.count(); ++i) {
     results_copy[i].provider = provider->name();
     results_copy[i].score = 0.0;
-    if (results_copy[i].artist.toLower() == request_.artist.toLower()) {
+    if (results_copy[i].artist.compare(request_.artist, Qt::CaseInsensitive) == 0) {
       results_copy[i].score += 0.5;
     }
-    if (results_copy[i].album.toLower() == request_.album.toLower()) {
+    if (results_copy[i].album.compare(request_.album, Qt::CaseInsensitive) == 0) {
       results_copy[i].score += 0.5;
     }
-    if (results_copy[i].title.toLower() == request_.title.toLower()) {
+    if (results_copy[i].title.compare(request_.title, Qt::CaseInsensitive) == 0) {
       results_copy[i].score += 0.5;
     }
-    if (results_copy[i].artist.toLower() != request_.artist.toLower() && results_copy[i].title.toLower() != request_.title.toLower()) {
+    if (results_copy[i].artist.compare(request_.artist, Qt::CaseInsensitive) != 0 && results_copy[i].title.compare(request_.title, Qt::CaseInsensitive) != 0) {
       results_copy[i].score -= 1.5;
     }
     if (results_copy[i].lyrics.length() > kGoodLyricsLength) results_copy[i].score += 1.0;
@@ -107,7 +108,7 @@ void LyricsFetcherSearch::ProviderSearchFinished(const quint64 id, const LyricsS
   std::stable_sort(results_.begin(), results_.end(), LyricsSearchResultCompareScore);
 
   if (!pending_requests_.isEmpty()) {
-    if (!results_.isEmpty() && higest_score >= kHighScore) { // Highest score, no need to wait for other providers.
+    if (!results_.isEmpty() && higest_score >= kHighScore) {  // Highest score, no need to wait for other providers.
       qLog(Debug) << "Got lyrics with high score from" << results_.last().provider << "for" << request_.artist << request_.title << "score" << results_.last().score << "finishing search.";
       TerminateSearch();
     }
