@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,11 +31,13 @@
 
 #include "lyricsfetcher.h"
 
+class NetworkAccessManager;
+
 class LyricsProvider : public QObject {
   Q_OBJECT
 
  public:
-  explicit LyricsProvider(const QString &name, const bool enabled, const bool authentication_required, QObject *parent);
+  explicit LyricsProvider(const QString &name, const bool enabled, const bool authentication_required, NetworkAccessManager *network, QObject *parent);
 
   typedef QPair<QString, QString> Param;
   typedef QList<Param> ParamList;
@@ -47,8 +49,8 @@ class LyricsProvider : public QObject {
   void set_enabled(const bool enabled) { enabled_ = enabled; }
   void set_order(const int order) { order_ = order; }
 
-  virtual bool StartSearch(const QString &artist, const QString &album, const QString &title, const quint64 id) = 0;
-  virtual void CancelSearch(const quint64 id) { Q_UNUSED(id); }
+  virtual bool StartSearch(const QString &artist, const QString &album, const QString &title, const int id) = 0;
+  virtual void CancelSearch(const int id) { Q_UNUSED(id); }
   virtual bool AuthenticationRequired() const { return authentication_required_; }
   virtual void Authenticate() {}
   virtual bool IsAuthenticated() const { return !authentication_required_; }
@@ -62,7 +64,8 @@ class LyricsProvider : public QObject {
   void AuthenticationFailure(QStringList);
   void SearchFinished(quint64 id, LyricsSearchResults results);
 
- private:
+ protected:
+  NetworkAccessManager *network_;
   QString name_;
   bool enabled_;
   int order_;
