@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -83,16 +83,20 @@
 #endif  // HAVE_QTSPARKLE
 
 #ifdef HAVE_DBUS
-#  include "core/mpris.h"
+#  include "core/mpris2.h"
 #endif
 #include "core/utilities.h"
 #include "core/metatypes.h"
 #include "core/iconloader.h"
 #include "core/mainwindow.h"
 #include "core/commandlineoptions.h"
-#include "core/systemtrayicon.h"
 #include "core/application.h"
 #include "core/networkproxyfactory.h"
+#ifdef Q_OS_MACOS
+#  include "core/macsystemtrayicon.h"
+#else
+#  include "core/qtsystemtrayicon.h"
+#endif
 #ifdef HAVE_TRANSLATIONS
 #  include "core/translations.h"
 #endif
@@ -107,7 +111,7 @@
 #  include "osd/osdbase.h"
 #endif
 
-int main(int argc, char* argv[]) {
+int main(int argc, char *argv[]) {
 
 #ifdef Q_OS_MACOS
   // Do Mac specific startup to get media keys working.
@@ -165,7 +169,7 @@ int main(int argc, char* argv[]) {
 
   // Output the version, so when people attach log output to bug reports they don't have to tell us which version they're using.
   qLog(Info) << "Strawberry" << STRAWBERRY_VERSION_DISPLAY;
-  qLog(Info) << QString("%1 %2 - (%3 %4) [%5]").arg(QSysInfo::prettyProductName()).arg(QSysInfo::productVersion()).arg(QSysInfo::kernelType()).arg(QSysInfo::kernelVersion()).arg(QSysInfo::currentCpuArchitecture());
+  qLog(Info) << QString("%1 %2 - (%3 %4) [%5]").arg(QSysInfo::prettyProductName(), QSysInfo::productVersion(), QSysInfo::kernelType(), QSysInfo::kernelVersion(), QSysInfo::currentCpuArchitecture());
 
   // Seed the random number generators.
   time_t t = time(nullptr);
@@ -188,7 +192,7 @@ int main(int argc, char* argv[]) {
     }
     return 0;
   }
-  a.setQuitOnLastWindowClosed(false);
+  QGuiApplication::setQuitOnLastWindowClosed(false);
 
 #if defined(USE_BUNDLE) && (defined(Q_OS_LINUX) || defined(Q_OS_MACOS))
   qLog(Debug) << "Looking for resources in" << QCoreApplication::applicationDirPath() + "/" + USE_BUNDLE_DIR;
@@ -218,7 +222,12 @@ int main(int argc, char* argv[]) {
     // Create the file if it doesn't exist already
     if (!QFile::exists(s.fileName())) {
       QFile file(s.fileName());
-      file.open(QIODevice::WriteOnly);
+      if (file.open(QIODevice::WriteOnly)) {
+        file.close();
+      }
+      else {
+        qLog(Error) << "Could not open settings file" << file.fileName() << "for writing:" << file.errorString();
+      }
     }
 
     // Set -rw-------
@@ -233,7 +242,7 @@ int main(int argc, char* argv[]) {
   Q_INIT_RESOURCE(translations);
 #endif
 
-#ifdef DEBUG
+#ifndef QT_NO_DEBUG_OUTPUT
   QLoggingCategory::defaultCategory()->setEnabled(QtDebugMsg, true);
 #endif
 
@@ -263,7 +272,7 @@ int main(int argc, char* argv[]) {
 #endif
   translations->LoadTranslation("strawberry", ":/translations", language);
   translations->LoadTranslation("strawberry", TRANSLATIONS_DIR, language);
-  translations->LoadTranslation("strawberry", a.applicationDirPath(), language);
+  translations->LoadTranslation("strawberry", QCoreApplication::applicationDirPath(), language);
   translations->LoadTranslation("strawberry", QDir::currentPath(), language);
 
 #ifdef HAVE_QTSPARKLE
@@ -278,31 +287,33 @@ int main(int argc, char* argv[]) {
   QNetworkProxyFactory::setApplicationProxyFactory(NetworkProxyFactory::Instance());
 
   // Create the tray icon and OSD
-  std::unique_ptr<SystemTrayIcon> tray_icon(SystemTrayIcon::CreateSystemTrayIcon());
+  std::shared_ptr<SystemTrayIcon> tray_icon = std::make_shared<SystemTrayIcon>();
+
 #if defined(Q_OS_MACOS)
-  OSDMac osd(tray_icon.get(), &app);
+  OSDMac osd(tray_icon, &app);
 #elif defined(HAVE_DBUS)
-  OSDDBus osd(tray_icon.get(), &app);
+  OSDDBus osd(tray_icon, &app);
 #else
-  OSDBase osd(tray_icon.get(), &app);
+  OSDBase osd(tray_icon, &app);
 #endif
 
 #ifdef HAVE_DBUS
-  mpris::Mpris mpris(&app);
+  mpris::Mpris2 mpris2(&app);
 #endif
 
   // Window
-  MainWindow w(&app, tray_icon.get(), &osd, options);
+  MainWindow w(&app, tray_icon, &osd, options);
+
 #ifdef Q_OS_MACOS
   mac::EnableFullScreen(w);
 #endif  // Q_OS_MACOS
 
 #ifdef HAVE_DBUS
-  QObject::connect(&mpris, &mpris::Mpris::RaiseMainWindow, &w, &MainWindow::Raise);
+  QObject::connect(&mpris2, &mpris::Mpris2::RaiseMainWindow, &w, &MainWindow::Raise);
 #endif
   QObject::connect(&a, &SingleApplication::receivedMessage, &w, QOverload<quint32, const QByteArray&>::of(&MainWindow::CommandlineOptionsReceived));
 
-  int ret = a.exec();
+  int ret = QCoreApplication::exec();
 
   return ret;
 }
