@@ -34,8 +34,8 @@
 
 MoodbarController::MoodbarController(Application *app, QObject *parent)
     : QObject(parent),
-    app_(app),
-    enabled_(false) {
+      app_(app),
+      enabled_(false) {
 
   QObject::connect(app_->playlist_manager(), &PlaylistManager::CurrentSongChanged, this, &MoodbarController::CurrentSongChanged);
   QObject::connect(app_->player(), &Player::Stopped, this, &MoodbarController::PlaybackStopped);
@@ -59,7 +59,7 @@ void MoodbarController::CurrentSongChanged(const Song &song) {
 
   QByteArray data;
   MoodbarPipeline *pipeline = nullptr;
-  const MoodbarLoader::Result result = app_->moodbar_loader()->Load(song.url(), &data, &pipeline);
+  const MoodbarLoader::Result result = app_->moodbar_loader()->Load(song.url(), song.has_cue(), &data, &pipeline);
 
   switch (result) {
     case MoodbarLoader::CannotLoad:
@@ -95,7 +95,7 @@ void MoodbarController::AsyncLoadComplete(MoodbarPipeline *pipeline, const QUrl
     return;
   }
   // Did we stop the song?
-  switch(app_->player()->GetState()) {
+  switch (app_->player()->GetState()) {
     case Engine::Error:
     case Engine::Empty:
     case Engine::Idle:
