@@ -15,6 +15,8 @@
    along with Strawberry.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <algorithm>
+
 #include <QApplication>
 #include <QtConcurrentRun>
 #include <QFuture>
@@ -96,15 +98,19 @@ void MoodbarItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
 
 }
 
-QPixmap MoodbarItemDelegate::PixmapForIndex(const QModelIndex &idx, const QSize &size) {
+QPixmap MoodbarItemDelegate::PixmapForIndex(const QModelIndex &idx, const QSize size) {
 
   // Pixmaps are keyed off URL.
-  const QUrl url(idx.sibling(idx.row(), Playlist::Column_Filename).data().toUrl());
+  const QUrl url = idx.sibling(idx.row(), Playlist::Column_Filename).data().toUrl();
+  const bool has_cue = idx.sibling(idx.row(), Playlist::Column_HasCUE).data().toBool();
 
-  Data *data = data_[url];
-  if (!data) {
+  Data *data = nullptr;
+  if (data_.contains(url)) {
+    data = data_[url];
+  }
+  else {
     data = new Data;
-    data_.insert(url, data);
+    if (!data_.insert(url, data)) return QPixmap();
   }
 
   data->indexes_.insert(idx);
@@ -130,20 +136,20 @@ QPixmap MoodbarItemDelegate::PixmapForIndex(const QModelIndex &idx, const QSize
   }
 
   // We have to start loading the data from scratch.
-  StartLoadingData(url, data);
+  StartLoadingData(url, has_cue, data);
 
   return QPixmap();
 
 }
 
-void MoodbarItemDelegate::StartLoadingData(const QUrl &url, Data *data) {
+void MoodbarItemDelegate::StartLoadingData(const QUrl &url, const bool has_cue, Data *data) {
 
   data->state_ = Data::State_LoadingData;
 
   // Load a mood file for this song and generate some colors from it
   QByteArray bytes;
   MoodbarPipeline *pipeline = nullptr;
-  switch (app_->moodbar_loader()->Load(url, &bytes, &pipeline)) {
+  switch (app_->moodbar_loader()->Load(url, has_cue, &bytes, &pipeline)) {
     case MoodbarLoader::CannotLoad:
       data->state_ = Data::State_CannotLoad;
       break;
@@ -163,11 +169,9 @@ void MoodbarItemDelegate::StartLoadingData(const QUrl &url, Data *data) {
 
 bool MoodbarItemDelegate::RemoveFromCacheIfIndexesInvalid(const QUrl &url, Data *data) {
 
-  for (const QPersistentModelIndex &idx : data->indexes_) {
-    if (idx.isValid()) {
-      return false;
-    }
-  }
+  QSet<QPersistentModelIndex> indexes = data->indexes_;
+
+  if (std::any_of(indexes.begin(), indexes.end(), [](const QPersistentModelIndex &idx) { return idx.isValid(); })) { return false; }
 
   data_.remove(url);
   return true;
@@ -180,7 +184,7 @@ void MoodbarItemDelegate::ReloadAllColors() {
     Data *data = data_[url];
 
     if (data->state_ == Data::State_Loaded) {
-      StartLoadingData(url, data);
+      StartLoadingData(url, false, data);
     }
   }
 
@@ -188,10 +192,9 @@ void MoodbarItemDelegate::ReloadAllColors() {
 
 void MoodbarItemDelegate::DataLoaded(const QUrl &url, MoodbarPipeline *pipeline) {
 
+  if (!data_.contains(url)) return;
+
   Data *data = data_[url];
-  if (!data) {
-    return;
-  }
 
   if (RemoveFromCacheIfIndexesInvalid(url, data)) {
     return;
@@ -213,20 +216,19 @@ void MoodbarItemDelegate::StartLoadingColors(const QUrl &url, const QByteArray &
 
   QFuture<ColorVector> future = QtConcurrent::run(MoodbarRenderer::Colors, bytes, style_, qApp->palette());
   QFutureWatcher<ColorVector> *watcher = new QFutureWatcher<ColorVector>();
-  watcher->setFuture(future);
   QObject::connect(watcher, &QFutureWatcher<ColorVector>::finished, this, [this, watcher, url]() {
     ColorsLoaded(url, watcher->result());
     watcher->deleteLater();
   });
+  watcher->setFuture(future);
 
 }
 
 void MoodbarItemDelegate::ColorsLoaded(const QUrl &url, const ColorVector &colors) {
 
+  if (!data_.contains(url)) return;
+
   Data *data = data_[url];
-  if (!data) {
-    return;
-  }
 
   if (RemoveFromCacheIfIndexesInvalid(url, data)) {
     return;
@@ -245,20 +247,19 @@ void MoodbarItemDelegate::StartLoadingImage(const QUrl &url, Data *data) {
 
   QFuture<QImage> future = QtConcurrent::run(MoodbarRenderer::RenderToImage, data->colors_, data->desired_size_);
   QFutureWatcher<QImage> *watcher = new QFutureWatcher<QImage>();
-  watcher->setFuture(future);
   QObject::connect(watcher, &QFutureWatcher<QImage>::finished, this, [this, watcher, url]() {
     ImageLoaded(url, watcher->result());
     watcher->deleteLater();
   });
+  watcher->setFuture(future);
 
 }
 
 void MoodbarItemDelegate::ImageLoaded(const QUrl &url, const QImage &image) {
 
+  if (!data_.contains(url)) return;
+
   Data *data = data_[url];
-  if (!data) {
-    return;
-  }
 
   if (RemoveFromCacheIfIndexesInvalid(url, data)) {
     return;
@@ -278,7 +279,7 @@ void MoodbarItemDelegate::ImageLoaded(const QUrl &url, const QImage &image) {
   const QSortFilterProxyModel *filter = playlist->proxy();
 
   // Update all the indices with the new pixmap.
-  for (const QPersistentModelIndex &idx : data->indexes_) {
+  for (const QPersistentModelIndex &idx : std::as_const(data->indexes_)) {
     if (idx.isValid() && idx.sibling(idx.row(), Playlist::Column_Filename).data().toUrl() == url) {
       QModelIndex source_index = idx;
       if (idx.model() == filter) {
