@@ -19,15 +19,16 @@
 #define MOODBARLOADER_H
 
 #include <QObject>
-#include <QThread>
 #include <QList>
 #include <QMap>
 #include <QSet>
-#include <QByteArray>
+#include <QString>
 #include <QStringList>
 #include <QUrl>
-#include <QNetworkDiskCache>
 
+class QThread;
+class QByteArray;
+class QNetworkDiskCache;
 class Application;
 class MoodbarPipeline;
 
@@ -35,8 +36,8 @@ class MoodbarLoader : public QObject {
   Q_OBJECT
 
  public:
-  MoodbarLoader(Application* app, QObject* parent = nullptr);
-  ~MoodbarLoader();
+  explicit MoodbarLoader(Application *app, QObject *parent = nullptr);
+  ~MoodbarLoader() override;
 
   enum Result {
     // The URL isn't a local file or the moodbar plugin was not available -
@@ -51,20 +52,20 @@ class MoodbarLoader : public QObject {
     WillLoadAsync
   };
 
-  Result Load(const QUrl& url, QByteArray* data, MoodbarPipeline** async_pipeline);
+  Result Load(const QUrl &url, const bool has_cue, QByteArray *data, MoodbarPipeline **async_pipeline);
 
  private slots:
   void ReloadSettings();
 
-  void RequestFinished(MoodbarPipeline* request, const QUrl& filename);
+  void RequestFinished(MoodbarPipeline *request, const QUrl &url);
   void MaybeTakeNextRequest();
 
  private:
-  static QStringList MoodFilenames(const QString& song_filename);
+  static QStringList MoodFilenames(const QString &song_filename);
 
  private:
-  QNetworkDiskCache* cache_;
-  QThread* thread_;
+  QNetworkDiskCache *cache_;
+  QThread *thread_;
 
   const int kMaxActiveRequests;
 
@@ -72,7 +73,6 @@ class MoodbarLoader : public QObject {
   QList<QUrl> queued_requests_;
   QSet<QUrl> active_requests_;
 
-  bool enabled_;
   bool save_;
 };
 
