@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -68,8 +68,8 @@ QDBusArgument &operator<<(QDBusArgument &arg, const QImage &image) {
   // ABGR -> GBAR
   QImage i(scaled.size(), scaled.format());
   for (int y = 0; y < i.height(); ++y) {
-    QRgb *p = (QRgb*)scaled.scanLine(y);
-    QRgb *q = (QRgb*)i.scanLine(y);
+    QRgb *p = reinterpret_cast<QRgb*>(scaled.scanLine(y));
+    QRgb *q = reinterpret_cast<QRgb*>(i.scanLine(y));
     QRgb *end = p + scaled.width();
     while (p < end) {
       *q = qRgba(qGreen(*p), qBlue(*p), qAlpha(*p), qRed(*p));
@@ -109,15 +109,20 @@ const QDBusArgument &operator>>(const QDBusArgument &arg, QImage &image) {
 
 }
 
-OSDDBus::OSDDBus(SystemTrayIcon *tray_icon, Application *app, QObject *parent) : OSDBase(tray_icon, app, parent), version_(1, 1), notification_id_(0) {
+OSDDBus::OSDDBus(std::shared_ptr<SystemTrayIcon> tray_icon, Application *app, QObject *parent)
+    : OSDBase(tray_icon, app, parent),
+      version_(1, 1),
+      notification_id_(0) {
+
   Init();
+
 }
 
 OSDDBus::~OSDDBus() = default;
 
 void OSDDBus::Init() {
 
-  interface_.reset(new OrgFreedesktopNotificationsInterface(OrgFreedesktopNotificationsInterface::staticInterfaceName(), "/org/freedesktop/Notifications", QDBusConnection::sessionBus()));
+  interface_ = std::make_unique<OrgFreedesktopNotificationsInterface>(OrgFreedesktopNotificationsInterface::staticInterfaceName(), "/org/freedesktop/Notifications", QDBusConnection::sessionBus());
   if (!interface_->isValid()) {
     qLog(Warning) << "Error connecting to notifications service.";
   }
@@ -159,7 +164,7 @@ void OSDDBus::ShowMessageNative(const QString &summary, const QString &message,
 
   hints["transient"] = QVariant(true);
 
-  int id = 0;
+  quint64 id = 0;
   if (last_notification_time_.secsTo(QDateTime::currentDateTime()) * 1000 < timeout_msec()) {
     // Reuse the existing popup if it's still open.  The reason we don't always
     // reuse the popup is because the notification daemon on KDE4 won't re-show the bubble if it's already gone to the tray.  See issue #118
@@ -176,7 +181,7 @@ void OSDDBus::CallFinished(QDBusPendingCallWatcher *watcher) {
 
   std::unique_ptr<QDBusPendingCallWatcher> w(watcher);
 
-  QDBusPendingReply<uint> reply = *w.get();
+  QDBusPendingReply<uint> reply = *w;
   if (reply.isError()) {
     qLog(Warning) << "Error sending notification" << reply.error().name();
     return;
