@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -217,8 +217,9 @@ void PlaylistContainer::SetViewModel(Playlist *playlist, const int scroll_positi
   UpdateNoMatchesLabel();
 
   // Ensure that tab is current
-  if (ui_->tab_bar->current_id() != manager_->current_id())
+  if (ui_->tab_bar->current_id() != manager_->current_id()) {
     ui_->tab_bar->set_current_id(manager_->current_id());
+  }
 
   // Sort out the undo/redo actions
   delete undo_;
@@ -260,6 +261,19 @@ void PlaylistContainer::ReloadSettings() {
     ui_->clear->hide();
   }
 
+  bool show_toolbar = settings_.value("show_toolbar", true).toBool();
+  ui_->toolbar->setVisible(show_toolbar);
+
+  if (!show_toolbar) ui_->filter->clear();
+
+}
+
+bool PlaylistContainer::SearchFieldHasFocus() const {
+  return ui_->filter->hasFocus();
+}
+
+void PlaylistContainer::FocusSearchField() {
+  if (ui_->toolbar->isVisible()) ui_->filter->setFocus();
 }
 
 void PlaylistContainer::ActivePlaying() {
@@ -363,7 +377,7 @@ void PlaylistContainer::GoToNextPlaylistTab() {
 void PlaylistContainer::GoToPreviousPlaylistTab() {
 
   // Get the next tab' id
-  int id_previous = ui_->tab_bar->id_of((ui_->tab_bar->currentIndex()+ui_->tab_bar->count()-1) % ui_->tab_bar->count());
+  int id_previous = ui_->tab_bar->id_of((ui_->tab_bar->currentIndex() + ui_->tab_bar->count() - 1) % ui_->tab_bar->count());
   // Switch to next tab
   manager_->SetCurrentPlaylist(id_previous);
 
@@ -441,20 +455,20 @@ void PlaylistContainer::resizeEvent(QResizeEvent *e) {
 
 void PlaylistContainer::FocusOnFilter(QKeyEvent *event) {
 
-  ui_->filter->setFocus();
-
-  switch (event->key()) {
-    case Qt::Key_Backspace:
-      break;
-
-    case Qt::Key_Escape:
-      ui_->filter->clear();
-      break;
-
-    default:
-      ui_->filter->setText(ui_->filter->text() + event->text());
-      break;
+  if (ui_->toolbar->isVisible()) {
+    ui_->filter->setFocus();
+    switch (event->key()) {
+      case Qt::Key_Backspace:
+        break;
+      case Qt::Key_Escape:
+        ui_->filter->clear();
+        break;
+      default:
+        ui_->filter->setText(ui_->filter->text() + event->text());
+        break;
+    }
   }
+
 }
 
 void PlaylistContainer::RepositionNoMatchesLabel(const bool force) {
