@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 
 #include "config.h"
 
+#include <cmath>
+
 #include <QtGlobal>
 #include <QApplication>
 #include <QObject>
@@ -45,6 +47,7 @@
 #include <QUrl>
 #include <QIcon>
 #include <QPixmap>
+#include <QPixmapCache>
 #include <QPainter>
 #include <QColor>
 #include <QPen>
@@ -84,7 +87,8 @@ const float QueuedItemDelegate::kQueueOpacityLowerBound = 0.4;
 const int PlaylistDelegateBase::kMinHeight = 19;
 
 QueuedItemDelegate::QueuedItemDelegate(QObject *parent, int indicator_column)
-    : QStyledItemDelegate(parent), indicator_column_(indicator_column) {}
+    : QStyledItemDelegate(parent),
+      indicator_column_(indicator_column) {}
 
 void QueuedItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &idx) const {
 
@@ -94,21 +98,17 @@ void QueuedItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
     bool ok = false;
     const int queue_pos = idx.data(Playlist::Role_QueuePosition).toInt(&ok);
     if (ok && queue_pos != -1) {
-      float opacity = kQueueOpacitySteps - qMin(kQueueOpacitySteps, queue_pos);
+      float opacity = static_cast<float>(kQueueOpacitySteps - qMin(kQueueOpacitySteps, queue_pos));
       opacity /= kQueueOpacitySteps;
-      opacity *= 1.0 - kQueueOpacityLowerBound;
+      opacity *= static_cast<float>(1.0) - static_cast<float>(kQueueOpacityLowerBound);
       opacity += kQueueOpacityLowerBound;
-      painter->setOpacity(opacity);
-
-      DrawBox(painter, option.rect, option.font, QString::number(queue_pos + 1), kQueueBoxLength);
-
-      painter->setOpacity(1.0);
+      DrawBox(painter, option.rect, option.font, QString::number(queue_pos + 1), kQueueBoxLength, opacity);
     }
   }
 
 }
 
-void QueuedItemDelegate::DrawBox(QPainter *painter, const QRect &line_rect, const QFont &font, const QString &text, int width) const {
+void QueuedItemDelegate::DrawBox(QPainter *painter, const QRect line_rect, const QFont &font, const QString &text, int width, const float opacity) {
 
   QFont smaller = font;
   smaller.setPointSize(smaller.pointSize() - 1);
@@ -135,6 +135,10 @@ void QueuedItemDelegate::DrawBox(QPainter *painter, const QRect &line_rect, cons
   gradient.setColorAt(0.0, kQueueBoxGradientColor1);
   gradient.setColorAt(1.0, kQueueBoxGradientColor2);
 
+  painter->save();
+
+  painter->setOpacity(opacity);
+
   // Turn on antialiasing
   painter->setRenderHint(QPainter::Antialiasing);
 
@@ -149,6 +153,8 @@ void QueuedItemDelegate::DrawBox(QPainter *painter, const QRect &line_rect, cons
   painter->drawText(rect, Qt::AlignCenter, text);
   painter->translate(-0.5, -0.5);
 
+  painter->restore();
+
 }
 
 int QueuedItemDelegate::queue_indicator_size(const QModelIndex &idx) const {
@@ -174,7 +180,7 @@ QString PlaylistDelegateBase::displayText(const QVariant &value, const QLocale&)
   QString text;
 
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-  switch(value.metaType().id()) {
+  switch (value.metaType().id()) {
 #else
   switch (static_cast<QMetaType::Type>(value.type())) {
 #endif
@@ -183,14 +189,18 @@ QString PlaylistDelegateBase::displayText(const QVariant &value, const QLocale&)
       if (v > 0) text = QString::number(v);
       break;
     }
-
+    case QMetaType::Long:
+    case QMetaType::LongLong: {
+      qint64 v = value.toLongLong();
+      if (v > 0) text = QString::number(v);
+      break;
+    }
     case QMetaType::Float:
     case QMetaType::Double: {
       double v = value.toDouble();
       if (v > 0) text = QString::number(v);
       break;
     }
-
     default:
       text = value.toString();
       break;
@@ -231,8 +241,9 @@ QStyleOptionViewItem PlaylistDelegateBase::Adjusted(const QStyleOptionViewItem &
 
   QPoint top_left(-view_->horizontalScrollBar()->value(), -view_->verticalScrollBar()->value());
 
-  if (view_->header()->logicalIndexAt(top_left) != idx.column())
+  if (view_->header()->logicalIndexAt(top_left) != idx.column()) {
     return option;
+  }
 
   QStyleOptionViewItem ret(option);
 
@@ -253,7 +264,6 @@ bool PlaylistDelegateBase::helpEvent(QHelpEvent *event, QAbstractItemView *view,
 
   if (!event || !view) return false;
 
-  QHelpEvent *he = static_cast<QHelpEvent*>(event);
   QString text = displayText(idx.data(), QLocale::system());
 
   // Special case: we want newlines in the comment tooltip
@@ -265,7 +275,7 @@ bool PlaylistDelegateBase::helpEvent(QHelpEvent *event, QAbstractItemView *view,
     text.replace("\n", "<br />");
   }
 
-  if (text.isEmpty() || !he) return false;
+  if (text.isEmpty() || !event) return false;
 
   switch (event->type()) {
     case QEvent::ToolTip: {
@@ -273,7 +283,7 @@ bool PlaylistDelegateBase::helpEvent(QHelpEvent *event, QAbstractItemView *view,
       QRect displayed_text = view->visualRect(idx);
       bool is_elided = displayed_text.width() < real_text.width();
       if (is_elided) {
-        QToolTip::showText(he->globalPos(), text, view);
+        QToolTip::showText(event->globalPos(), text, view);
       }
       else {  // in case that another text was previously displayed
         QToolTip::hideText();
@@ -285,7 +295,7 @@ bool PlaylistDelegateBase::helpEvent(QHelpEvent *event, QAbstractItemView *view,
       return true;
 
     case QEvent::WhatsThis:
-      QWhatsThis::showText(he->globalPos(), text, view);
+      QWhatsThis::showText(event->globalPos(), text, view);
       return true;
 
     default:
@@ -310,9 +320,9 @@ QString LengthItemDelegate::displayText(const QVariant &value, const QLocale&) c
 QString SizeItemDelegate::displayText(const QVariant &value, const QLocale&) const {
 
   bool ok = false;
-  int bytes = value.toInt(&ok);
+  qint64 bytes = value.toLongLong(&ok);
 
-  if (ok) return Utilities::PrettySize(bytes);
+  if (ok && bytes > 0) return Utilities::PrettySize(static_cast<quint64>(bytes));
   return QString();
 
 }
@@ -324,8 +334,9 @@ QString DateItemDelegate::displayText(const QVariant &value, const QLocale &loca
   bool ok = false;
   qint64 time = value.toLongLong(&ok);
 
-  if (!ok || time == -1)
+  if (!ok || time == -1) {
     return QString();
+  }
 
   return QDateTime::fromSecsSinceEpoch(time).toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat));
 
@@ -336,8 +347,9 @@ QString LastPlayedItemDelegate::displayText(const QVariant &value, const QLocale
   bool ok = false;
   const qint64 time = value.toLongLong(&ok);
 
-  if (!ok || time == -1)
+  if (!ok || time == -1) {
     return tr("Never");
+  }
 
   return Utilities::Ago(time, locale);
 
@@ -362,7 +374,7 @@ QWidget *TextItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
   return new QLineEdit(parent);
 }
 
-TagCompletionModel::TagCompletionModel(CollectionBackend *backend, Playlist::Column column) {
+TagCompletionModel::TagCompletionModel(CollectionBackend *backend, const Playlist::Column column, QObject *parent) : QStringListModel(parent) {
 
   QString col = database_column(column);
   if (!col.isEmpty()) {
@@ -402,8 +414,8 @@ TagCompleter::TagCompleter(CollectionBackend *backend, Playlist::Column column,
 
   QFuture<TagCompletionModel*> future = QtConcurrent::run(&InitCompletionModel, backend, column);
   QFutureWatcher<TagCompletionModel*> *watcher = new QFutureWatcher<TagCompletionModel*>();
-  watcher->setFuture(future);
   QObject::connect(watcher, &QFutureWatcher<TagCompletionModel*>::finished, this, &TagCompleter::ModelReady);
+  watcher->setFuture(future);
 
 }
 
@@ -464,17 +476,17 @@ QString SongSourceDelegate::displayText(const QVariant &value, const QLocale&) c
   return QString();
 }
 
-QPixmap SongSourceDelegate::LookupPixmap(const Song::Source &source, const QSize &size) const {
+QPixmap SongSourceDelegate::LookupPixmap(const Song::Source source, const QSize size) const {
 
   QPixmap pixmap;
   QString cache_key = QString("%1-%2x%3").arg(Song::TextForSource(source)).arg(size.width()).arg(size.height());
-  if (pixmap_cache_.find(cache_key, &pixmap)) {
+  if (QPixmapCache::find(cache_key, &pixmap)) {
     return pixmap;
   }
 
   QIcon icon(Song::IconForSource(source));
   pixmap = icon.pixmap(size.height());
-  pixmap_cache_.insert(cache_key, pixmap);
+  QPixmapCache::insert(cache_key, pixmap);
 
   return pixmap;
 
@@ -492,7 +504,7 @@ void SongSourceDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
   QPixmap pixmap = LookupPixmap(source, option_copy.decorationSize);
 
   QWidget *parent_widget = qobject_cast<QWidget*>(parent());
-  int device_pixel_ratio = parent_widget->devicePixelRatio();
+  qreal device_pixel_ratio = parent_widget->devicePixelRatio();
 
   // Draw the pixmap in the middle of the rectangle
   QRect draw_rect(QPoint(0, 0), option_copy.decorationSize / device_pixel_ratio);
@@ -514,7 +526,7 @@ void RatingItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
 
   const bool hover = mouse_over_index_.isValid() && (mouse_over_index_ == idx || (selected_indexes_.contains(mouse_over_index_) && selected_indexes_.contains(idx)));
 
-  const double rating = (hover ? RatingPainter::RatingForPos(mouse_over_pos_, option.rect) : idx.data().toDouble());
+  const float rating = (hover ? RatingPainter::RatingForPos(mouse_over_pos_, option.rect) : idx.data().toFloat());
 
   painter_.Paint(painter, option.rect, rating);
 
@@ -530,10 +542,10 @@ QSize RatingItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QMo
 
 QString RatingItemDelegate::displayText(const QVariant &value, const QLocale&) const {
 
-  if (value.isNull() || value.toDouble() <= 0) return QString();
+  if (value.isNull() || value.toFloat() <= 0) return QString();
 
   // Round to the nearest 0.5
-  const double rating = float(int(value.toDouble()  * RatingPainter::kStarCount * 2 + 0.5)) / 2;
+  const float rating = static_cast<float>(lround(value.toFloat() * RatingPainter::kStarCount * 2)) / 2;
 
   return QString::number(rating, 'f', 1);
 
