@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 PlaylistHeader::PlaylistHeader(Qt::Orientation orientation, PlaylistView *view, QWidget *parent)
     : StretchHeaderView(orientation, parent),
       view_(view),
+      menu_section_(0),
       menu_(new QMenu(this)),
       action_hide_(nullptr),
       action_reset_(nullptr),
@@ -50,8 +51,7 @@ PlaylistHeader::PlaylistHeader(Qt::Orientation orientation, PlaylistView *view,
       action_rating_lock_(nullptr),
       action_align_left_(nullptr),
       action_align_center_(nullptr),
-      action_align_right_(nullptr)
-      {
+      action_align_right_(nullptr) {
 
   action_hide_ = menu_->addAction(tr("&Hide..."), this, &PlaylistHeader::HideCurrent);
   action_stretch_ = menu_->addAction(tr("&Stretch columns to fit window"), this, &PlaylistHeader::ToggleStretchEnabled);
@@ -92,8 +92,9 @@ void PlaylistHeader::contextMenuEvent(QContextMenuEvent *e) {
 
   menu_section_ = logicalIndexAt(e->pos());
 
-  if (menu_section_ == -1 || (menu_section_ == logicalIndex(0) && logicalIndex(1) == -1))
+  if (menu_section_ == -1 || (menu_section_ == logicalIndex(0) && logicalIndex(1) == -1)) {
     action_hide_->setVisible(false);
+  }
   else {
     action_hide_->setVisible(true);
 
@@ -112,7 +113,7 @@ void PlaylistHeader::contextMenuEvent(QContextMenuEvent *e) {
 
   qDeleteAll(show_actions_);
   show_actions_.clear();
-  for (int i = 0 ; i < count() ; ++i) {
+  for (int i = 0; i < count(); ++i) {
     AddColumnAction(i);
   }
 
@@ -135,7 +136,7 @@ void PlaylistHeader::AddColumnAction(int index) {
   action->setChecked(!isSectionHidden(index));
   show_actions_ << action;
 
-  QObject::connect(action, &QAction::triggered, [this, index]() { ToggleVisible(index); } );
+  QObject::connect(action, &QAction::triggered, this, [this, index]() { ToggleVisible(index); });
 
 }
 
