@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,14 +81,14 @@ class PlaylistManagerInterface : public QObject {
   virtual void PlaySmartPlaylist(PlaylistGeneratorPtr generator, const bool as_new, const bool clear) = 0;
 
  public slots:
-  virtual void New(const QString &name, const SongList& songs = SongList(), const QString &special_type = QString()) = 0;
+  virtual void New(const QString &name, const SongList &songs = SongList(), const QString &special_type = QString()) = 0;
   virtual void Load(const QString &filename) = 0;
   virtual void Save(const int id, const QString &filename, const Playlist::Path path_type) = 0;
   virtual void Rename(const int id, const QString &new_name) = 0;
   virtual void Delete(const int id) = 0;
   virtual bool Close(const int id) = 0;
   virtual void Open(const int id) = 0;
-  virtual void ChangePlaylistOrder(const QList<int>& ids) = 0;
+  virtual void ChangePlaylistOrder(const QList<int> &ids) = 0;
 
   virtual void SongChangeRequestProcessed(const QUrl &url, const bool valid) = 0;
 
@@ -108,9 +108,9 @@ class PlaylistManagerInterface : public QObject {
   virtual void SetActiveStopped() = 0;
 
   // Rate current song using 0.0 - 1.0 scale.
-  virtual void RateCurrentSong(const double rating) = 0;
+  virtual void RateCurrentSong(const float rating) = 0;
   // Rate current song using 0 - 5 scale.
-  virtual void RateCurrentSong(const int rating) = 0;
+  virtual void RateCurrentSong2(const int rating) = 0;
 
  signals:
   void PlaylistManagerInitialized();
@@ -133,7 +133,7 @@ class PlaylistManagerInterface : public QObject {
 
   // Signals that one of manager's playlists has changed (new items, new ordering etc.) - the argument shows which.
   void PlaylistChanged(Playlist *playlist);
-  void EditingFinished(QModelIndex idx);
+  void EditingFinished(int playlist_id, QModelIndex idx);
   void PlayRequested(QModelIndex idx, Playlist::AutoScroll autoscroll);
 };
 
@@ -206,21 +206,21 @@ class PlaylistManager : public PlaylistManagerInterface {
   void RemoveDuplicatesCurrent() override;
   void RemoveUnavailableCurrent() override;
 
-  void SongChangeRequestProcessed(const QUrl& url, const bool valid) override;
+  void SongChangeRequestProcessed(const QUrl &url, const bool valid) override;
 
   void InsertUrls(const int id, const QList<QUrl> &urls, const int pos = -1, const bool play_now = false, const bool enqueue = false);
   void InsertSongs(const int id, const SongList &songs, const int pos = -1, const bool play_now = false, const bool enqueue = false);
   // Removes items with given indices from the playlist. This operation is not undoable.
-  void RemoveItemsWithoutUndo(const int id, const QList<int>& indices);
+  void RemoveItemsWithoutUndo(const int id, const QList<int> &indices);
   // Remove the current playing song
-  void RemoveCurrentSong();
+  void RemoveCurrentSong() const;
 
   void PlaySmartPlaylist(PlaylistGeneratorPtr generator, const bool as_new, const bool clear) override;
 
   // Rate current song using 0.0 - 1.0 scale.
-  void RateCurrentSong(const double rating) override;
+  void RateCurrentSong(const float rating) override;
   // Rate current song using 0 - 5 scale.
-  void RateCurrentSong(const int rating) override;
+  void RateCurrentSong2(const int rating) override;
 
  private slots:
   void SetActivePlaying() override;
@@ -229,8 +229,8 @@ class PlaylistManager : public PlaylistManagerInterface {
 
   void OneOfPlaylistsChanged();
   void UpdateSummaryText();
-  void SongsDiscovered(const SongList& songs);
-  void ItemsLoadedForSavePlaylist(const SongList &songs, const QString& filename, const Playlist::Path path_type);
+  void SongsDiscovered(const SongList &songs);
+  void ItemsLoadedForSavePlaylist(const SongList &songs, const QString &filename, const Playlist::Path path_type);
   void PlaylistLoaded();
 
  private:
