@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018-2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,15 +33,18 @@ SongPlaylistItem::SongPlaylistItem(const Song::Source source) : PlaylistItem(sou
 SongPlaylistItem::SongPlaylistItem(const Song &song) : PlaylistItem(song.source()), song_(song) {}
 
 bool SongPlaylistItem::InitFromQuery(const SqlRow &query) {
-  song_.InitFromQuery(query, false, (Song::kColumns.count()+1));
+  song_.InitFromQuery(query, false);
   return true;
 }
 
 QUrl SongPlaylistItem::Url() const { return song_.url(); }
 
 void SongPlaylistItem::Reload() {
+
   if (!song_.url().isLocalFile()) return;
   TagReaderClient::Instance()->ReadFileBlocking(song_.url().toLocalFile(), &song_);
+  UpdateTemporaryMetadata(song_);
+
 }
 
 Song SongPlaylistItem::Metadata() const {
