@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +41,7 @@
 class CollectionBackendInterface;
 
 const char *CueParser::kFileLineRegExp = "(\\S+)\\s+(?:\"([^\"]+)\"|(\\S+))\\s*(?:\"([^\"]+)\"|(\\S+))?";
-const char *CueParser::kIndexRegExp = "(\\d{2,3}):(\\d{2}):(\\d{2})";
+const char *CueParser::kIndexRegExp = "(\\d{1,3}):(\\d{2}):(\\d{2})";
 
 const char *CueParser::kPerformer = "performer";
 const char *CueParser::kTitle = "title";
@@ -57,7 +58,7 @@ const char *CueParser::kDisc = "discnumber";
 CueParser::CueParser(CollectionBackendInterface *collection, QObject *parent)
     : ParserBase(collection, parent) {}
 
-SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const QDir &dir) const {
+SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const QDir &dir, const bool collection_search) const {
 
   SongList ret;
 
@@ -90,54 +91,53 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
       if (splitted.size() < 2) {
         continue;
       }
-      QString line_name = splitted[0].toLower();
-      QString line_value = splitted[1];
+      const QString &line_name = splitted[0];
+      const QString &line_value = splitted[1];
 
-      if (line_name == kPerformer) {
+      if (line_name.compare(kPerformer, Qt::CaseInsensitive) == 0) {
         album_artist = line_value;
       }
-      else if (line_name == kTitle) {
+      else if (line_name.compare(kTitle, Qt::CaseInsensitive) == 0) {
         album = line_value;
       }
-      else if (line_name == kSongWriter) {
+      else if (line_name.compare(kSongWriter, Qt::CaseInsensitive) == 0) {
         album_composer = line_value;
       }
-      else if (line_name == kFile) {
+      else if (line_name.compare(kFile, Qt::CaseInsensitive) == 0) {
         file = QDir::isAbsolutePath(line_value) ? line_value : dir.absoluteFilePath(line_value);
         if (splitted.size() > 2) {
           file_type = splitted[2];
         }
       }
-      else if (line_name == kRem) {
+      else if (line_name.compare(kRem, Qt::CaseInsensitive) == 0) {
         if (splitted.size() < 3) {
           break;
         }
-        if (line_value.toLower() == kGenre) {
+        if (line_value.compare(kGenre, Qt::CaseInsensitive) == 0) {
           album_genre = splitted[2];
         }
-        else if (line_value.toLower() == kDate) {
+        else if (line_value.compare(kDate, Qt::CaseInsensitive) == 0) {
           album_date = splitted[2];
         }
-        else if (line_value.toLower() == kDisc) {
+        else if (line_value.compare(kDisc, Qt::CaseInsensitive) == 0) {
           disc = splitted[2];
         }
       }
       // end of the header -> go into the track mode
-      else if (line_name == kTrack) {
+      else if (line_name.compare(kTrack, Qt::CaseInsensitive) == 0) {
         files++;
         break;
       }
       // just ignore the rest of possible field types for now...
-    }
-    while (!(line = text_stream.readLine()).isNull());
+    } while (!(line = text_stream.readLine()).isNull());
 
-    if(line.isNull()) {
+    if (line.isNull()) {
       qLog(Warning) << "the .cue file from " << dir_path << " defines no tracks!";
       return ret;
     }
 
     // if this is a data file, all of it's tracks will be ignored
-    bool valid_file = file_type.compare("BINARY", Qt::CaseInsensitive) && file_type.compare("MOTOROLA", Qt::CaseInsensitive);
+    bool valid_file = file_type.compare("BINARY", Qt::CaseInsensitive) != 0 && file_type.compare("MOTOROLA", Qt::CaseInsensitive) != 0;
 
     QString track_type;
     QString index;
@@ -156,15 +156,15 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
         continue;
       }
 
-      QString line_name = splitted[0].toLower();
-      QString line_value = splitted[1];
+      const QString &line_name = splitted[0];
+      const QString &line_value = splitted[1];
       QString line_additional = splitted.size() > 2 ? splitted[2].toLower() : "";
 
-      if (line_name == kTrack) {
+      if (line_name.compare(kTrack, Qt::CaseInsensitive) == 0) {
 
         // the beginning of another track's definition - we're saving the current one for later (if it's valid of course)
         // please note that the same code is repeated just after this 'do-while' loop
-        if (valid_file && !index.isEmpty() && (track_type.isEmpty() || track_type == kAudioTrackType)) {
+        if (valid_file && !index.isEmpty() && (track_type.isEmpty() || track_type.compare(kAudioTrackType, Qt::CaseInsensitive) == 0)) {
           entries.append(CueEntry(file, index, title, artist, album_artist, album, composer, album_composer, (genre.isEmpty() ? album_genre : genre), (date.isEmpty() ? album_date : date), disc));
         }
 
@@ -176,7 +176,7 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
         }
 
       }
-      else if (line_name == kIndex) {
+      else if (line_name.compare(kIndex, Qt::CaseInsensitive) == 0) {
 
         // We need the index's position field
         if (!line_additional.isEmpty()) {
@@ -188,37 +188,36 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
           }
         }
       }
-      else if (line_name == kTitle) {
+      else if (line_name.compare(kTitle, Qt::CaseInsensitive) == 0) {
         title = line_value;
       }
-      else if (line_name == kDate) {
+      else if (line_name.compare(kDate, Qt::CaseInsensitive) == 0) {
         date = line_value;
       }
-      else if (line_name == kPerformer) {
+      else if (line_name.compare(kPerformer, Qt::CaseInsensitive) == 0) {
         artist = line_value;
       }
-      else if (line_name == kSongWriter) {
+      else if (line_name.compare(kSongWriter, Qt::CaseInsensitive) == 0) {
         composer = line_value;
         // End of track's for the current file -> parse next one
       }
-      else if (line_name == kRem && splitted.size() >= 3) {
-        if (line_value.toLower() == kGenre) {
+      else if (line_name.compare(kRem, Qt::CaseInsensitive) == 0 && splitted.size() >= 3) {
+        if (line_value.compare(kGenre, Qt::CaseInsensitive) == 0) {
           genre = splitted[2];
         }
-        else if (line_value.toLower() == kDate) {
+        else if (line_value.compare(kDate, Qt::CaseInsensitive) == 0) {
           date = splitted[2];
         }
       }
-      else if (line_name == kFile) {
+      else if (line_name.compare(kFile, Qt::CaseInsensitive) == 0) {
         break;
       }
 
       // Just ignore the rest of possible field types for now...
-    }
-    while (!(line = text_stream.readLine()).isNull());
+    } while (!(line = text_stream.readLine()).isNull());
 
     // We didn't add the last song yet...
-    if (valid_file && !index.isEmpty() && (track_type.isEmpty() || track_type == kAudioTrackType)) {
+    if (valid_file && !index.isEmpty() && (track_type.isEmpty() || track_type.compare(kAudioTrackType, Qt::CaseInsensitive) == 0)) {
       entries.append(CueEntry(file, index, title, artist, album_artist, album, composer, album_composer, (genre.isEmpty() ? album_genre : genre), (date.isEmpty() ? album_date : date), disc));
     }
   }
@@ -229,7 +228,7 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
   for (int i = 0; i < entries.length(); i++) {
     CueEntry entry = entries.at(i);
 
-    Song song = LoadSong(entry.file, IndexToMarker(entry.index), dir);
+    Song song = LoadSong(entry.file, IndexToMarker(entry.index), dir, collection_search);
 
     // Cue song has mtime equal to qMax(media_file_mtime, cue_sheet_mtime)
     if (cue_mtime.isValid()) {
@@ -245,7 +244,7 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
     }
 
     // The last TRACK for every FILE gets it's 'end' marker from the media file's length
-    if(i + 1 < entries.size() && entries.at(i).file == entries.at(i + 1).file) {
+    if (i + 1 < entries.size() && entries.at(i).file == entries.at(i + 1).file) {
       // incorrect indices?
       if (!UpdateSong(entry, entries.at(i + 1).index, &song)) {
         continue;
@@ -266,7 +265,7 @@ SongList CueParser::Load(QIODevice *device, const QString &playlist_path, const
 
 // This and the kFileLineRegExp do most of the "dirty" work, namely: splitting the raw .cue
 // line into logical parts and getting rid of all the unnecessary whitespaces and quoting.
-QStringList CueParser::SplitCueLine(const QString &line) const {
+QStringList CueParser::SplitCueLine(const QString &line) {
 
   QRegularExpression line_regexp(kFileLineRegExp);
   QRegularExpressionMatch re_match = line_regexp.match(line.trimmed());
@@ -280,7 +279,7 @@ QStringList CueParser::SplitCueLine(const QString &line) const {
 }
 
 // Updates the song with data from the .cue entry. This one mustn't be used for the last song in the .cue file.
-bool CueParser::UpdateSong(const CueEntry &entry, const QString &next_index, Song *song) const {
+bool CueParser::UpdateSong(const CueEntry &entry, const QString &next_index, Song *song) {
 
   qint64 beginning = IndexToMarker(entry.index);
   qint64 end = IndexToMarker(next_index);
@@ -306,7 +305,7 @@ bool CueParser::UpdateSong(const CueEntry &entry, const QString &next_index, Son
 }
 
 // Updates the song with data from the .cue entry. This one must be used only for the last song in the .cue file.
-bool CueParser::UpdateLastSong(const CueEntry &entry, Song *song) const {
+bool CueParser::UpdateLastSong(const CueEntry &entry, Song *song) {
 
   qint64 beginning = IndexToMarker(entry.index);
 
@@ -337,7 +336,7 @@ bool CueParser::UpdateLastSong(const CueEntry &entry, Song *song) const {
 
 }
 
-qint64 CueParser::IndexToMarker(const QString &index) const {
+qint64 CueParser::IndexToMarker(const QString &index) {
 
   QRegularExpression index_regexp(kIndexRegExp);
   QRegularExpressionMatch re_match = index_regexp.match(index);
@@ -346,7 +345,7 @@ qint64 CueParser::IndexToMarker(const QString &index) const {
   }
 
   QStringList splitted = re_match.capturedTexts().mid(1, -1);
-  qlonglong frames = splitted.at(0).toLongLong() * 60 * 75 + splitted.at(1).toLongLong() * 75 + splitted.at(2).toLongLong();
+  qint64 frames = splitted.at(0).toLongLong() * 60 * 75 + splitted.at(1).toLongLong() * 75 + splitted.at(2).toLongLong();
   return (frames * kNsecPerSec) / 75;
 
 }
@@ -380,3 +379,16 @@ bool CueParser::TryMagic(const QByteArray &data) const {
   return false;
 
 }
+
+QString CueParser::FindCueFilename(const QString &filename) {
+
+  QStringList cue_files = QStringList() << filename + ".cue"
+                                        << filename.section('.', 0, -2) + ".cue";
+
+  for (const QString &cuefile : cue_files) {
+    if (QFileInfo::exists(cuefile)) return cuefile;
+  }
+
+  return QString();
+
+}
