@@ -49,7 +49,7 @@ class M3UParser : public ParserBase {
 
   bool TryMagic(const QByteArray &data) const override;
 
-  SongList Load(QIODevice *device, const QString &playlist_path = "", const QDir &dir = QDir()) const override;
+  SongList Load(QIODevice *device, const QString &playlist_path = "", const QDir &dir = QDir(), const bool collection_search = true) const override;
   void Save(const SongList &songs, QIODevice *device, const QDir &dir = QDir(), Playlist::Path path_type = Playlist::Path_Automatic) const override;
 
  private:
@@ -66,7 +66,7 @@ class M3UParser : public ParserBase {
     qint64 length;
   };
 
-  bool ParseMetadata(const QString &line, Metadata *metadata) const;
+  static bool ParseMetadata(const QString &line, Metadata *metadata);
 
 };
 
