@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@
 #include <QFile>
 #include <QFileInfo>
 #include <QString>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QUrl>
 
 #include "core/logging.h"
@@ -35,7 +36,7 @@
 ParserBase::ParserBase(CollectionBackendInterface *collection, QObject *parent)
     : QObject(parent), collection_(collection) {}
 
-void ParserBase::LoadSong(const QString &filename_or_url, qint64 beginning, const QDir &dir, Song *song) const {
+void ParserBase::LoadSong(const QString &filename_or_url, const qint64 beginning, const QDir &dir, Song *song, const bool collection_search) const {
 
   if (filename_or_url.isEmpty()) {
     return;
@@ -43,13 +44,13 @@ void ParserBase::LoadSong(const QString &filename_or_url, qint64 beginning, cons
 
   QString filename = filename_or_url;
 
-  if (filename_or_url.contains(QRegExp("^[a-z]{2,}:"))) {
+  if (filename_or_url.contains(QRegularExpression("^[a-z]{2,}:", QRegularExpression::CaseInsensitiveOption))) {
     QUrl url(filename_or_url);
     song->set_source(Song::SourceFromURL(url));
     if (song->source() == Song::Source_LocalFile) {
       filename = url.toLocalFile();
     }
-    else if (song->source() == Song::Source_Stream || song->source() == Song::Source_Tidal) {
+    else if (song->is_stream()) {
       song->set_url(QUrl::fromUserInput(filename_or_url));
       song->set_filetype(Song::FileType_Stream);
       song->set_valid(true);
@@ -57,6 +58,7 @@ void ParserBase::LoadSong(const QString &filename_or_url, qint64 beginning, cons
     }
     else {
       qLog(Error) << "Don't know how to handle" << url;
+      return;
     }
   }
 
@@ -77,30 +79,29 @@ void ParserBase::LoadSong(const QString &filename_or_url, qint64 beginning, cons
   const QUrl url = QUrl::fromLocalFile(filename);
 
   // Search in the collection
-  Song collection_song(Song::Source_Collection);
-  if (collection_) {
-    collection_song = collection_->GetSongByUrl(url, beginning);
+  if (collection_ && collection_search) {
+    Song collection_song = collection_->GetSongByUrl(url, beginning);
+    // If it was found in the collection then use it, otherwise load metadata from disk.
+    if (collection_song.is_valid()) {
+      *song = collection_song;
+      return;
+    }
   }
 
-  // If it was found in the collection then use it, otherwise load metadata from disk.
-  if (collection_song.is_valid()) {
-    *song = collection_song;
-  }
-  else {
-    TagReaderClient::Instance()->ReadFileBlocking(filename, song);
-  }
+  TagReaderClient::Instance()->ReadFileBlocking(filename, song);
 
 }
 
-Song ParserBase::LoadSong(const QString &filename_or_url, qint64 beginning, const QDir &dir) const {
+Song ParserBase::LoadSong(const QString &filename_or_url, const qint64 beginning, const QDir &dir, const bool collection_search) const {
 
   Song song(Song::Source_LocalFile);
-  LoadSong(filename_or_url, beginning, dir, &song);
+  LoadSong(filename_or_url, beginning, dir, &song, collection_search);
+
   return song;
 
 }
 
-QString ParserBase::URLOrFilename(const QUrl &url, const QDir &dir, Playlist::Path path_type) const {
+QString ParserBase::URLOrFilename(const QUrl &url, const QDir &dir, Playlist::Path path_type) {
 
   if (!url.isLocalFile()) return url.toString();
 
@@ -109,8 +110,9 @@ QString ParserBase::URLOrFilename(const QUrl &url, const QDir &dir, Playlist::Pa
   if (path_type != Playlist::Path_Absolute && QDir::isAbsolutePath(filename)) {
     const QString relative = dir.relativeFilePath(filename);
 
-    if (!relative.startsWith("../") || path_type == Playlist::Path_Relative)
+    if (!relative.startsWith("../") || path_type == Playlist::Path_Relative) {
       return relative;
+    }
   }
   return filename;
 
