@@ -36,15 +36,15 @@
 class CollectionBackendInterface;
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
-#  define qt_endl Qt::endl
+constexpr auto qt_endl = Qt::endl;
 #else
-#  define qt_endl endl
+constexpr auto qt_endl = endl;
 #endif
 
 PLSParser::PLSParser(CollectionBackendInterface *collection, QObject *parent)
     : ParserBase(collection, parent) {}
 
-SongList PLSParser::Load(QIODevice *device, const QString &playlist_path, const QDir &dir) const {
+SongList PLSParser::Load(QIODevice *device, const QString &playlist_path, const QDir &dir, const bool collection_search) const {
 
   Q_UNUSED(playlist_path);
 
@@ -53,7 +53,7 @@ SongList PLSParser::Load(QIODevice *device, const QString &playlist_path, const
 
   while (!device->atEnd()) {
     QString line = QString::fromUtf8(device->readLine()).trimmed();
-    int equals = line.indexOf('=');
+    qint64 equals = line.indexOf('=');
     QString key = line.left(equals).toLower();
     QString value = line.mid(equals + 1);
 
@@ -61,12 +61,13 @@ SongList PLSParser::Load(QIODevice *device, const QString &playlist_path, const
     int n = re_match.captured(0).toInt();
 
     if (key.startsWith("file")) {
-      Song song = LoadSong(value, 0, dir);
+      Song song = LoadSong(value, 0, dir, collection_search);
 
       // Use the title and length we've already loaded if any
       if (!songs[n].title().isEmpty()) song.set_title(songs[n].title());
-      if (songs[n].length_nanosec() != -1)
+      if (songs[n].length_nanosec() != -1) {
         song.set_length_nanosec(songs[n].length_nanosec());
+      }
 
       songs[n] = song;
     }
