@@ -18,6 +18,7 @@
  *
  */
 
+#include <QtGlobal>
 #include <QObject>
 #include <QIODevice>
 #include <QDir>
@@ -40,7 +41,7 @@ bool WplParser::TryMagic(const QByteArray &data) const {
   return data.contains("<?wpl") || data.contains("<smil>");
 }
 
-SongList WplParser::Load(QIODevice *device, const QString &playlist_path, const QDir &dir) const {
+SongList WplParser::Load(QIODevice *device, const QString &playlist_path, const QDir &dir, const bool collection_search) const {
 
   Q_UNUSED(playlist_path);
 
@@ -52,34 +53,35 @@ SongList WplParser::Load(QIODevice *device, const QString &playlist_path, const
   }
 
   while (!reader.atEnd() && Utilities::ParseUntilElement(&reader, "seq")) {
-    ParseSeq(dir, &reader, &ret);
+    ParseSeq(dir, &reader, &ret, collection_search);
   }
   return ret;
 
 }
 
-void WplParser::ParseSeq(const QDir &dir, QXmlStreamReader *reader, SongList *songs) const {
+void WplParser::ParseSeq(const QDir &dir, QXmlStreamReader *reader, SongList *songs, const bool collection_search) const {
 
   while (!reader->atEnd()) {
     QXmlStreamReader::TokenType type = reader->readNext();
+    QString name = reader->name().toString();
     switch (type) {
       case QXmlStreamReader::StartElement: {
-        QStringRef name = reader->name();
         if (name == "media") {
-          QStringRef src = reader->attributes().value("src");
+          QString src = reader->attributes().value("src").toString();
           if (!src.isEmpty()) {
-            Song song = LoadSong(src.toString(), 0, dir);
+            Song song = LoadSong(src, 0, dir, collection_search);
             if (song.is_valid()) {
               songs->append(song);
             }
           }
-        } else {
+        }
+        else {
           Utilities::ConsumeCurrentElement(reader);
         }
         break;
       }
       case QXmlStreamReader::EndElement: {
-        if (reader->name() == "seq") {
+        if (name == "seq") {
           return;
         }
         break;
@@ -119,7 +121,7 @@ void WplParser::Save(const SongList &songs, QIODevice *device, const QDir &dir,
   }
 }
 
-void WplParser::WriteMeta(const QString &name, const QString &content, QXmlStreamWriter *writer) const {
+void WplParser::WriteMeta(const QString &name, const QString &content, QXmlStreamWriter *writer) {
 
   writer->writeStartElement("meta");
   writer->writeAttribute("name", name);
