@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2019, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <QPair>
 #include <QSet>
 #include <QList>
+#include <QMap>
 #include <QVariant>
 #include <QByteArray>
 #include <QString>
@@ -52,8 +53,6 @@ class QobuzStreamURLRequest;
 class CollectionBackend;
 class CollectionModel;
 
-using std::shared_ptr;
-
 class QobuzService : public InternetService {
   Q_OBJECT
 
@@ -62,6 +61,7 @@ class QobuzService : public InternetService {
   ~QobuzService();
 
   static const Song::Source kSource;
+  static const char *kApiUrl;
 
   void Exit() override;
   void ReloadSettings() override;
@@ -93,7 +93,7 @@ class QobuzService : public InternetService {
   bool login_sent() { return login_sent_; }
   bool login_attempts() { return login_attempts_; }
 
-  void GetStreamURL(const QUrl &url);
+  uint GetStreamURL(const QUrl &url, QString &error);
 
   CollectionBackend *artists_collection_backend() override { return artists_collection_backend_; }
   CollectionBackend *albums_collection_backend() override { return albums_collection_backend_; }
@@ -107,15 +107,6 @@ class QobuzService : public InternetService {
   QSortFilterProxyModel *albums_collection_sort_model() override { return albums_collection_sort_model_; }
   QSortFilterProxyModel *songs_collection_sort_model() override { return songs_collection_sort_model_; }
 
-  enum QueryType {
-    QueryType_Artists,
-    QueryType_Albums,
-    QueryType_Songs,
-    QueryType_SearchArtists,
-    QueryType_SearchAlbums,
-    QueryType_SearchSongs,
-  };
-
  public slots:
   void ShowConfig() override;
   void TryLogin();
@@ -130,14 +121,14 @@ class QobuzService : public InternetService {
 
  private slots:
   void ExitReceived();
-  void HandleLoginSSLErrors(QList<QSslError> ssl_errors);
+  void HandleLoginSSLErrors(const QList<QSslError> &ssl_errors);
   void HandleAuthReply(QNetworkReply *reply);
   void ResetLoginAttempts();
   void StartSearch();
-  void ArtistsResultsReceived(const int id, const SongList &songs, const QString &error);
-  void AlbumsResultsReceived(const int id, const SongList &songs, const QString &error);
-  void SongsResultsReceived(const int id, const SongList &songs, const QString &error);
-  void SearchResultsReceived(const int id, const SongList &songs, const QString &error);
+  void ArtistsResultsReceived(const int id, const SongMap &songs, const QString &error);
+  void AlbumsResultsReceived(const int id, const SongMap &songs, const QString &error);
+  void SongsResultsReceived(const int id, const SongMap &songs, const QString &error);
+  void SearchResultsReceived(const int id, const SongMap &songs, const QString &error);
   void ArtistsUpdateStatusReceived(const int id, const QString &text);
   void AlbumsUpdateStatusReceived(const int id, const QString &text);
   void SongsUpdateStatusReceived(const int id, const QString &text);
@@ -147,7 +138,8 @@ class QobuzService : public InternetService {
   void ArtistsUpdateProgressReceived(const int id, const int progress);
   void AlbumsUpdateProgressReceived(const int id, const int progress);
   void SongsUpdateProgressReceived(const int id, const int progress);
-  void HandleStreamURLFinished(const QUrl &original_url, const QUrl &stream_url, const Song::FileType filetype, const int samplerate, const int bit_depth, const qint64 duration, QString error);
+  void HandleStreamURLFailure(const uint id, const QUrl &original_url, const QString &error);
+  void HandleStreamURLSuccess(const uint id, const QUrl &original_url, const QUrl &stream_url, const Song::FileType filetype, const int samplerate, const int bit_depth, const qint64 duration);
 
  private:
   typedef QPair<QString, QString> Param;
@@ -157,6 +149,7 @@ class QobuzService : public InternetService {
   void LoginError(const QString &error = QString(), const QVariant &debug = QVariant());
 
   static const char *kAuthUrl;
+
   static const int kLoginAttempts;
   static const int kTimeResetLoginAttempts;
 
@@ -219,7 +212,8 @@ class QobuzService : public InternetService {
   bool login_sent_;
   int login_attempts_;
 
-  QList<QobuzStreamURLRequest*> stream_url_requests_;
+  uint next_stream_url_request_id_;
+  QMap<uint, std::shared_ptr<QobuzStreamURLRequest>> stream_url_requests_;
 
   QStringList login_errors_;
 
