@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,18 +41,17 @@
 #  include "subsonicscrobbler.h"
 #endif
 
-AudioScrobbler::AudioScrobbler(Application *app, QObject *parent) :
-  QObject(parent),
-  app_(app),
-  scrobbler_services_(new ScrobblerServices(this)),
-  enabled_(false),
-  offline_(false),
-  scrobble_button_(false),
-  love_button_(false),
-  submit_delay_(0),
-  prefer_albumartist_(false),
-  show_error_dialog_(false)
-  {
+AudioScrobbler::AudioScrobbler(Application *app, QObject *parent)
+    : QObject(parent),
+      app_(app),
+      scrobbler_services_(new ScrobblerServices(this)),
+      enabled_(false),
+      offline_(false),
+      scrobble_button_(false),
+      love_button_(false),
+      submit_delay_(0),
+      prefer_albumartist_(false),
+      show_error_dialog_(false) {
 
   scrobbler_services_->AddService(new LastFMScrobbler(app_, scrobbler_services_));
   scrobbler_services_->AddService(new LibreFMScrobbler(app_, scrobbler_services_));
@@ -94,7 +93,9 @@ void AudioScrobbler::ReloadSettings() {
              << Song::Source_Stream
              << Song::Source_Tidal
              << Song::Source_Subsonic
-             << Song::Source_Qobuz;
+             << Song::Source_Qobuz
+             << Song::Source_SomaFM
+             << Song::Source_RadioParadise;
   }
   else {
     for (const QString &source : sources) {
@@ -168,7 +169,7 @@ void AudioScrobbler::ClearPlaying() {
 
 }
 
-void AudioScrobbler::Scrobble(const Song &song, const int scrobble_point) {
+void AudioScrobbler::Scrobble(const Song &song, const qint64 scrobble_point) {
 
   if (!sources_.contains(song.source())) return;
 
@@ -208,6 +209,6 @@ void AudioScrobbler::WriteCache() {
 
 }
 
-void AudioScrobbler::ErrorReceived(QString error) {
+void AudioScrobbler::ErrorReceived(const QString &error) {
   emit ErrorMessage(error);
 }
