@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,14 +52,14 @@ class AudioScrobbler : public QObject {
 
   void UpdateNowPlaying(const Song &song);
   void ClearPlaying();
-  void Scrobble(const Song &song, const int scrobble_point);
+  void Scrobble(const Song &song, const qint64 scrobble_point);
   void ShowConfig();
 
   ScrobblerService *ServiceByName(const QString &name) const { return scrobbler_services_->ServiceByName(name); }
 
   template <typename T>
   T *Service() {
-    return qobject_cast<T*>(this->ServiceByName(T::kName));
+    return qobject_cast<T*>(ServiceByName(T::kName));
   }
 
  public slots:
@@ -68,7 +68,7 @@ class AudioScrobbler : public QObject {
   void Submit();
   void Love();
   void WriteCache();
-  void ErrorReceived(QString);
+  void ErrorReceived(const QString &error);
 
  signals:
   void ErrorMessage(QString);
@@ -78,7 +78,6 @@ class AudioScrobbler : public QObject {
   void LoveButtonVisibilityChanged(bool value);
 
  private:
-
   Application *app_;
   ScrobblerServices *scrobbler_services_;
 
