@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2020, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2020-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,10 @@
 
 const char *CoversSettingsPage::kSettingsGroup = "Covers";
 
-CoversSettingsPage::CoversSettingsPage(SettingsDialog *parent) : SettingsPage(parent), ui_(new Ui::CoversSettingsPage), provider_selected_(false) {
+CoversSettingsPage::CoversSettingsPage(SettingsDialog *dialog, QWidget *parent)
+    : SettingsPage(dialog, parent),
+      ui_(new Ui::CoversSettingsPage),
+      provider_selected_(false) {
 
   ui_->setupUi(this);
   setWindowIcon(IconLoader::Load("cdcase"));
@@ -65,7 +68,7 @@ CoversSettingsPage::CoversSettingsPage(SettingsDialog *parent) : SettingsPage(pa
   NoProviderSelected();
   DisableAuthentication();
 
-  dialog()->installEventFilter(this);
+  dialog->installEventFilter(this);
 
 }
 
@@ -94,9 +97,9 @@ void CoversSettingsPage::Load() {
 void CoversSettingsPage::Save() {
 
   QStringList providers;
-  for (int i = 0 ; i < ui_->providers->count() ; ++i) {
+  for (int i = 0; i < ui_->providers->count(); ++i) {
     const QListWidgetItem *item = ui_->providers->item(i);
-    if (item->checkState() == Qt::Checked) providers << item->text();
+    if (item->checkState() == Qt::Checked) providers << item->text();  // clazy:exclude=reserve-candidates
   }
 
   QSettings s;
@@ -118,27 +121,29 @@ void CoversSettingsPage::CurrentItemChanged(QListWidgetItem *item_current, QList
     ui_->providers_up->setEnabled(row != 0);
     ui_->providers_down->setEnabled(row != ui_->providers->count() - 1);
     CoverProvider *provider = dialog()->app()->cover_providers()->ProviderByName(item_current->text());
-    if (provider && provider->AuthenticationRequired()) {
-      if (provider->name() == "Tidal" && !provider->IsAuthenticated()) {
-        DisableAuthentication();
-        ui_->label_auth_info->setText(tr("Use Tidal settings to authenticate."));
-      }
-      else if (provider->name() == "Qobuz" && !provider->IsAuthenticated()) {
-        DisableAuthentication();
-        ui_->label_auth_info->setText(tr("Use Qobuz settings to authenticate."));
+    if (provider) {
+      if (provider->AuthenticationRequired()) {
+        if (provider->name() == "Tidal" && !provider->IsAuthenticated()) {
+          DisableAuthentication();
+          ui_->label_auth_info->setText(tr("Use Tidal settings to authenticate."));
+        }
+        else if (provider->name() == "Qobuz" && !provider->IsAuthenticated()) {
+          DisableAuthentication();
+          ui_->label_auth_info->setText(tr("Use Qobuz settings to authenticate."));
+        }
+        else {
+          ui_->login_state->SetLoggedIn(provider->IsAuthenticated() ? LoginStateWidget::LoggedIn : LoginStateWidget::LoggedOut);
+          ui_->button_authenticate->setEnabled(true);
+          ui_->button_authenticate->show();
+          ui_->login_state->show();
+          ui_->label_auth_info->setText(tr("%1 needs authentication.").arg(provider->name()));
+        }
       }
       else {
-        ui_->login_state->SetLoggedIn(provider->IsAuthenticated() ? LoginStateWidget::LoggedIn : LoginStateWidget::LoggedOut);
-        ui_->button_authenticate->setEnabled(true);
-        ui_->button_authenticate->show();
-        ui_->login_state->show();
-        ui_->label_auth_info->setText(tr("%1 needs authentication.").arg(provider->name()));
+        DisableAuthentication();
+        ui_->label_auth_info->setText(tr("%1 does not need authentication.").arg(provider->name()));
       }
     }
-    else {
-      DisableAuthentication();
-      ui_->label_auth_info->setText(tr("%1 does not need authentication.").arg(provider->name()));
-    }
     provider_selected_ = true;
   }
   else {
@@ -204,7 +209,7 @@ void CoversSettingsPage::DisableAuthentication() {
 
 }
 
-void CoversSettingsPage::DisconnectAuthentication(CoverProvider *provider) {
+void CoversSettingsPage::DisconnectAuthentication(CoverProvider *provider) const {
 
   QObject::disconnect(provider, &CoverProvider::AuthenticationFailure, this, &CoversSettingsPage::AuthenticationFailure);
   QObject::disconnect(provider, &CoverProvider::AuthenticationSuccess, this, &CoversSettingsPage::AuthenticationSuccess);
@@ -252,7 +257,7 @@ void CoversSettingsPage::AuthenticationSuccess() {
   if (!provider) return;
   DisconnectAuthentication(provider);
 
-  if (!this->isVisible() || !ui_->providers->currentItem() || ui_->providers->currentItem()->text() != provider->name()) return;
+  if (!isVisible() || !ui_->providers->currentItem() || ui_->providers->currentItem()->text() != provider->name()) return;
 
   ui_->login_state->SetLoggedIn(LoginStateWidget::LoggedIn);
   ui_->button_authenticate->setEnabled(true);
@@ -265,7 +270,7 @@ void CoversSettingsPage::AuthenticationFailure(const QStringList &errors) {
   if (!provider) return;
   DisconnectAuthentication(provider);
 
-  if (!this->isVisible() || !ui_->providers->currentItem() || ui_->providers->currentItem()->text() != provider->name()) return;
+  if (!isVisible() || !ui_->providers->currentItem() || ui_->providers->currentItem()->text() != provider->name()) return;
 
   QMessageBox::warning(this, tr("Authentication failed"), errors.join("\n"));
 
