@@ -2,7 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,6 @@
 #include "core/iconloader.h"
 #include "core/logging.h"
 #include "core/utilities.h"
-#ifdef Q_OS_MACOS
-#  include "core/mac_utilities.h"
-#endif
 #include "globalshortcuts/globalshortcutgrabber.h"
 #include "globalshortcuts/globalshortcutsmanager.h"
 #include "settingspage.h"
@@ -54,8 +51,8 @@
 
 const char *GlobalShortcutsSettingsPage::kSettingsGroup = "GlobalShortcuts";
 
-GlobalShortcutsSettingsPage::GlobalShortcutsSettingsPage(SettingsDialog *dialog)
-    : SettingsPage(dialog),
+GlobalShortcutsSettingsPage::GlobalShortcutsSettingsPage(SettingsDialog *dialog, QWidget *parent)
+    : SettingsPage(dialog, parent),
       ui_(new Ui_GlobalShortcutsSettingsPage),
       initialized_(false),
       grabber_(new GlobalShortcutGrabber) {
@@ -71,38 +68,32 @@ GlobalShortcutsSettingsPage::GlobalShortcutsSettingsPage(SettingsDialog *dialog)
   QObject::connect(ui_->radio_custom, &QRadioButton::clicked, this, &GlobalShortcutsSettingsPage::ChangeClicked);
   QObject::connect(ui_->button_change, &QPushButton::clicked, this, &GlobalShortcutsSettingsPage::ChangeClicked);
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
-#  ifdef HAVE_DBUS
-  QObject::connect(ui_->checkbox_gsd, &QCheckBox::clicked, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
-  QObject::connect(ui_->checkbox_kde, &QCheckBox::clicked, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
-  QObject::connect(ui_->button_gsd_open, &QPushButton::clicked, this, &GlobalShortcutsSettingsPage::OpenGnomeKeybindingProperties);
-#  endif
-#  ifdef HAVE_X11EXTRAS
-  QObject::connect(ui_->checkbox_x11, &QCheckBox::clicked, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
-#  endif
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
+  QObject::connect(ui_->checkbox_kde, &QCheckBox::toggled, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
+  QObject::connect(ui_->checkbox_gnome, &QCheckBox::toggled, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
+  QObject::connect(ui_->checkbox_mate, &QCheckBox::toggled, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
+  QObject::connect(ui_->button_gnome_open, &QPushButton::clicked, this, &GlobalShortcutsSettingsPage::OpenGnomeKeybindingProperties);
+  QObject::connect(ui_->button_mate_open, &QPushButton::clicked, this, &GlobalShortcutsSettingsPage::OpenMateKeybindingProperties);
 #else
-  ui_->widget_gsd->hide();
   ui_->widget_kde->hide();
+  ui_->widget_gnome->hide();
+  ui_->widget_mate->hide();
+#endif  // defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
+
+#ifdef HAVE_X11_GLOBALSHORTCUTS
+  QObject::connect(ui_->checkbox_x11, &QCheckBox::toggled, this, &GlobalShortcutsSettingsPage::ShortcutOptionsChanged);
+#else
   ui_->widget_x11->hide();
-#endif  // defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
+#endif  // HAVE_X11_GLOBALSHORTCUTS
+
+#ifndef Q_OS_MACOS
+  ui_->widget_macos_access->hide();
+#endif  // Q_OS_MACOS
 
 }
 
 GlobalShortcutsSettingsPage::~GlobalShortcutsSettingsPage() { delete ui_; }
 
-bool GlobalShortcutsSettingsPage::IsEnabled() const {
-
-#ifdef Q_OS_MACOS
-  qLog(Debug) << Utilities::GetMacOsVersion();
-  if (Utilities::GetMacOsVersion() < 6) {  // Leopard and earlier.
-    return false;
-  }
-#endif
-
-  return true;
-
-}
-
 void GlobalShortcutsSettingsPage::Load() {
 
   QSettings s;
@@ -116,19 +107,13 @@ void GlobalShortcutsSettingsPage::Load() {
     de_ = Utilities::DesktopEnvironment();
     ui_->widget_warning->hide();
 
-    QObject::connect(ui_->button_macos_open, &QPushButton::clicked, manager, &GlobalShortcutsManager::ShowMacAccessibilityDialog);
+#ifdef Q_OS_MACOS
+    QObject::connect(ui_->button_macos_preferences, &QPushButton::clicked, manager, &GlobalShortcutsManager::ShowMacAccessibilityDialog);
+#endif
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
-    if (manager->IsGsdAvailable()) {
-      qLog(Debug) << "Gnome (GSD) backend is available.";
-      ui_->widget_gsd->show();
-    }
-    else {
-      qLog(Debug) << "Gnome (GSD) backend is unavailable.";
-      ui_->widget_gsd->hide();
-    }
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
 
-    if (manager->IsKdeAvailable()) {
+    if (GlobalShortcutsManager::IsKdeAvailable()) {
       qLog(Debug) << "KDE (KGlobalAccel) backend is available.";
       ui_->widget_kde->show();
     }
@@ -137,7 +122,28 @@ void GlobalShortcutsSettingsPage::Load() {
       ui_->widget_kde->hide();
     }
 
-    if (manager->IsX11Available()) {
+    if (GlobalShortcutsManager::IsGnomeAvailable()) {
+      qLog(Debug) << "Gnome (GSD) backend is available.";
+      ui_->widget_gnome->show();
+    }
+    else {
+      qLog(Debug) << "Gnome (GSD) backend is unavailable.";
+      ui_->widget_gnome->hide();
+    }
+
+    if (GlobalShortcutsManager::IsMateAvailable()) {
+      qLog(Debug) << "MATE backend is available.";
+      ui_->widget_mate->show();
+    }
+    else {
+      qLog(Debug) << "MATE backend is unavailable.";
+      ui_->widget_mate->hide();
+    }
+
+#endif  // defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
+
+#ifdef HAVE_X11_GLOBALSHORTCUTS
+    if (GlobalShortcutsManager::IsX11Available()) {
       qLog(Debug) << "X11 backend is available.";
       ui_->widget_x11->show();
     }
@@ -145,9 +151,10 @@ void GlobalShortcutsSettingsPage::Load() {
       qLog(Debug) << "X11 backend is unavailable.";
       ui_->widget_x11->hide();
     }
-#endif
+#endif  // HAVE_X11_GLOBALSHORTCUTS
 
-    for (const GlobalShortcutsManager::Shortcut &i : manager->shortcuts().values()) {
+    QList<GlobalShortcutsManager::Shortcut> shortcuts = manager->shortcuts().values();
+    for (const GlobalShortcutsManager::Shortcut &i : shortcuts) {
       Shortcut shortcut;
       shortcut.s = i;
       shortcut.key = i.action->shortcut();
@@ -160,33 +167,39 @@ void GlobalShortcutsSettingsPage::Load() {
     ItemClicked(ui_->list->topLevelItem(0));
   }
 
-  for (const Shortcut &shortcut : shortcuts_.values()) {
+  QList<Shortcut> shortcuts = shortcuts_.values();
+  for (const Shortcut &shortcut : shortcuts) {
     SetShortcut(shortcut.s.id, shortcut.s.action->shortcut());
   }
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
-  if (ui_->widget_gsd->isVisibleTo(this)) {
-    ui_->checkbox_gsd->setChecked(s.value("use_gsd", true).toBool());
-  }
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
 
   if (ui_->widget_kde->isVisibleTo(this)) {
     ui_->checkbox_kde->setChecked(s.value("use_kde", true).toBool());
   }
 
+  if (ui_->widget_gnome->isVisibleTo(this)) {
+    ui_->checkbox_gnome->setChecked(s.value("use_gnome", true).toBool());
+  }
+
+  if (ui_->widget_mate->isVisibleTo(this)) {
+    ui_->checkbox_mate->setChecked(s.value("use_mate", true).toBool());
+  }
+
+#endif // defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
+
+#ifdef HAVE_X11_GLOBALSHORTCUTS
   if (ui_->widget_x11->isVisibleTo(this)) {
     ui_->checkbox_x11->setChecked(s.value("use_x11", false).toBool());
   }
-#endif
+#endif  // HAVE_X11_GLOBALSHORTCUTS
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && (defined(HAVE_DBUS) || defined(HAVE_X11_GLOBALSHORTCUTS))
   ShortcutOptionsChanged();
 #endif
 
-  ui_->widget_macos->setVisible(!manager->IsMacAccessibilityEnabled());
 #ifdef Q_OS_MACOS
-  qint32 macos_version = Utilities::GetMacOsVersion();
-  ui_->label_macos->setVisible(macos_version < 9);
-  ui_->label_macos_mavericks->setVisible(macos_version >= 9);
+  ui_->widget_macos_access->setVisible(!GlobalShortcutsManager::IsMacAccessibilityEnabled());
 #endif  // Q_OS_MACOS
 
   s.endGroup();
@@ -202,17 +215,22 @@ void GlobalShortcutsSettingsPage::Save() {
   QSettings s;
   s.beginGroup(kSettingsGroup);
 
-  for (const Shortcut &shortcut : shortcuts_.values()) {
+  QList<Shortcut> shortcuts = shortcuts_.values();
+  for (const Shortcut &shortcut : shortcuts) {
     shortcut.s.action->setShortcut(shortcut.key);
     shortcut.s.shortcut->setKey(shortcut.key);
     s.setValue(shortcut.s.id, shortcut.key.toString());
   }
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
-  s.setValue("use_gsd", ui_->checkbox_gsd->isChecked());
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
   s.setValue("use_kde", ui_->checkbox_kde->isChecked());
+  s.setValue("use_gnome", ui_->checkbox_gnome->isChecked());
+  s.setValue("use_mate", ui_->checkbox_mate->isChecked());
+#endif  // defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && defined(HAVE_DBUS)
+
+#ifdef HAVE_X11_GLOBALSHORTCUTS
   s.setValue("use_x11", ui_->checkbox_x11->isChecked());
-#endif
+#endif  // HAVE_X11_GLOBALSHORTCUTS
 
   s.endGroup();
 
@@ -248,6 +266,16 @@ void GlobalShortcutsSettingsPage::OpenGnomeKeybindingProperties() {
 
 }
 
+void GlobalShortcutsSettingsPage::OpenMateKeybindingProperties() {
+
+  if (!QProcess::startDetached("mate-keybinding-properties", QStringList())) {
+    if (!QProcess::startDetached("mate-control-center", QStringList() << "keyboard")) {
+      QMessageBox::warning(this, "Error", tr("The \"%1\" command could not be started.").arg("mate-keybinding-properties"));
+    }
+  }
+
+}
+
 void GlobalShortcutsSettingsPage::SetShortcut(const QString &id, const QKeySequence &key) {
 
   Shortcut &shortcut = shortcuts_[id];
@@ -266,12 +294,15 @@ void GlobalShortcutsSettingsPage::ItemClicked(QTreeWidgetItem *item) {
   ui_->shortcut_options->setEnabled(true);
   ui_->shortcut_options->setTitle(tr("Shortcut for %1").arg(shortcut.s.action->text()));
 
-  if (shortcut.key == shortcut.s.default_key)
+  if (shortcut.key == shortcut.s.default_key) {
     ui_->radio_default->setChecked(true);
-  else if (shortcut.key.isEmpty())
+  }
+  else if (shortcut.key.isEmpty()) {
     ui_->radio_none->setChecked(true);
-  else
+  }
+  else {
     ui_->radio_custom->setChecked(true);
+  }
 
 }
 
@@ -299,7 +330,8 @@ void GlobalShortcutsSettingsPage::ChangeClicked() {
   if (key.isEmpty()) return;
 
   // Check if this key sequence is used by any other actions
-  for (const QString &id : shortcuts_.keys()) {
+  QStringList ids = shortcuts_.keys();
+  for (const QString &id : ids) {
     if (shortcuts_[id].key == key) SetShortcut(id, QKeySequence());
   }
 
@@ -312,14 +344,21 @@ void GlobalShortcutsSettingsPage::ChangeClicked() {
 
 void GlobalShortcutsSettingsPage::X11Warning() {
 
-  if (de_.toLower() == "kde" || de_.toLower() == "gnome" || de_.toLower() == "x-cinnamon") {
+  QString de = de_.toLower();
+  if (de == "kde" || de == "gnome" || de == "x-cinnamon" || de == "mate") {
     QString text(tr("Using X11 shortcuts on %1 is not recommended and can cause keyboard to become unresponsive!").arg(de_));
-    if (de_.toLower() == "kde")
+    if (de == "kde") {
       text += tr(" Shortcuts on %1 are usually used through MPRIS and KGlobalAccel.").arg(de_);
-    else if (de_.toLower() == "gnome")
-      text += tr(" Shortcuts on %1 are usually used through GSD and should be configured in gnome-settings-daemon instead.").arg(de_);
-    else if (de_.toLower() == "x-cinnamon")
-      text += tr(" Shortcuts on %1 are usually used through GSD and should be configured in cinnamon-settings-daemon instead.").arg(de_);
+    }
+    else if (de == "gnome") {
+      text += tr(" Shortcuts on %1 are usually used through Gnome Settings Daemon and should be configured in gnome-settings-daemon instead.").arg(de_);
+    }
+    else if (de == "x-cinnamon") {
+      text += tr(" Shortcuts on %1 are usually used through Gnome Settings Daemon and should be configured in cinnamon-settings-daemon instead.").arg(de_);
+    }
+    else if (de == "mate") {
+      text += tr(" Shortcuts on %1 are usually used through MATE Settings Daemon and should be configured there instead.").arg(de_);
+    }
     ui_->label_warn_text->setText(text);
     ui_->widget_warning->show();
   }
