@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,8 +57,10 @@
 class QHideEvent;
 class QShowEvent;
 
-NotificationsSettingsPage::NotificationsSettingsPage(SettingsDialog *dialog)
-    : SettingsPage(dialog), ui_(new Ui_NotificationsSettingsPage), pretty_popup_(new OSDPretty(OSDPretty::Mode_Draggable)) {
+NotificationsSettingsPage::NotificationsSettingsPage(SettingsDialog *dialog, QWidget *parent)
+    : SettingsPage(dialog, parent),
+      ui_(new Ui_NotificationsSettingsPage),
+      pretty_popup_(new OSDPretty(OSDPretty::Mode_Draggable)) {
 
   ui_->setupUi(this);
   setWindowIcon(IconLoader::Load("help-hint"));
@@ -108,9 +111,18 @@ NotificationsSettingsPage::NotificationsSettingsPage(SettingsDialog *dialog)
   QObject::connect(ui_->notifications_exp_chooser2, &QToolButton::triggered, this, &NotificationsSettingsPage::InsertVariableSecondLine);
   QObject::connect(ui_->notifications_disable_duration, &QCheckBox::toggled, ui_->notifications_duration, &NotificationsSettingsPage::setDisabled);
 
-  if (!dialog->osd()->SupportsNativeNotifications())
+#ifdef Q_OS_WIN32
+  if (!dialog->osd()->SupportsNativeNotifications() && !dialog->osd()->SupportsTrayPopups()) {
     ui_->notifications_native->setEnabled(false);
-  if (!dialog->osd()->SupportsTrayPopups()) ui_->notifications_tray->setEnabled(false);
+  }
+#else
+  if (!dialog->osd()->SupportsNativeNotifications()) {
+    ui_->notifications_native->setEnabled(false);
+  }
+#endif
+  if (!dialog->osd()->SupportsTrayPopups()) {
+    ui_->notifications_tray->setEnabled(false);
+  }
 
   QObject::connect(ui_->notifications_pretty, &QRadioButton::toggled, this, &NotificationsSettingsPage::UpdatePopupVisible);
 
@@ -146,11 +158,15 @@ void NotificationsSettingsPage::Load() {
   OSDBase::Behaviour osd_behaviour = OSDBase::Behaviour(s.value("Behaviour", OSDBase::Native).toInt());
   switch (osd_behaviour) {
     case OSDBase::Native:
+#ifdef Q_OS_WIN32
+      if (dialog()->osd()->SupportsNativeNotifications() || dialog()->osd()->SupportsTrayPopups()) {
+#else
       if (dialog()->osd()->SupportsNativeNotifications()) {
+#endif
         ui_->notifications_native->setChecked(true);
         break;
       }
-    // Fallthrough
+      // Fallthrough
 
     case OSDBase::Pretty:
       ui_->notifications_pretty->setChecked(true);
@@ -161,7 +177,7 @@ void NotificationsSettingsPage::Load() {
         ui_->notifications_tray->setChecked(true);
         break;
       }
-    // Fallthrough
+      // Fallthrough
 
     case OSDBase::Disabled:
     default:
@@ -185,17 +201,23 @@ void NotificationsSettingsPage::Load() {
 
   // Pretty OSD
   pretty_popup_->ReloadSettings();
-  ui_->notifications_opacity->setValue(pretty_popup_->background_opacity() * 100);
+  ui_->notifications_opacity->setValue(static_cast<int>(pretty_popup_->background_opacity() * 100));
 
   QRgb color = pretty_popup_->background_color();
-  if (color == OSDPretty::kPresetBlue)
+  if (color == OSDPretty::kPresetBlue) {
     ui_->notifications_bg_preset->setCurrentIndex(0);
-  else if (color == OSDPretty::kPresetRed)
+  }
+  else if (color == OSDPretty::kPresetRed) {
     ui_->notifications_bg_preset->setCurrentIndex(1);
-  else
+  }
+  else {
     ui_->notifications_bg_preset->setCurrentIndex(2);
+  }
   ui_->notifications_bg_preset->setItemData(2, QColor(color), Qt::DecorationRole);
   ui_->notifications_disable_duration->setChecked(pretty_popup_->disable_duration());
+
+  ui_->notifications_fading->setChecked(pretty_popup_->fading());
+
   UpdatePopupVisible();
 
   Init(ui_->layout_notificationssettingspage->parentWidget());
@@ -215,7 +237,7 @@ void NotificationsSettingsPage::Save() {
   else if (ui_->notifications_pretty->isChecked()) osd_behaviour = OSDBase::Pretty;
 
   s.beginGroup(OSDBase::kSettingsGroup);
-  s.setValue("Behaviour", int(osd_behaviour));
+  s.setValue("Behaviour", static_cast<int>(osd_behaviour));
   s.setValue("Timeout", ui_->notifications_duration->value() * 1000);
   s.setValue("ShowOnVolumeChange", ui_->notifications_volume->isChecked());
   s.setValue("ShowOnPlayModeChange", ui_->notifications_play_mode->isChecked());
@@ -235,6 +257,7 @@ void NotificationsSettingsPage::Save() {
   s.setValue("popup_pos", pretty_popup_->popup_pos());
   s.setValue("font", pretty_popup_->font().toString());
   s.setValue("disable_duration", ui_->notifications_disable_duration->isChecked());
+  s.setValue("fading", ui_->notifications_fading->isChecked());
   s.endGroup();
 
 }
@@ -278,8 +301,7 @@ void NotificationsSettingsPage::PrettyColorPresetChanged(int index) {
 void NotificationsSettingsPage::ChooseBgColor() {
 
   QColor color = QColorDialog::getColor(pretty_popup_->background_color(), this);
-  if (!color.isValid())
-    return;
+  if (!color.isValid()) return;
 
   pretty_popup_->set_background_color(color.rgb());
   ui_->notifications_bg_preset->setItemData(2, color, Qt::DecorationRole);
@@ -291,8 +313,7 @@ void NotificationsSettingsPage::ChooseBgColor() {
 void NotificationsSettingsPage::ChooseFgColor() {
 
   QColor color = QColorDialog::getColor(pretty_popup_->foreground_color(), this);
-  if (!color.isValid())
-    return;
+  if (!color.isValid()) return;
 
   pretty_popup_->set_foreground_color(color.rgb());
 
@@ -302,7 +323,7 @@ void NotificationsSettingsPage::ChooseFgColor() {
 
 void NotificationsSettingsPage::ChooseFont() {
 
-  bool ok;
+  bool ok = false;
   QFont font = QFontDialog::getFont(&ok, pretty_popup_->font(), this);
   if (ok) {
     pretty_popup_->set_font(font);
@@ -341,17 +362,17 @@ void NotificationsSettingsPage::PrepareNotificationPreview() {
 
 }
 
-void NotificationsSettingsPage::InsertVariableFirstLine(QAction* action) {
+void NotificationsSettingsPage::InsertVariableFirstLine(QAction *action) {
   // We use action name, therefore those shouldn't be translatable
   ui_->notifications_custom_text1->insert(action->text());
 }
 
-void NotificationsSettingsPage::InsertVariableSecondLine(QAction* action) {
+void NotificationsSettingsPage::InsertVariableSecondLine(QAction *action) {
   // We use action name, therefore those shouldn't be translatable
   ui_->notifications_custom_text2->insert(action->text());
 }
 
-void NotificationsSettingsPage::ShowMenuTooltip(QAction* action) {
+void NotificationsSettingsPage::ShowMenuTooltip(QAction *action) {
   QToolTip::showText(QCursor::pos(), action->toolTip());
 }
 
