@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +36,7 @@ class PlaylistSettingsPage : public SettingsPage {
   Q_OBJECT
 
  public:
-  explicit PlaylistSettingsPage(SettingsDialog* dialog);
+  explicit PlaylistSettingsPage(SettingsDialog *dialog, QWidget *parent = nullptr);
   ~PlaylistSettingsPage() override;
   static const char *kSettingsGroup;
 
@@ -43,7 +44,7 @@ class PlaylistSettingsPage : public SettingsPage {
   void Save() override;
 
  private:
-  Ui_PlaylistSettingsPage* ui_;
+  Ui_PlaylistSettingsPage *ui_;
 
 };
 
