@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2019-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,7 +49,7 @@ class SettingsPage : public QWidget {
   Q_OBJECT
 
  public:
-  explicit SettingsPage(SettingsDialog *dialog);
+  explicit SettingsPage(SettingsDialog *dialog, QWidget *parent = nullptr);
 
   void Init(QWidget *ui_widget);
 
@@ -66,8 +67,8 @@ class SettingsPage : public QWidget {
 
   void set_changed() { changed_ = true; }
 
-  void ComboBoxLoadFromSettings(const QSettings &s, QComboBox *combobox, const QString &setting, const QString &default_value);
-  void ComboBoxLoadFromSettings(const QSettings &s, QComboBox *combobox, const QString &setting, const int default_value);
+  static void ComboBoxLoadFromSettings(const QSettings &s, QComboBox *combobox, const QString &setting, const QString &default_value);
+  static void ComboBoxLoadFromSettings(const QSettings &s, QComboBox *combobox, const QString &setting, const int default_value);
 
  private:
   virtual void Save() = 0;
