@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,10 +43,10 @@
 
 const char *TidalSettingsPage::kSettingsGroup = "Tidal";
 
-TidalSettingsPage::TidalSettingsPage(SettingsDialog *parent)
-    : SettingsPage(parent),
+TidalSettingsPage::TidalSettingsPage(SettingsDialog *dialog, QWidget *parent)
+    : SettingsPage(dialog, parent),
       ui_(new Ui::TidalSettingsPage),
-      service_(dialog()->app()->internet_services()->Service<TidalService>()) {
+      service_(dialog->app()->internet_services()->Service<TidalService>()) {
 
   ui_->setupUi(this);
   setWindowIcon(IconLoader::Load("tidal"));
@@ -61,7 +61,7 @@ TidalSettingsPage::TidalSettingsPage(SettingsDialog *parent)
   QObject::connect(service_, &InternetService::LoginFailure, this, &TidalSettingsPage::LoginFailure);
   QObject::connect(service_, &InternetService::LoginSuccess, this, &TidalSettingsPage::LoginSuccess);
 
-  dialog()->installEventFilter(this);
+  dialog->installEventFilter(this);
 
   ui_->quality->addItem("Low", "LOW");
   ui_->quality->addItem("High", "HIGH");
@@ -148,8 +148,6 @@ void TidalSettingsPage::Save() {
   s.setValue("album_explicit", ui_->checkbox_album_explicit->isChecked());
   s.endGroup();
 
-  service_->ReloadSettings();
-
 }
 
 void TidalSettingsPage::LoginClicked() {
@@ -184,7 +182,6 @@ bool TidalSettingsPage::eventFilter(QObject *object, QEvent *event) {
 
   if (object == dialog() && event->type() == QEvent::Enter) {
     ui_->button_login->setEnabled(true);
-    return false;
   }
 
   return SettingsPage::eventFilter(object, event);
@@ -201,19 +198,25 @@ void TidalSettingsPage::OAuthClicked(const bool enabled) {
 }
 
 void TidalSettingsPage::LogoutClicked() {
+
   service_->Logout();
   ui_->button_login->setEnabled(true);
   ui_->login_state->SetLoggedIn(LoginStateWidget::LoggedOut);
+
 }
 
 void TidalSettingsPage::LoginSuccess() {
-  if (!this->isVisible()) return;
+
+  if (!isVisible()) return;
   ui_->login_state->SetLoggedIn(LoginStateWidget::LoggedIn);
   ui_->button_login->setEnabled(true);
+
 }
 
 void TidalSettingsPage::LoginFailure(const QString &failure_reason) {
-  if (!this->isVisible()) return;
+
+  if (!isVisible()) return;
   QMessageBox::warning(this, tr("Authentication failed"), failure_reason);
   ui_->button_login->setEnabled(true);
+
 }
