@@ -51,7 +51,7 @@ class SmartPlaylistSearchTermWidget : public QWidget {
   float overlay_opacity() const;
   void set_overlay_opacity(const float opacity);
 
-  void SetTerm(const SmartPlaylistSearchTerm& term);
+  void SetTerm(const SmartPlaylistSearchTerm &term);
   SmartPlaylistSearchTerm Term() const;
 
  signals:
@@ -72,7 +72,7 @@ class SmartPlaylistSearchTermWidget : public QWidget {
 
  private slots:
   void FieldChanged(const int index);
-  void OpChanged(const int index);
+  void OpChanged(const int idx);
   void RelativeValueChanged();
   void Grab();
 
