@@ -1,6 +1,6 @@
 /*
  * Strawberry Music Player
- * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,15 +40,12 @@
 #include "tidalservice.h"
 #include "tidalbaserequest.h"
 
-const char *TidalBaseRequest::kApiUrl = "https://api.tidalhifi.com/v1";
-
-TidalBaseRequest::TidalBaseRequest(TidalService *service, NetworkAccessManager *network, QObject *parent) :
-      QObject(parent),
+TidalBaseRequest::TidalBaseRequest(TidalService *service, NetworkAccessManager *network, QObject *parent)
+    : QObject(parent),
       service_(service),
-      network_(network)
-      {}
+      network_(network) {}
 
-QNetworkReply *TidalBaseRequest::CreateRequest(const QString &ressource_name, const QList<Param> &params_provided) {
+QNetworkReply *TidalBaseRequest::CreateRequest(const QString &ressource_name, const ParamList &params_provided) {
 
   ParamList params = ParamList() << params_provided
                                  << Param("countryCode", country_code());
@@ -58,7 +55,7 @@ QNetworkReply *TidalBaseRequest::CreateRequest(const QString &ressource_name, co
     url_query.addQueryItem(QUrl::toPercentEncoding(param.first), QUrl::toPercentEncoding(param.second));
   }
 
-  QUrl url(kApiUrl + QString("/") + ressource_name);
+  QUrl url(TidalService::kApiUrl + QString("/") + ressource_name);
   url.setQuery(url_query);
   QNetworkRequest req(url);
 #if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
@@ -67,8 +64,8 @@ QNetworkReply *TidalBaseRequest::CreateRequest(const QString &ressource_name, co
   req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
 #endif
   req.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-  if (!access_token().isEmpty()) req.setRawHeader("authorization", "Bearer " + access_token().toUtf8());
-  if (!session_id().isEmpty()) req.setRawHeader("X-Tidal-SessionId", session_id().toUtf8());
+  if (oauth() && !access_token().isEmpty()) req.setRawHeader("authorization", "Bearer " + access_token().toUtf8());
+  else if (!session_id().isEmpty()) req.setRawHeader("X-Tidal-SessionId", session_id().toUtf8());
 
   QNetworkReply *reply = network_->get(req);
   QObject::connect(reply, &QNetworkReply::sslErrors, this, &TidalBaseRequest::HandleSSLErrors);
@@ -79,9 +76,9 @@ QNetworkReply *TidalBaseRequest::CreateRequest(const QString &ressource_name, co
 
 }
 
-void TidalBaseRequest::HandleSSLErrors(QList<QSslError> ssl_errors) {
+void TidalBaseRequest::HandleSSLErrors(const QList<QSslError> &ssl_errors) {
 
-  for (QSslError &ssl_error : ssl_errors) {
+  for (const QSslError &ssl_error : ssl_errors) {
     Error(ssl_error.errorString());
   }
 
@@ -125,12 +122,17 @@ QByteArray TidalBaseRequest::GetReplyData(QNetworkReply *reply, const bool send_
         }
       }
       if (status == 401 && sub_status == 6001) {  // User does not have a valid session
-        emit service_->Logout();
+        service_->Logout();
         if (!oauth() && send_login && login_attempts() < max_login_attempts() && !api_token().isEmpty() && !username().isEmpty() && !password().isEmpty()) {
           qLog(Error) << "Tidal:" << error;
-          qLog(Info) << "Tidal:" << "Attempting to login.";
-          NeedLogin();
-          emit service_->Login();
+          set_need_login();
+          if (login_sent()) {
+            qLog(Info) << "Tidal:" << "Waiting for login.";
+          }
+          else {
+            qLog(Info) << "Tidal:" << "Attempting to login.";
+            emit RequestLogin();
+          }
         }
         else {
           Error(error);
@@ -169,7 +171,7 @@ QJsonObject TidalBaseRequest::ExtractJsonObj(const QByteArray &data) {
 
   QJsonObject json_obj = json_doc.object();
   if (json_obj.isEmpty()) {
-     Error("Received empty Json object.", json_doc);
+    Error("Received empty Json object.", json_doc);
     return QJsonObject();
   }
 
