@@ -32,14 +32,22 @@
 class QHideEvent;
 class QShowEvent;
 
-BusyIndicator::BusyIndicator(const QString &text, QWidget* parent)
-  : QWidget(parent) {
+BusyIndicator::BusyIndicator(const QString &text, QWidget *parent)
+    : QWidget(parent),
+      movie_(nullptr),
+      label_(nullptr) {
+
   Init(text);
+
 }
 
-BusyIndicator::BusyIndicator(QWidget* parent)
-  : QWidget(parent) {
+BusyIndicator::BusyIndicator(QWidget *parent)
+  : QWidget(parent),
+    movie_(nullptr),
+    label_(nullptr) {
+
   Init(QString());
+
 }
 
 void BusyIndicator::Init(const QString &text) {
@@ -84,4 +92,3 @@ void BusyIndicator::set_text(const QString &text) {
 QString BusyIndicator::text() const {
   return label_->text();
 }
-
