@@ -22,10 +22,7 @@
 #include <QWidget>
 #include <QLabel>
 
-ClickableLabel::ClickableLabel(QWidget *parent)
-  : QLabel(parent)
-{
-}
+ClickableLabel::ClickableLabel(QWidget *parent) : QLabel(parent){}
 
 void ClickableLabel::mousePressEvent(QMouseEvent *event) {
   emit Clicked();
