@@ -24,21 +24,20 @@
 #include "config.h"
 
 #include <QObject>
-#include <QWidget>
 #include <QListView>
 #include <QAbstractItemModel>
 #include <QAbstractItemView>
 #include <QItemSelectionModel>
 #include <QString>
 #include <QFont>
-#include <QPainter>
 #include <QPalette>
 #include <QPoint>
 #include <QRect>
 #include <QRegion>
 #include <QVector>
-#include <QtEvents>
 
+class QWidget;
+class QPainter;
 class QModelIndex;
 class QPaintEvent;
 class QResizeEvent;
@@ -60,49 +59,49 @@ class GroupedIconView : public QListView {
   Q_PROPERTY(QString header_text READ header_text WRITE set_header_text)
 
  public:
-  GroupedIconView(QWidget *parent = nullptr);
+  explicit GroupedIconView(QWidget *parent = nullptr);
 
   enum Role {
     Role_Group = 1158300,
   };
 
-  void AddSortSpec(int role, Qt::SortOrder order = Qt::AscendingOrder);
+  void AddSortSpec(const int role, const Qt::SortOrder order = Qt::AscendingOrder);
 
   int header_spacing() const { return header_spacing_; }
   int header_indent() const { return header_indent_; }
   int item_indent() const { return item_indent_; }
   const QString &header_text() const { return header_text_;}
 
-  void set_header_spacing(int value) { header_spacing_ = value; }
-  void set_header_indent(int value) { header_indent_ = value; }
-  void set_item_indent(int value) { item_indent_ = value; }
+  void set_header_spacing(const int value) { header_spacing_ = value; }
+  void set_header_indent(const int value) { header_indent_ = value; }
+  void set_item_indent(const int value) { item_indent_ = value; }
   void set_header_text(const QString &value) { header_text_ = value; }
 
   // QAbstractItemView
-  QModelIndex moveCursor(CursorAction action, Qt::KeyboardModifiers modifiers);
-  void setModel(QAbstractItemModel *model);
+  QModelIndex moveCursor(CursorAction action, Qt::KeyboardModifiers modifiers) override;
+  void setModel(QAbstractItemModel *model) override;
 
-  static void DrawHeader(QPainter *painter, const QRect &rect, const QFont &font, const QPalette &palette, const QString &text);
+  static void DrawHeader(QPainter *painter, const QRect rect, const QFont &font, const QPalette &palette, const QString &text);
 
-protected:
+ protected:
   virtual int header_height() const;
 
   // QWidget
-  void paintEvent(QPaintEvent *e);
-  void resizeEvent(QResizeEvent *e);
+  void paintEvent(QPaintEvent *e) override;
+  void resizeEvent(QResizeEvent *e) override;
 
   // QAbstractItemView
-  void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int>& = QVector<int>());
-  QModelIndex indexAt(const QPoint &p) const;
-  void rowsInserted(const QModelIndex &parent, int start, int end);
-  void setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command);
-  QRect visualRect(const QModelIndex &index) const;
-  QRegion visualRegionForSelection(const QItemSelection &selection) const;
+  void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int>& = QVector<int>()) override;
+  QModelIndex indexAt(const QPoint &p) const override;
+  void rowsInserted(const QModelIndex &parent, int start, int end) override;
+  void setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command) override;
+  QRect visualRect(const QModelIndex &idx) const override;
+  QRegion visualRegionForSelection(const QItemSelection &selection) const override;
 
-private slots:
+ private slots:
   void LayoutItems();
 
-private:
+ private:
   static const int kBarThickness;
   static const int kBarMarginTop;
 
@@ -113,10 +112,10 @@ private:
   };
 
   // Returns the items that are wholly or partially inside the rect.
-  QVector<QModelIndex> IntersectingItems(const QRect &rect) const;
+  QVector<QModelIndex> IntersectingItems(const QRect rect) const;
 
   // Returns the index of the item above (d=-1) or below (d=+1) the given item.
-  int IndexAboveOrBelow(int index, int d) const;
+  int IndexAboveOrBelow(int index, const int d) const;
 
   MultiSortFilterProxy *proxy_model_;
   QVector<QRect> visual_rects_;
