@@ -24,25 +24,24 @@
 #include "config.h"
 
 #include <QObject>
-#include <QWidget>
 #include <QSize>
 #include <QString>
 #include <QTextEdit>
-#include <QtEvents>
 
+class QWidget;
 class QKeyEvent;
 
 class LineTextEdit : public QTextEdit {
   Q_OBJECT
 
  public:
-  LineTextEdit(QWidget *parent = nullptr);
+  explicit LineTextEdit(QWidget *parent = nullptr);
 
-  QSize sizeHint() const;
-  QSize minimumSizeHint() const;
+  QSize sizeHint() const override;
+  QSize minimumSizeHint() const override;
 
-protected:
-  void keyPressEvent(QKeyEvent *e);
+ protected:
+  void keyPressEvent(QKeyEvent *e) override;
 };
 
 #endif  // LINETEXTEDIT_H
