@@ -43,15 +43,15 @@ class MultiLoadingIndicator : public QWidget {
   static const int kHorizontalPadding;
   static const int kSpacing;
 
-  void SetTaskManager(TaskManager* task_manager);
+  void SetTaskManager(TaskManager *task_manager);
 
-  QSize sizeHint() const;
+  QSize sizeHint() const override;
 
  signals:
   void TaskCountChange(int tasks);
 
  protected:
-  void paintEvent(QPaintEvent *);
+  void paintEvent(QPaintEvent*) override;
 
  private slots:
   void UpdateText();
@@ -64,4 +64,3 @@ class MultiLoadingIndicator : public QWidget {
 };
 
 #endif  // MULTILOADINGINDICATOR_H
-
