@@ -27,11 +27,9 @@
 #include "stickyslider.h"
 
 StickySlider::StickySlider(QWidget *parent)
-  : QSlider(parent),
-    sticky_center_(-1),
-    sticky_threshold_(10)
-{
-}
+    : QSlider(parent),
+      sticky_center_(-1),
+      sticky_threshold_(10) {}
 
 void StickySlider::mouseMoveEvent(QMouseEvent *e) {
 
@@ -40,6 +38,8 @@ void StickySlider::mouseMoveEvent(QMouseEvent *e) {
   if (sticky_center_ == -1) return;
 
   const int v = sliderPosition();
-  if (v <= sticky_center_ + sticky_threshold_ && v >= sticky_center_ - sticky_threshold_) setSliderPosition(sticky_center_);
+  if (v <= sticky_center_ + sticky_threshold_ && v >= sticky_center_ - sticky_threshold_) {
+    setSliderPosition(sticky_center_);
+  }
 
 }
