@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This file was part of Clementine.
  * Copyright 2010, David Sansome <me@davidsansome.com>
+ * Copyright 2018-2021, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,10 +44,11 @@ const int StretchHeaderView::kMagicNumber = 0x502c950f;
 StretchHeaderView::StretchHeaderView(const Qt::Orientation orientation, QWidget *parent)
     : QHeaderView(orientation, parent),
       stretch_enabled_(false),
-    in_mouse_move_event_(false) {
+      in_mouse_move_event_(false) {
 
   QObject::connect(this, &StretchHeaderView::sectionResized, this, &StretchHeaderView::SectionResized);
   setMinimumSectionSize(kMinimumColumnWidth);
+  setTextElideMode(Qt::ElideRight);
 
 }
 
@@ -61,7 +63,7 @@ void StretchHeaderView::setModel(QAbstractItemModel *model) {
 
 }
 
-void StretchHeaderView::NormaliseWidths(const QList<int>& sections) {
+void StretchHeaderView::NormaliseWidths(const QList<int> &sections) {
 
   if (!stretch_enabled_) return;
 
@@ -70,44 +72,45 @@ void StretchHeaderView::NormaliseWidths(const QList<int>& sections) {
 
   if (!sections.isEmpty()) {
     selected_sum = 0.0;
-    for (int i = 0 ; i < count() ; ++i)
-      if (sections.contains(i))
+    for (int i = 0; i < count(); ++i)
+      if (sections.contains(i)) {
         selected_sum += column_widths_[i];
+      }
   }
 
   if (total_sum != 0.0 && !qFuzzyCompare(total_sum, 1.0)) {
     const ColumnWidthType mult = (selected_sum + (1.0 - total_sum)) / selected_sum;
-    for (int i = 0 ; i < column_widths_.count() ; ++i) {
-      if (sections.isEmpty() || sections.contains(i))
+    for (int i = 0; i < column_widths_.count(); ++i) {
+      if (sections.isEmpty() || sections.contains(i)) {
         column_widths_[i] *= mult;
+      }
     }
   }
 }
 
 void StretchHeaderView::UpdateWidths(const QList<int> &sections) {
 
-  if (!stretch_enabled_)
-    return;
-
-  ColumnWidthType total_w = 0.0;
+  if (!stretch_enabled_) return;
 
   for (int i = 0; i < column_widths_.count(); ++i) {
     const ColumnWidthType w = column_widths_[i];
-    int pixels = w * width();
 
-    total_w += w;
+    int pixels = static_cast<int>(w * width());
 
-    if (!sections.isEmpty() && !sections.contains(i))
+    if (!sections.isEmpty() && !sections.contains(i)) {
       continue;
+    }
 
-    if (pixels == 0 && !isSectionHidden(i))
+    if (pixels == 0 && !isSectionHidden(i)) {
       hideSection(i);
+    }
     else if (pixels != 0 && isSectionHidden(i)) {
       showSection(i);
     }
 
-    if (pixels != 0)
+    if (pixels != 0) {
       resizeSection(i, pixels);
+    }
   }
 
 }
@@ -116,7 +119,7 @@ void StretchHeaderView::HideSection(const int logical) {
 
   // Would this hide the last section?
   bool all_hidden = true;
-  for (int i = 0 ; i < count() ; ++i) {
+  for (int i = 0; i < count(); ++i) {
     if (i != logical && !isSectionHidden(i) && sectionSize(i) > 0) {
       all_hidden = false;
       break;
@@ -146,9 +149,10 @@ void StretchHeaderView::ShowSection(int logical) {
 
   // How many sections are visible already?
   int visible_count = 0;
-  for (int i = 0 ; i < count() ; ++i) {
-    if (!isSectionHidden(i))
-      visible_count ++;
+  for (int i = 0; i < count(); ++i) {
+    if (!isSectionHidden(i)) {
+      ++visible_count;
+    }
   }
 
   column_widths_[logical] = visible_count == 0 ? 1.0 : 1.0 / visible_count;
@@ -198,8 +202,9 @@ void StretchHeaderView::SectionResized(const int logical, const int, const int n
     int visual = visualIndex(logical);
     QList<int> logical_sections_to_resize;
     for (int i = 0; i < count(); ++i) {
-      if (!isSectionHidden(i) && visualIndex(i) > visual)
+      if (!isSectionHidden(i) && visualIndex(i) > visual) {
         logical_sections_to_resize << i;
+      }
     }
 
     // Resize just those columns
@@ -243,9 +248,11 @@ void StretchHeaderView::SetColumnWidth(const int logical, const ColumnWidthType
   column_widths_[logical] = width;
 
   QList<int> other_columns;
-  for (int i = 0 ; i < count() ; ++i)
-    if (!isSectionHidden(i) && i != logical)
+  for (int i = 0; i < count(); ++i) {
+    if (!isSectionHidden(i) && i != logical) {
       other_columns << i;
+    }
+  }
 
   NormaliseWidths(other_columns);
 
@@ -277,7 +284,7 @@ bool StretchHeaderView::RestoreState(const QByteArray &sdata) {
 
   setSortIndicator(sort_indicator_section, Qt::SortOrder(sort_indicator_order));
 
-  const int persisted_column_count = qMin(qMin(visual_indices.count(), pixel_widths.count()), column_widths_.count());
+  const qint64 persisted_column_count = qMin(qMin(visual_indices.count(), pixel_widths.count()), column_widths_.count());
 
   // Set column visible state, visual indices and, if we're not in stretch mode, pixel widths.
   for (int i = 0; i < count() && i < persisted_column_count; ++i) {
@@ -313,6 +320,9 @@ QByteArray StretchHeaderView::SaveState() const {
   QList<int> pixel_widths;
   QList<int> visual_indices;
 
+  pixel_widths.reserve(count());
+  visual_indices.reserve(count());
+
   for (int i = 0; i < count(); ++i) {
     pixel_widths << sectionSize(i);
     visual_indices << logicalIndex(i);
@@ -349,7 +359,11 @@ QByteArray StretchHeaderView::ResetState() {
 
   QList<int> visual_indices;
   QList<int> pixel_widths;
-  for (int i = 0 ; i < count() ; ++i) {
+
+  visual_indices.reserve(count());
+  pixel_widths.reserve(count());
+
+  for (int i = 0; i < count(); ++i) {
     pixel_widths << 10;
     visual_indices << count();
   }
