@@ -44,7 +44,7 @@ class TrackSlider : public QWidget {
   explicit TrackSlider(QWidget *parent = nullptr);
   ~TrackSlider() override;
 
-  void SetApplication(Application* app);
+  void SetApplication(Application *app);
 
   // QWidget
   QSize sizeHint() const override;
@@ -80,7 +80,7 @@ class TrackSlider : public QWidget {
  private:
   void UpdateTimes(const int elapsed);
   void UpdateLabelWidth();
-  void UpdateLabelWidth(QLabel *label, const QString &text);
+  static void UpdateLabelWidth(QLabel *label, const QString &text);
 
  private:
   Ui_TrackSlider *ui_;
@@ -91,7 +91,7 @@ class TrackSlider : public QWidget {
 
   bool setting_value_;
   bool show_remaining_time_;
-  int slider_maximum_value_; //we cache it to avoid unnecessary updates
+  int slider_maximum_value_;  // We cache it to avoid unnecessary updates
 };
 
 #endif  // TRACKSLIDER_H
