@@ -39,19 +39,24 @@
 #include "collection/collectionbackend.h"
 #include "collection/collection.h"
 
+// clazy:excludeall=non-pod-global-static,returning-void-expression
+
 namespace {
 
 class CollectionModelTest : public ::testing::Test {
+ public:
+  CollectionModelTest() : added_dir_(false) {}
+
  protected:
   void SetUp() override {
-    database_.reset(new MemoryDatabase(nullptr));
-    backend_.reset(new CollectionBackend);
-    backend_->Init(database_.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kDirsTable, SCollection::kSubdirsTable, SCollection::kFtsTable);
-    model_.reset(new CollectionModel(backend_.get(), nullptr));
+    database_ = std::make_shared<MemoryDatabase>(nullptr);
+    backend_ = std::make_unique<CollectionBackend>();
+    backend_->Init(database_.get(), nullptr, Song::Source_Collection, SCollection::kSongsTable, SCollection::kFtsTable, SCollection::kDirsTable, SCollection::kSubdirsTable);
+    model_ = std::make_unique<CollectionModel>(backend_.get(), nullptr);
 
     added_dir_ = false;
 
-    model_sorted_.reset(new QSortFilterProxyModel);
+    model_sorted_ =  std::make_unique<QSortFilterProxyModel>();
     model_sorted_->setSourceModel(model_.get());
     model_sorted_->setSortRole(CollectionModel::Role_SortText);
     model_sorted_->setDynamicSortFilter(true);
@@ -83,12 +88,12 @@ class CollectionModelTest : public ::testing::Test {
     return AddSong(song);
   }
 
-  std::shared_ptr<Database> database_;
-  std::unique_ptr<CollectionBackend> backend_;
-  std::unique_ptr<CollectionModel> model_;
-  std::unique_ptr<QSortFilterProxyModel> model_sorted_;
+  std::shared_ptr<Database> database_;  // NOLINT(cppcoreguidelines-non-private-member-variables-in-classes)
+  std::unique_ptr<CollectionBackend> backend_;  // NOLINT(cppcoreguidelines-non-private-member-variables-in-classes)
+  std::unique_ptr<CollectionModel> model_;  // NOLINT(cppcoreguidelines-non-private-member-variables-in-classes)
+  std::unique_ptr<QSortFilterProxyModel> model_sorted_;  // NOLINT(cppcoreguidelines-non-private-member-variables-in-classes)
 
-  bool added_dir_;
+  bool added_dir_;  // NOLINT(cppcoreguidelines-non-private-member-variables-in-classes)
 };
 
 TEST_F(CollectionModelTest, Initialization) {
@@ -209,7 +214,7 @@ TEST_F(CollectionModelTest, VariousArtistSongs) {
     song.Init("Title " + n, "Artist " + n, "Album", 0);
     song.set_mtime(0);
     song.set_ctime(0);
-    songs << song;
+    songs << song;  // clazy:exclude=reserve-candidates
   }
 
   // Different ways of putting songs in "Various Artist".  Make sure they all work
@@ -218,7 +223,7 @@ TEST_F(CollectionModelTest, VariousArtistSongs) {
   songs[2].set_compilation_on(true);
   songs[3].set_compilation_detected(true); songs[3].set_artist("Various Artists");
 
-  for (int i=0 ; i<4 ; ++i)
+  for (int i=0 ; i < 4 ; ++i)
     AddSong(songs[i]);
   model_->Init(false);
 
@@ -253,9 +258,9 @@ TEST_F(CollectionModelTest, RemoveSongsLazyLoaded) {
   ASSERT_EQ(3, model_->rowCount(album_index));
 
   // Remove the first two songs
-  QSignalSpy spy_preremove(model_.get(), SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)));
-  QSignalSpy spy_remove(model_.get(), SIGNAL(rowsRemoved(QModelIndex,int,int)));
-  QSignalSpy spy_reset(model_.get(), SIGNAL(modelReset()));
+  QSignalSpy spy_preremove(model_.get(), &CollectionModel::rowsAboutToBeRemoved);
+  QSignalSpy spy_remove(model_.get(), &CollectionModel::rowsRemoved);
+  QSignalSpy spy_reset(model_.get(), &CollectionModel::modelReset);
 
   backend_->DeleteSongs(SongList() << one << two);
 
@@ -278,9 +283,9 @@ TEST_F(CollectionModelTest, RemoveSongsNotLazyLoaded) {
   model_->Init(false);
 
   // Remove the first two songs
-  QSignalSpy spy_preremove(model_.get(), SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)));
-  QSignalSpy spy_remove(model_.get(), SIGNAL(rowsRemoved(QModelIndex,int,int)));
-  QSignalSpy spy_reset(model_.get(), SIGNAL(modelReset()));
+  QSignalSpy spy_preremove(model_.get(), &CollectionModel::rowsAboutToBeRemoved);
+  QSignalSpy spy_remove(model_.get(), &CollectionModel::rowsRemoved);
+  QSignalSpy spy_reset(model_.get(), &CollectionModel::modelReset);
 
   backend_->DeleteSongs(SongList() << one << two);
 
@@ -517,6 +522,26 @@ TEST_F(CollectionModelTest, TestContainerNodes) {
     songs << song;
   }
 
+  // Albums with Album ID.
+  for (int album_id = 0; album_id <= 2 ; ++album_id) {
+    Song song(Song::Source_Collection);
+    song.set_url(QUrl(QString("file:///tmp/song-with-album-id-1")));
+    song.set_artist("Artist with Album ID");
+    song.set_album(QString("Album %1 with Album ID").arg(album_id));
+    song.set_album_id(QString("Album ID %1").arg(album_id));
+    song.set_mtime(1);
+    song.set_ctime(1);
+    song.set_directory_id(1);
+    song.set_filetype(Song::FileType_FLAC);
+    song.set_filesize(1);
+    song.set_year(1970);
+    for (int i = 0; i <= 3 ; ++i) {
+      song.set_title(QString("Title %1 %2").arg(album_id).arg(i));
+      song.set_track(i);
+      songs << song;
+    }
+  }
+
   for (int f = CollectionModel::GroupBy_None + 1 ; f < CollectionModel::GroupByCount ; ++f) {
     for (int s = CollectionModel::GroupBy_None ; s < CollectionModel::GroupByCount ; ++s) {
       for (int t = CollectionModel::GroupBy_None ; t < CollectionModel::GroupByCount ; ++t) {
@@ -533,18 +558,18 @@ TEST_F(CollectionModelTest, TestContainerNodes) {
         std::unique_ptr<CollectionModel> model2;
         std::unique_ptr<CollectionModel> model3;
 
-        database1.reset(new MemoryDatabase(nullptr));
-        database2.reset(new MemoryDatabase(nullptr));
-        database3.reset(new MemoryDatabase(nullptr));
-        backend1.reset(new CollectionBackend);
-        backend2.reset(new CollectionBackend);
-        backend3.reset(new CollectionBackend);
-        backend1->Init(database1.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kDirsTable, SCollection::kSubdirsTable, SCollection::kFtsTable);
-        backend2->Init(database2.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kDirsTable, SCollection::kSubdirsTable, SCollection::kFtsTable);
-        backend3->Init(database3.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kDirsTable, SCollection::kSubdirsTable, SCollection::kFtsTable);
-        model1.reset(new CollectionModel(backend1.get(), nullptr));
-        model2.reset(new CollectionModel(backend2.get(), nullptr));
-        model3.reset(new CollectionModel(backend3.get(), nullptr));
+        database1 = std::make_unique<MemoryDatabase>(nullptr);
+        database2 = std::make_unique<MemoryDatabase>(nullptr);
+        database3 = std::make_unique<MemoryDatabase>(nullptr);
+        backend1 = std::make_unique<CollectionBackend>();
+        backend2= std::make_unique<CollectionBackend>();
+        backend3 = std::make_unique<CollectionBackend>();
+        backend1->Init(database1.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kFtsTable, SCollection::kDirsTable, SCollection::kSubdirsTable);
+        backend2->Init(database2.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kFtsTable, SCollection::kDirsTable, SCollection::kSubdirsTable);
+        backend3->Init(database3.get(), Song::Source_Collection, SCollection::kSongsTable, SCollection::kFtsTable, SCollection::kDirsTable, SCollection::kSubdirsTable);
+        model1 = std::make_unique<CollectionModel>(backend1.get(), nullptr);
+        model2 = std::make_unique<CollectionModel>(backend2.get(), nullptr);
+        model3 = std::make_unique<CollectionModel>(backend3.get(), nullptr);
 
         backend1->AddDirectory("/mnt/music");
         backend2->AddDirectory("/mnt/music");
@@ -556,9 +581,9 @@ TEST_F(CollectionModelTest, TestContainerNodes) {
 
         model3->set_use_lazy_loading(false);
 
-        QSignalSpy model1_update(model1.get(), SIGNAL(rowsInserted(QModelIndex, int, int)));
-        QSignalSpy model2_update(model2.get(), SIGNAL(rowsInserted(QModelIndex, int, int)));
-        QSignalSpy model3_update(model3.get(), SIGNAL(rowsInserted(QModelIndex, int, int)));
+        QSignalSpy model1_update(model1.get(), &CollectionModel::rowsInserted);
+        QSignalSpy model2_update(model2.get(), &CollectionModel::rowsInserted);
+        QSignalSpy model3_update(model3.get(), &CollectionModel::rowsInserted);
 
         backend1->AddOrUpdateSongs(songs);
         backend2->AddOrUpdateSongs(songs);
@@ -578,7 +603,7 @@ TEST_F(CollectionModelTest, TestContainerNodes) {
         ASSERT_EQ(model2->song_nodes().count(), songs.count());
         ASSERT_EQ(model3->song_nodes().count(), songs.count());
 
-        // Container nodes for all models should now be indentical.
+        // Container nodes for all models should now be identical.
         for (int i = 0 ; i < 3 ; ++i) {
           for (CollectionItem *node : model1->container_nodes(i).values()) {
             ASSERT_TRUE(model2->container_nodes(i).keys().contains(node->key));
@@ -626,9 +651,9 @@ TEST_F(CollectionModelTest, TestContainerNodes) {
           }
         }
 
-        QSignalSpy database_reset_1(backend1.get(), SIGNAL(DatabaseReset()));
-        QSignalSpy database_reset_2(backend2.get(), SIGNAL(DatabaseReset()));
-        QSignalSpy database_reset_3(backend3.get(), SIGNAL(DatabaseReset()));
+        QSignalSpy database_reset_1(backend1.get(), &CollectionBackend::DatabaseReset);
+        QSignalSpy database_reset_2(backend2.get(), &CollectionBackend::DatabaseReset);
+        QSignalSpy database_reset_3(backend3.get(), &CollectionBackend::DatabaseReset);
 
         backend1->DeleteAll();
         backend2->DeleteAll();
